@@ -945,6 +945,7 @@ class IDLParser(object):
     """PrimitiveType : UnsignedIntegerType
                      | UnrestrictedFloatType
                      | StringType
+                     | BIGINT
                      | BOOLEAN
                      | BYTE
                      | OCTET
@@ -1053,6 +1054,7 @@ class IDLParser(object):
     """ExtendedAttribute : ExtendedAttributeNoArgs
                          | ExtendedAttributeArgList
                          | ExtendedAttributeIdent
+                         | ExtendedAttributeWildcard
                          | ExtendedAttributeIdentList
                          | ExtendedAttributeNamedArgList
                          | ExtendedAttributeStringLiteral
@@ -1089,6 +1091,11 @@ class IDLParser(object):
     value = self.BuildAttribute('VALUE', p[3])
     p[0] = self.BuildNamed('ExtAttribute', p, 1, value)
 
+  def p_ExtendedAttributeWildcard(self, p):
+    """ExtendedAttributeWildcard : identifier '=' '*'"""
+    value = self.BuildAttribute('VALUE', p[3])
+    p[0] = self.BuildNamed('ExtAttribute', p, 1, value)
+
   def p_ExtendedAttributeIdentList(self, p):
     """ExtendedAttributeIdentList : identifier '=' '(' IdentifierList ')'"""
     value = self.BuildAttribute('VALUE', p[4])
@@ -1309,7 +1316,7 @@ class IDLParser(object):
 
 def ParseFile(parser, filename):
   """Parse a file and return a File type of node."""
-  with open(filename) as fileobject:
+  with open(filename, encoding='utf-8') as fileobject:
     try:
       out = parser.ParseText(filename, fileobject.read())
       out.SetProperty('ERRORS', parser.GetErrors())
