@@ -8,23 +8,14 @@ See http://dev.chromium.org/developers/how-tos/depottools/presubmit-scripts
 for more details about the presubmit API built into depot_tools.
 """
 
-USE_PYTHON3 = True
 ALLOWEDLIST = [r'.+_test.py$']
 
 
 def CheckChangeOnUpload(input_api, output_api):
   return input_api.canned_checks.RunUnitTestsInDirectory(
-      input_api,
-      output_api,
-      '.',
-      files_to_check=ALLOWEDLIST,
-      run_on_python2=False)
+      input_api, output_api, '.', files_to_check=ALLOWEDLIST)
 
 
 def CheckChangeOnCommit(input_api, output_api):
   return input_api.canned_checks.RunUnitTestsInDirectory(
-      input_api,
-      output_api,
-      '.',
-      files_to_check=ALLOWEDLIST,
-      run_on_python2=False)
+      input_api, output_api, '.', files_to_check=ALLOWEDLIST)
