@@ -21,7 +21,8 @@ using testing::UnorderedElementsAre;
 
 namespace ui {
 
-using BasicAXTreeSerializer = AXTreeSerializer<const AXNode*>;
+using BasicAXTreeSerializer =
+    AXTreeSerializer<const AXNode*, std::vector<const AXNode*>>;
 
 // The framework for these tests is that each test sets up |treedata0_|
 // and |treedata1_| and then calls GetTreeSerializer, which creates a
@@ -198,7 +199,7 @@ TEST_F(AXTreeSerializerTest, ReparentingUpdatesSubtree) {
 // Similar to ReparentingUpdatesSubtree, except that InvalidateSubtree is
 // called on id=1 - we need to make sure that the reparenting is still
 // detected.
-TEST_F(AXTreeSerializerTest, ReparentingWithInvalidationUpdatesSubtree) {
+TEST_F(AXTreeSerializerTest, ReparentingWithDirtySubtreeUpdates) {
   // (1 (2 (3 (4 (5)))))
   treedata0_.root_id = 1;
   treedata0_.nodes.resize(5);
@@ -229,7 +230,7 @@ TEST_F(AXTreeSerializerTest, ReparentingWithInvalidationUpdatesSubtree) {
 
   CreateTreeSerializer();
   AXTreeUpdate update;
-  serializer_->InvalidateSubtree(tree1_->GetFromId(1));
+  serializer_->MarkSubtreeDirty(tree1_->GetFromId(1));
   ASSERT_TRUE(serializer_->SerializeChanges(tree1_->GetFromId(4), &update));
 
   // The update should unserialize without errors.
@@ -237,8 +238,8 @@ TEST_F(AXTreeSerializerTest, ReparentingWithInvalidationUpdatesSubtree) {
   EXPECT_TRUE(dst_tree.Unserialize(update)) << dst_tree.error();
 }
 
-// A variant of AXTreeSource that returns true for IsValid() for one
-// particular id.
+// A variant of AXTreeSource that does not serialize one particular id,
+// returning nullptr from methods that try to retrieve it.
 class AXTreeSourceWithInvalidId : public AXTreeSource<const AXNode*> {
  public:
   AXTreeSourceWithInvalidId(AXTree* tree, int invalid_id)
@@ -257,22 +258,26 @@ class AXTreeSourceWithInvalidId : public AXTreeSource<const AXNode*> {
     return true;
   }
   AXNode* GetRoot() const override { return tree_->root(); }
-  AXNode* GetFromId(AXNodeID id) const override { return tree_->GetFromId(id); }
-  AXNodeID GetId(const AXNode* node) const override { return node->id(); }
-  void GetChildren(const AXNode* node,
-                   std::vector<const AXNode*>* out_children) const override {
-    *out_children = std::vector<const AXNode*>(node->children().cbegin(),
-                                               node->children().cend());
+  AXNode* GetFromId(AXNodeID id) const override {
+    return id == invalid_id_ ? nullptr : tree_->GetFromId(id);
   }
+  AXNodeID GetId(const AXNode* node) const override { return node->id(); }
+  void CacheChildrenIfNeeded(const AXNode*) override {}
+  size_t GetChildCount(const AXNode* node) const override {
+    return node->children().size();
+  }
+  AXNode* ChildAt(const AXNode* node, size_t index) const override {
+    AXNode* result = node->children()[index];
+    return result->id() == invalid_id_ ? nullptr : result;
+  }
+  void ClearChildCache(const AXNode*) override {}
+
   AXNode* GetParent(const AXNode* node) const override {
     return node->parent();
   }
   bool IsIgnored(const AXNode* node) const override {
     return node->IsIgnored();
   }
-  bool IsValid(const AXNode* node) const override {
-    return node != nullptr && node->id() != invalid_id_;
-  }
   bool IsEqual(const AXNode* node1, const AXNode* node2) const override {
     return node1 == node2;
   }
@@ -288,7 +293,8 @@ class AXTreeSourceWithInvalidId : public AXTreeSource<const AXNode*> {
   int invalid_id_;
 };
 
-// Test that the serializer skips invalid children.
+// Test that the serializer DCHECKs when it finds invalid children, or skips
+// them when DCHECKs are off.
 TEST(AXTreeSerializerInvalidTest, InvalidChild) {
   // (1 (2 3))
   AXTreeUpdate treedata;
@@ -305,8 +311,14 @@ TEST(AXTreeSerializerInvalidTest, InvalidChild) {
 
   BasicAXTreeSerializer serializer(&source);
   AXTreeUpdate update;
+  // TODO(crbug.com/1432184, crbug.com/1432126, crbug.com/1431535,
+  // crbug.com/1418319): Once the DCHECKs in BlinkAXTreeSource::ChildAt()
+  // are resolved, and CHECKs for ChildAt() return values are restored in
+  // AXTreeSerializer, turn this into a death expectation.
+  // EXPECT_DEATH_IF_SUPPORTED(serializer.SerializeChanges(tree.root(),
+  // &update),
+  //                           "");
   ASSERT_TRUE(serializer.SerializeChanges(tree.root(), &update));
-
   ASSERT_EQ(2U, update.nodes.size());
   EXPECT_EQ(1, update.nodes[0].id);
   EXPECT_EQ(2, update.nodes[1].id);
@@ -466,159 +478,6 @@ TEST_F(AXTreeSerializerTest, ResetWorksWithNewRootId) {
   EXPECT_TRUE(dst_tree.Unserialize(update)) << dst_tree.error();
 }
 
-// Wraps an AXTreeSource and provides access to the results of the
-// SerializerClearedNode callback.
-class AXTreeSourceTestWrapper : public AXTreeSource<const AXNode*> {
- public:
-  explicit AXTreeSourceTestWrapper(AXTreeSource<const AXNode*>* tree_source)
-      : tree_source_(tree_source) {}
-  ~AXTreeSourceTestWrapper() override = default;
-
-  // Override SerializerClearedNode and provide a way to access it.
-  void SerializerClearedNode(AXNodeID node_id) override {
-    cleared_node_ids_.insert(node_id);
-  }
-
-  void ClearClearedNodeIds() { cleared_node_ids_.clear(); }
-  std::set<AXNodeID>& cleared_node_ids() { return cleared_node_ids_; }
-
-  // The rest of the AXTreeSource implementation just calls through to
-  // tree_source_.
-  bool GetTreeData(AXTreeData* data) const override {
-    return tree_source_->GetTreeData(data);
-  }
-  const AXNode* GetRoot() const override { return tree_source_->GetRoot(); }
-  const AXNode* GetFromId(AXNodeID id) const override {
-    return tree_source_->GetFromId(id);
-  }
-  AXNodeID GetId(const AXNode* node) const override {
-    return tree_source_->GetId(node);
-  }
-  void GetChildren(const AXNode* node,
-                   std::vector<const AXNode*>* out_children) const override {
-    return tree_source_->GetChildren(node, out_children);
-  }
-  const AXNode* GetParent(const AXNode* node) const override {
-    return tree_source_->GetParent(node);
-  }
-  bool IsIgnored(const AXNode* node) const override {
-    return tree_source_->IsIgnored(node);
-  }
-  bool IsValid(const AXNode* node) const override {
-    return tree_source_->IsValid(node);
-  }
-  bool IsEqual(const AXNode* node1, const AXNode* node2) const override {
-    return tree_source_->IsEqual(node1, node2);
-  }
-  const AXNode* GetNull() const override { return tree_source_->GetNull(); }
-  void SerializeNode(const AXNode* node, AXNodeData* out_data) const override {
-    tree_source_->SerializeNode(node, out_data);
-  }
-
- private:
-  raw_ptr<AXTreeSource<const AXNode*>> tree_source_;
-  std::set<AXNodeID> cleared_node_ids_;
-};
-
-TEST_F(AXTreeSerializerTest, TestClearedNodesWhenUpdatingRoot) {
-  // (1 (2 (3 (4))))
-  treedata0_.root_id = 1;
-  treedata0_.nodes.resize(4);
-  treedata0_.nodes[0].id = 1;
-  treedata0_.nodes[0].child_ids.push_back(2);
-  treedata0_.nodes[1].id = 2;
-  treedata0_.nodes[1].child_ids.push_back(3);
-  treedata0_.nodes[2].id = 3;
-  treedata0_.nodes[2].child_ids.push_back(4);
-  treedata0_.nodes[3].id = 4;
-
-  // (5 (2 (3 (4))))
-  treedata1_.root_id = 5;
-  treedata1_.nodes.resize(4);
-  treedata1_.nodes[0].id = 5;
-  treedata1_.nodes[0].child_ids.push_back(2);
-  treedata1_.nodes[1].id = 2;
-  treedata1_.nodes[1].child_ids.push_back(3);
-  treedata1_.nodes[2].id = 3;
-  treedata1_.nodes[2].child_ids.push_back(4);
-  treedata1_.nodes[3].id = 4;
-
-  // Similar sequence to CreateTreeSerializer, but using
-  // AXTreeSourceTestWrapper instead.
-  tree0_ = std::make_unique<AXSerializableTree>(treedata0_);
-  tree1_ = std::make_unique<AXSerializableTree>(treedata1_);
-  tree0_source_.reset(tree0_->CreateTreeSource());
-  AXTreeSourceTestWrapper tree0_source_wrapper(tree0_source_.get());
-  serializer_ = std::make_unique<BasicAXTreeSerializer>(&tree0_source_wrapper);
-  AXTreeUpdate unused_update;
-  ASSERT_TRUE(serializer_->SerializeChanges(tree0_->root(), &unused_update));
-  tree1_source_.reset(tree1_->CreateTreeSource());
-  AXTreeSourceTestWrapper tree1_source_wrapper(tree1_source_.get());
-  serializer_->ChangeTreeSourceForTesting(&tree1_source_wrapper);
-  ASSERT_EQ(4U, serializer_->ClientTreeNodeCount());
-
-  // If we swap out the root, all of the node IDs should have
-  // SerializerClearedNode called on them.
-  tree1_source_wrapper.ClearClearedNodeIds();
-  ASSERT_TRUE(serializer_->SerializeChanges(tree1_->root(), &unused_update));
-  EXPECT_THAT(tree1_source_wrapper.cleared_node_ids(),
-              UnorderedElementsAre(1, 2, 3, 4));
-
-  // Destroy the serializer first so that the AXTreeSources it points to
-  // don't go out of scope first.
-  serializer_.reset();
-}
-
-TEST_F(AXTreeSerializerTest, TestClearedNodesWhenUpdatingBranch) {
-  // (1 (2 (3 (4))))
-  treedata0_.root_id = 1;
-  treedata0_.nodes.resize(4);
-  treedata0_.nodes[0].id = 1;
-  treedata0_.nodes[0].child_ids.push_back(2);
-  treedata0_.nodes[1].id = 2;
-  treedata0_.nodes[1].child_ids.push_back(3);
-  treedata0_.nodes[2].id = 3;
-  treedata0_.nodes[2].child_ids.push_back(4);
-  treedata0_.nodes[3].id = 4;
-
-  // (1 (2 (5 (6))))
-  treedata1_.root_id = 1;
-  treedata1_.nodes.resize(4);
-  treedata1_.nodes[0].id = 1;
-  treedata1_.nodes[0].child_ids.push_back(2);
-  treedata1_.nodes[1].id = 2;
-  treedata1_.nodes[1].child_ids.push_back(5);
-  treedata1_.nodes[2].id = 5;
-  treedata1_.nodes[2].child_ids.push_back(6);
-  treedata1_.nodes[3].id = 6;
-
-  // Similar sequence to CreateTreeSerializer, but using
-  // AXTreeSourceTestWrapper instead.
-  tree0_ = std::make_unique<AXSerializableTree>(treedata0_);
-  tree1_ = std::make_unique<AXSerializableTree>(treedata1_);
-  tree0_source_.reset(tree0_->CreateTreeSource());
-  AXTreeSourceTestWrapper tree0_source_wrapper(tree0_source_.get());
-  serializer_ = std::make_unique<BasicAXTreeSerializer>(&tree0_source_wrapper);
-  AXTreeUpdate unused_update;
-  ASSERT_TRUE(serializer_->SerializeChanges(tree0_->root(), &unused_update));
-  tree1_source_.reset(tree1_->CreateTreeSource());
-  AXTreeSourceTestWrapper tree1_source_wrapper(tree1_source_.get());
-  serializer_->ChangeTreeSourceForTesting(&tree1_source_wrapper);
-  ASSERT_EQ(4U, serializer_->ClientTreeNodeCount());
-
-  // If we replace one branch with another, we should get calls to
-  // SerializerClearedNode with all of the node IDs no longer in the tree.
-  tree1_source_wrapper.ClearClearedNodeIds();
-  ASSERT_TRUE(
-      serializer_->SerializeChanges(tree1_->GetFromId(2), &unused_update));
-  EXPECT_THAT(tree1_source_wrapper.cleared_node_ids(),
-              UnorderedElementsAre(3, 4));
-
-  // Destroy the serializer first so that the AXTreeSources it points to
-  // don't go out of scope first.
-  serializer_.reset();
-}
-
 TEST_F(AXTreeSerializerTest, TestPartialSerialization) {
   // Serialize only part of the tree.
 
@@ -677,7 +536,7 @@ TEST_F(AXTreeSerializerTest, TestPartialSerialization) {
     // The result should be indistinguishable from the source tree.
     std::unique_ptr<AXTreeSource<const AXNode*>> dst_tree_source(
         dst_tree.CreateTreeSource());
-    AXTreeSerializer<const AXNode*> serializer(dst_tree_source.get());
+    BasicAXTreeSerializer serializer(dst_tree_source.get());
     AXTreeUpdate dst_update;
     CHECK(serializer.SerializeChanges(dst_tree.root(), &dst_update));
     ASSERT_EQ(treedata1_.ToString(), dst_update.ToString());
