@@ -11,10 +11,11 @@
 #include <string>
 #include <utility>
 
+#include "base/component_export.h"
+#include "base/memory/raw_ptr_exclusion.h"
 #include "base/strings/utf_offset_string_conversions.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 #include "ui/accessibility/ax_enums.mojom-forward.h"
-#include "ui/accessibility/ax_export.h"
 #include "ui/accessibility/platform/ax_platform_node_base.h"
 
 // This deleter is used in order to ensure that we properly always free memory
@@ -40,7 +41,9 @@ using AtkAttributes = std::unique_ptr<AtkAttributeSet, AtkAttributeSetDeleter>;
 namespace ui {
 
 struct FindInPageResultInfo {
-  AtkObject* node;
+  // This field is not a raw_ptr<> because it was filtered by the rewriter for:
+  // #union
+  RAW_PTR_EXCLUSION AtkObject* node;
   int start_offset;
   int end_offset;
 
@@ -62,7 +65,7 @@ struct FindInPageResultInfo {
 // AtkTableCellIface below are overridden by the runtime version.
 // TODO(accessibility) Remove AtkTableCellInterface when 2.12 is the minimum
 // supported version.
-struct AX_EXPORT AtkTableCellInterface {
+struct COMPONENT_EXPORT(AX_PLATFORM) AtkTableCellInterface {
   typedef struct _AtkTableCell AtkTableCell;
   static GType GetType();
   static GPtrArray* GetColumnHeaderCells(AtkTableCell* cell);
@@ -110,7 +113,8 @@ class ImplementedAtkInterfaces {
 };
 
 // Implements accessibility on Aura Linux using ATK.
-class AX_EXPORT AXPlatformNodeAuraLinux : public AXPlatformNodeBase {
+class COMPONENT_EXPORT(AX_PLATFORM) AXPlatformNodeAuraLinux
+    : public AXPlatformNodeBase {
  public:
   ~AXPlatformNodeAuraLinux() override;
   AXPlatformNodeAuraLinux(const AXPlatformNodeAuraLinux&) = delete;
@@ -193,7 +197,6 @@ class AX_EXPORT AXPlatformNodeAuraLinux : public AXPlatformNodeBase {
   void GetFloatAttributeInGValue(ax::mojom::FloatAttribute attr, GValue* value);
 
   // Event helpers
-  void OnActiveDescendantChanged();
   void OnBusyStateChanged(bool is_busy);
   void OnCheckedStateChanged();
   void OnEnabledChanged();
@@ -228,7 +231,6 @@ class AX_EXPORT AXPlatformNodeAuraLinux : public AXPlatformNodeBase {
   void ResendFocusSignalsForCurrentlyFocusedNode();
   void SetAsCurrentlyFocusedNode();
   bool SupportsSelectionWithAtkSelection();
-  bool SelectionAndFocusAreTheSame();
   void SetActiveViewsDialog();
 
   // AXPlatformNode overrides.
@@ -398,11 +400,14 @@ class AX_EXPORT AXPlatformNodeAuraLinux : public AXPlatformNodeBase {
   ImplementedAtkInterfaces interface_mask_;
 
   // We own a reference to these ref-counted objects.
-  AtkObject* atk_object_ = nullptr;
-  AtkHyperlink* atk_hyperlink_ = nullptr;
+  // These fields are not a raw_ptr<> because of in-out-arg usage.
+  RAW_PTR_EXCLUSION AtkObject* atk_object_ = nullptr;
+  RAW_PTR_EXCLUSION AtkHyperlink* atk_hyperlink_ = nullptr;
 
   // A weak pointers which help us track the ATK embeds relation.
-  AtkObject* document_parent_ = nullptr;
+  // This field is not a raw_ptr<> because it was filtered by the rewriter for:
+  // #addr-of
+  RAW_PTR_EXCLUSION AtkObject* document_parent_ = nullptr;
 
   // Whether or not this node (if it is a frame or a window) was
   // minimized the last time it's visibility changed.
