@@ -16,17 +16,18 @@
 #include <string>
 #include <vector>
 
+#include "base/component_export.h"
 #include "base/gtest_prod_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/observer_list.h"
 #include "base/win/atl.h"
 #include "third_party/iaccessible2/ia2_api_all.h"
 #include "ui/accessibility/ax_enums.mojom-forward.h"
-#include "ui/accessibility/ax_export.h"
 #include "ui/accessibility/ax_text_utils.h"
 #include "ui/accessibility/platform/ax_platform_node_base.h"
 #include "ui/accessibility/platform/ax_platform_text_boundary.h"
 #include "ui/accessibility/platform/ichromeaccessible.h"
+#include "ui/accessibility/platform/sequence_affine_com_object_root_win.h"
 #include "ui/gfx/range/range.h"
 
 // This nonstandard GUID is taken directly from the Mozilla sources
@@ -326,6 +327,11 @@ enum {
     return E_INVALIDARG;                  \
   *arg = {};
 
+// A helper for tracing calls for functions implementing accessibility COM
+// interfaces.
+#define WIN_ACCESSIBILITY_API_TRACE_EVENT(function) \
+  TRACE_EVENT("accessibility", function, perfetto::Flow::FromPointer(this))
+
 namespace base {
 namespace win {
 class VariantVector;
@@ -340,11 +346,13 @@ class AXPlatformRelationWin;
 // A simple interface for a class that wants to be notified when Windows
 // accessibility APIs are used by a client, a strong indication that full
 // accessibility support should be enabled.
-class AX_EXPORT WinAccessibilityAPIUsageObserver {
+class COMPONENT_EXPORT(AX_PLATFORM) WinAccessibilityAPIUsageObserver {
  public:
   WinAccessibilityAPIUsageObserver();
   virtual ~WinAccessibilityAPIUsageObserver();
-  virtual void OnIAccessible2Used() = 0;
+  virtual void OnMSAAUsed() = 0;
+  virtual void OnBasicIAccessible2Used() = 0;
+  virtual void OnAdvancedIAccessible2Used() = 0;
   virtual void OnScreenReaderHoneyPotQueried() = 0;
   virtual void OnAccNameCalled() = 0;
   virtual void OnBasicUIAutomationUsed() = 0;
@@ -358,50 +366,51 @@ class AX_EXPORT WinAccessibilityAPIUsageObserver {
 
 // Get an observer list that allows modules across the codebase to
 // listen to when usage of Windows accessibility APIs is detected.
-extern AX_EXPORT
-    base::ObserverList<WinAccessibilityAPIUsageObserver>::Unchecked&
-    GetWinAccessibilityAPIUsageObserverList();
+extern COMPONENT_EXPORT(
+    AX_PLATFORM) base::ObserverList<WinAccessibilityAPIUsageObserver>::
+    Unchecked& GetWinAccessibilityAPIUsageObserverList();
 
 // Used to simplify calling StartFiringUIAEvents and EndFiringEvents
-class AX_EXPORT WinAccessibilityAPIUsageScopedUIAEventsNotifier {
+class COMPONENT_EXPORT(AX_PLATFORM)
+    WinAccessibilityAPIUsageScopedUIAEventsNotifier {
  public:
   WinAccessibilityAPIUsageScopedUIAEventsNotifier();
   ~WinAccessibilityAPIUsageScopedUIAEventsNotifier();
 };
 
-// TODO(nektar): Remove multithread superclass since we don't support it.
-class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
-    AXPlatformNodeWin : public CComObjectRootEx<CComMultiThreadModel>,
-                        public IDispatchImpl<IAccessible2_4,
-                                             &IID_IAccessible2_4,
-                                             &LIBID_IAccessible2Lib>,
-                        public IAccessibleEx,
-                        public IAccessibleHypertext,
-                        public IAccessibleTable,
-                        public IAccessibleTable2,
-                        public IAccessibleTableCell,
-                        public IAccessibleTextSelectionContainer,
-                        public IAccessibleValue,
-                        public IAnnotationProvider,
-                        public IExpandCollapseProvider,
-                        public IGridItemProvider,
-                        public IGridProvider,
-                        public IInvokeProvider,
-                        public IRangeValueProvider,
-                        public IRawElementProviderFragment,
-                        public IRawElementProviderSimple2,
-                        public IScrollItemProvider,
-                        public IScrollProvider,
-                        public ISelectionItemProvider,
-                        public ISelectionProvider,
-                        public IServiceProvider,
-                        public ITableItemProvider,
-                        public ITableProvider,
-                        public IToggleProvider,
-                        public IValueProvider,
-                        public IWindowProvider,
-                        public IChromeAccessible,
-                        public AXPlatformNodeBase {
+class COMPONENT_EXPORT(AX_PLATFORM) __declspec(
+    uuid("26f5641a-246d-457b-a96d-07f3fae6acf2")) AXPlatformNodeWin
+    : public SequenceAffineComObjectRoot,
+      public IDispatchImpl<IAccessible2_4,
+                           &IID_IAccessible2_4,
+                           &LIBID_IAccessible2Lib>,
+      public IAccessibleEx,
+      public IAccessibleHypertext,
+      public IAccessibleTable,
+      public IAccessibleTable2,
+      public IAccessibleTableCell,
+      public IAccessibleTextSelectionContainer,
+      public IAccessibleValue,
+      public IAnnotationProvider,
+      public IExpandCollapseProvider,
+      public IGridItemProvider,
+      public IGridProvider,
+      public IInvokeProvider,
+      public IRangeValueProvider,
+      public IRawElementProviderFragment,
+      public IRawElementProviderSimple2,
+      public IScrollItemProvider,
+      public IScrollProvider,
+      public ISelectionItemProvider,
+      public ISelectionProvider,
+      public IServiceProvider,
+      public ITableItemProvider,
+      public ITableProvider,
+      public IToggleProvider,
+      public IValueProvider,
+      public IWindowProvider,
+      public IChromeAccessible,
+      public AXPlatformNodeBase {
   using IDispatchImpl::Invoke;
 
  public:
@@ -460,6 +469,19 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   void Destroy() override;
   bool IsPlatformCheckable() const override;
 
+  // CComObjectRootEx (non-virtual) overrides.
+  ULONG InternalAddRef();
+  ULONG InternalRelease();
+
+  // Invoked when the instance's refcount rises above 1. This generally means
+  // that a reference to an interface pointer is being handed out to an
+  // accessibility consumer.
+  virtual void OnReferenced();
+
+  // Invoked when the instance's refcount drops to 1. This generally means that
+  // an accessibility consumer has released its last reference to the instance.
+  virtual void OnDereferenced();
+
   //
   // IAccessible methods.
   //
@@ -1161,6 +1183,9 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   // depth-first pre-order traversal.
   AXPlatformNodeWin* GetFirstTextOnlyDescendant();
 
+  // Clear the computed hypertext.
+  void ResetComputedHypertext();
+
   // Convert a mojo event to an MSAA event. Exposed for testing.
   static absl::optional<DWORD> MojoEventToMSAAEvent(ax::mojom::Event event);
 
@@ -1216,8 +1241,6 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   // Relationships between this node and other nodes.
   std::vector<Microsoft::WRL::ComPtr<AXPlatformRelationWin>> relations_;
 
-  AXLegacyHypertext old_hypertext_;
-
   // These protected methods are still used by BrowserAccessibilityComWin. At
   // some point post conversion, we can probably move these to be private
   // methods.
@@ -1229,9 +1252,6 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   // Also, in IA2, text that includes embedded objects is called hypertext.
   // Returns true if the current object is an IA2 hyperlink.
   bool IsHyperlink();
-  void ComputeHypertextRemovedAndInserted(size_t* start,
-                                          size_t* old_len,
-                                          size_t* new_len);
 
   // If offset is a member of IA2TextSpecialOffsets this function updates the
   // value of offset and returns, otherwise offset remains unchanged.
@@ -1277,7 +1297,6 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
 
   AXPlatformNodeWin* GetParentPlatformNodeWin() const;
 
-  bool ShouldNodeHaveFocusableState() const;
   int GetAnnotationTypeImpl() const;
 
   // Get the value attribute as a Bstr, this means something different depending
@@ -1360,14 +1379,14 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   // argument. The function will skip over any ids that cannot be resolved as
   // valid relation target.
   std::vector<AXPlatformNodeWin*> CreatePlatformNodeVectorFromRelationIdVector(
-      std::vector<int32_t>& relation_id_list);
+      const std::vector<int32_t>& relation_id_list);
 
   // Create a safearray of automation elements from a vector of
   // AXPlatformNodeWin.
   // The caller should validate that all of the given ax platform nodes are
   // valid relation targets.
   SAFEARRAY* CreateUIAElementsSafeArray(
-      std::vector<AXPlatformNodeWin*>& platform_node_list);
+      const std::vector<AXPlatformNodeWin*>& platform_node_list);
 
   // Return an array that contains the center x, y coordinates of the
   // clickable point.
@@ -1424,6 +1443,9 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
   // Helper method getting the selected status.
   bool ISelectionItemProviderIsSelected() const;
 
+  // Helper method for getting the toggle state.
+  ToggleState GetToggleStateImpl();
+
   // Helper method for IsInaccessibleForUIA.
   bool IsNodeInaccessibleForUIA() const;
 
@@ -1515,6 +1537,13 @@ class AX_EXPORT __declspec(uuid("26f5641a-246d-457b-a96d-07f3fae6acf2"))
       const std::wstring& active_composition_text,
       bool is_composition_committed);
 
+  // Notifies observers that basic MSAA usage was detected. Only some of the
+  // APIs were chosen to avoid accessibility being enabled unnecessarily or
+  // unexpectedly in a test environment, while still ensuring that clients that
+  // only use MSAA/IAccessible have a way to turn on accessibility.
+  void NotifyObserverForMSAAUsage() const;
+
+  void NotifyAddAXModeFlagsForIA2(const uint32_t ax_modes) const;
   void NotifyAPIObserverForPatternRequest(PATTERNID pattern_id) const;
   void NotifyAPIObserverForPropertyRequest(PROPERTYID property_id) const;
 
