@@ -5,7 +5,7 @@
 #ifndef UI_ACCESSIBILITY_PLATFORM_AX_PLATFORM_TREE_MANAGER_H_
 #define UI_ACCESSIBILITY_PLATFORM_AX_PLATFORM_TREE_MANAGER_H_
 
-#include "ui/accessibility/ax_export.h"
+#include "base/component_export.h"
 #include "ui/accessibility/ax_node.h"
 #include "ui/accessibility/ax_tree_id.h"
 #include "ui/accessibility/ax_tree_manager.h"
@@ -15,10 +15,11 @@ namespace ui {
 class AXPlatformNode;
 class AXPlatformNodeDelegate;
 
-// Abstract interface for a class that owns an AXTree and manages its
-// connections to other AXTrees in the same page or desktop (parent and child
-// trees).
-class AX_EXPORT AXPlatformTreeManager : public AXTreeManager {
+// Abstract interface for a class that manages AXPlatformNodes and is
+// able to query for them via `GetPlatformNodeFromTree`. Extends
+// AXTreeManager, so AXNodes are also managed.
+class COMPONENT_EXPORT(AX_PLATFORM) AXPlatformTreeManager
+    : public AXTreeManager {
  public:
   // Returns an AXPlatformNode with the specified and |node_id|.
   virtual AXPlatformNode* GetPlatformNodeFromTree(
@@ -31,10 +32,11 @@ class AX_EXPORT AXPlatformTreeManager : public AXTreeManager {
   // of the accessibility tree.
   virtual AXPlatformNodeDelegate* RootDelegate() const = 0;
 
+  bool IsPlatformTreeManager() const override;
+
  protected:
-  explicit AXPlatformTreeManager(const AXTreeID& tree_id,
-                                 std::unique_ptr<AXTree> tree)
-      : AXTreeManager(tree_id, std::move(tree)) {}
+  explicit AXPlatformTreeManager(std::unique_ptr<AXTree> tree)
+      : AXTreeManager(std::move(tree)) {}
 };
 
 }  // namespace ui
