@@ -5,11 +5,12 @@
 #ifndef UI_ACCESSIBILITY_PLATFORM_INSPECT_AX_INSPECT_UTILS_MAC_H_
 #define UI_ACCESSIBILITY_PLATFORM_INSPECT_AX_INSPECT_UTILS_MAC_H_
 
-#import <Cocoa/Cocoa.h>
+#include <ApplicationServices/ApplicationServices.h>
 
-#include "base/callback_forward.h"
+#include "base/apple/scoped_cftyperef.h"
+#include "base/component_export.h"
+#include "base/functional/callback_forward.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
-#include "ui/accessibility/ax_export.h"
 #include "ui/accessibility/platform/inspect/ax_inspect.h"
 
 using ui::AXTreeSelector;
@@ -18,19 +19,26 @@ namespace ui {
 
 // Returns true if the given accessibility attribute is valid, and could have
 // been exposed on certain accessibility objects.
-AX_EXPORT bool IsValidAXAttribute(const std::string& attribute);
+COMPONENT_EXPORT(AX_PLATFORM)
+bool IsValidAXAttribute(const std::string& attribute);
 
 // Return AXElement in a tree by a given criteria.
 using AXFindCriteria = base::RepeatingCallback<bool(const AXUIElementRef)>;
-AX_EXPORT AXUIElementRef FindAXUIElement(const AXUIElementRef node,
-                                         const AXFindCriteria& criteria);
+COMPONENT_EXPORT(AX_PLATFORM)
+base::apple::ScopedCFTypeRef<AXUIElementRef> FindAXUIElement(
+    const AXUIElementRef node,
+    const AXFindCriteria& criteria);
 
 // Returns AXUIElement and its application process id by a given tree selector.
-AX_EXPORT std::pair<AXUIElementRef, int> FindAXUIElement(const AXTreeSelector&);
+COMPONENT_EXPORT(AX_PLATFORM)
+std::pair<base::apple::ScopedCFTypeRef<AXUIElementRef>, int> FindAXUIElement(
+    const AXTreeSelector&);
 
 // Returns AXUIElement for a window having title matching the given pattern.
-AX_EXPORT AXUIElementRef FindAXWindowChild(AXUIElementRef parent,
-                                           const std::string& pattern);
+COMPONENT_EXPORT(AX_PLATFORM)
+base::apple::ScopedCFTypeRef<AXUIElementRef> FindAXWindowChild(
+    AXUIElementRef parent,
+    const std::string& pattern);
 
 }  // namespace ui
 
