@@ -6,6 +6,7 @@
 #define UI_BASE_IME_ASH_IME_KEYBOARD_IMPL_H_
 
 #include "base/component_export.h"
+#include "base/memory/raw_ptr.h"
 #include "ui/base/ime/ash/ime_keyboard.h"
 
 namespace ui {
@@ -28,16 +29,13 @@ class COMPONENT_EXPORT(UI_BASE_IME_ASH) ImeKeyboardImpl : public ImeKeyboard {
   // ImeKeyboard:
   bool SetCurrentKeyboardLayoutByName(const std::string& layout_name) override;
   bool SetAutoRepeatRate(const AutoRepeatRate& rate) override;
-  bool SetAutoRepeatEnabled(bool enabled) override;
+  void SetAutoRepeatEnabled(bool enabled) override;
   bool GetAutoRepeatEnabled() override;
-  bool ReapplyCurrentKeyboardLayout() override;
-  void ReapplyCurrentModifierLockStatus() override;
-  void DisableNumLock() override;
   void SetCapsLockEnabled(bool enable_caps_lock) override;
-  bool CapsLockIsEnabled() override;
+  bool IsCapsLockEnabled() override;
 
  private:
-  ui::InputController* const input_controller_;
+  const raw_ptr<ui::InputController, ExperimentalAsh> input_controller_;
 };
 
 }  // namespace input_method
