@@ -7,13 +7,13 @@
 #include "services/metrics/public/cpp/ukm_builders.h"
 #include "ui/base/ui_base_features.h"
 
-namespace ui {
+namespace ash {
 
 void RecordUkmNonCompliantApi(
     ukm::SourceId source,
-    const ash::ime::mojom::InputMethodApiOperation operation) {
+    const ime::mojom::InputMethodApiOperation operation) {
   if (source == ukm::kInvalidSourceId ||
-      operation == ash::ime::mojom::InputMethodApiOperation::kUnknown) {
+      operation == ime::mojom::InputMethodApiOperation::kUnknown) {
     return;
   }
 
@@ -33,4 +33,4 @@ void RecordUkmAssistiveMatch(ukm::SourceId source, const int64_t type) {
       ukm::UkmRecorder::Get());
 }
 
-}  // namespace ui
+}  // namespace ash
