@@ -7,9 +7,8 @@
 namespace ash {
 
 MockIMECandidateWindowHandler::MockIMECandidateWindowHandler()
-    : set_cursor_bounds_call_count_(0),
-      update_lookup_table_call_count_(0) {
-}
+    : set_cursor_and_composition_bounds_call_count_(0),
+      update_lookup_table_call_count_(0) {}
 
 MockIMECandidateWindowHandler::~MockIMECandidateWindowHandler() = default;
 
@@ -30,10 +29,10 @@ void MockIMECandidateWindowHandler::UpdatePreeditText(
     uint32_t cursor_pos,
     bool visible) {}
 
-void MockIMECandidateWindowHandler::SetCursorBounds(
+void MockIMECandidateWindowHandler::SetCursorAndCompositionBounds(
     const gfx::Rect& cursor_bounds,
-    const gfx::Rect& composition_head) {
-  ++set_cursor_bounds_call_count_;
+    const gfx::Rect& composition_bounds) {
+  ++set_cursor_and_composition_bounds_call_count_;
 }
 
 gfx::Rect MockIMECandidateWindowHandler::GetCursorBounds() const {
@@ -41,7 +40,7 @@ gfx::Rect MockIMECandidateWindowHandler::GetCursorBounds() const {
 }
 
 void MockIMECandidateWindowHandler::Reset() {
-  set_cursor_bounds_call_count_ = 0;
+  set_cursor_and_composition_bounds_call_count_ = 0;
   update_lookup_table_call_count_ = 0;
 }
 
