@@ -10,8 +10,8 @@
 #include <map>
 #include <set>
 
-#include "base/bind.h"
-#include "base/callback.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/logging.h"
 #include "base/test/mock_callback.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -23,9 +23,9 @@ namespace ui {
 
 namespace {
 
-NSMenu* const kFakeMenu1 = reinterpret_cast<NSMenu*>(1);
-NSMenu* const kFakeMenu2 = reinterpret_cast<NSMenu*>(2);
-NSMenu* const kFakeMenu3 = reinterpret_cast<NSMenu*>(3);
+NSMenu* const kFakeMenu1 = [[NSMenu alloc] initWithTitle:@"1"];
+NSMenu* const kFakeMenu2 = [[NSMenu alloc] initWithTitle:@"2"];
+NSMenu* const kFakeMenu3 = [[NSMenu alloc] initWithTitle:@"3"];
 
 constexpr gfx::Rect kScreenBounds1(30, 50, 100, 200);
 constexpr gfx::Rect kScreenBounds2(60, 70, 120, 220);
@@ -86,7 +86,7 @@ class ExpectedCall {
     ASSERT_TRUE(element->IsA<TrackedElementMac>());
     if (expected_bounds_.has_value()) {
       EXPECT_EQ(expected_bounds_.value(),
-                element->AsA<TrackedElementMac>()->screen_bounds());
+                element->AsA<TrackedElementMac>()->GetScreenBounds());
       expected_bounds_.reset();
     }
     EXPECT_EQ(identifier_, element->identifier());
@@ -391,9 +391,11 @@ TEST_F(ElementTrackerMacTest, NestMenusSendHideEvents) {
   element_tracker_->NotifyMenuItemShown(kFakeMenu2, kElementIdentifier2,
                                         kScreenBounds2);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu2, kElementIdentifier2);
-  EXPECT_EQ(1, hidden2.count());
+  // Note that the "hide" event isn't sent until the menu is destroyed or
+  // another submenu is shown.
   element_tracker_->NotifyMenuItemShown(kFakeMenu2, kElementIdentifier2,
                                         kScreenBounds3);
+  EXPECT_EQ(1, hidden2.count());
   element_tracker_->NotifyMenuItemActivated(kFakeMenu2, kElementIdentifier2);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu2, kElementIdentifier2);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu1, kElementIdentifier1);
@@ -427,9 +429,11 @@ TEST_F(ElementTrackerMacTest, NestMenusSendHideEvents_SecondSubmenuOpened) {
   element_tracker_->NotifyMenuItemShown(kFakeMenu2, kElementIdentifier2,
                                         kScreenBounds2);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu2, kElementIdentifier2);
-  EXPECT_EQ(1, hidden2.count());
+  // Note that the "hide" event isn't sent until the menu is destroyed or
+  // another submenu is shown.
   element_tracker_->NotifyMenuItemShown(kFakeMenu3, kElementIdentifier3,
                                         kScreenBounds3);
+  EXPECT_EQ(1, hidden2.count());
   element_tracker_->NotifyMenuItemActivated(kFakeMenu3, kElementIdentifier3);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu3, kElementIdentifier3);
   element_tracker_->NotifyMenuItemHidden(kFakeMenu1, kElementIdentifier1);
@@ -471,4 +475,49 @@ TEST_F(ElementTrackerMacTest, NestMenusSendHideEvents_DoubleSubmenu) {
   element_tracker_->NotifyMenuDoneShowing(kFakeMenu1);
 }
 
+// The following tests are regression tests for the likely cause of
+// crbug.com/1432480.
+
+TEST_F(ElementTrackerMacTest, TrackerIsRobustToOutOfOrderActivation) {
+  ExpectedCall shown(kElementIdentifier1, kElementContext1,
+                     ExpectedCall::kShown, 1);
+  ExpectedCall hidden(kElementIdentifier1, kElementContext1,
+                      ExpectedCall::kHidden, 1);
+  ExpectedCall activated(kElementIdentifier1, kElementContext1,
+                         ExpectedCall::kActivated, 1);
+
+  element_tracker_->NotifyMenuWillShow(kFakeMenu1, kElementContext1);
+  element_tracker_->NotifyMenuItemShown(kFakeMenu1, kElementIdentifier1,
+                                        kScreenBounds1);
+  element_tracker_->NotifyMenuItemHidden(kFakeMenu1, kElementIdentifier1);
+  element_tracker_->NotifyMenuItemActivated(kFakeMenu1, kElementIdentifier1);
+  element_tracker_->NotifyMenuDoneShowing(kFakeMenu1);
+}
+
+TEST_F(ElementTrackerMacTest,
+       TrackerIsRobustToOutOfOrderActivationInNestedMenu) {
+  ExpectedCall shown(kElementIdentifier1, kElementContext1,
+                     ExpectedCall::kShown, 1);
+  ExpectedCall hidden(kElementIdentifier1, kElementContext1,
+                      ExpectedCall::kHidden, 1);
+  ExpectedCall activated(kElementIdentifier1, kElementContext1,
+                         ExpectedCall::kActivated, 0);
+  ExpectedCall shown2(kElementIdentifier2, kElementContext1,
+                      ExpectedCall::kShown, 1);
+  ExpectedCall hidden2(kElementIdentifier2, kElementContext1,
+                       ExpectedCall::kHidden, 1);
+  ExpectedCall activated2(kElementIdentifier2, kElementContext1,
+                          ExpectedCall::kActivated, 1);
+  element_tracker_->SetParent(kFakeMenu2, kFakeMenu1);
+  element_tracker_->NotifyMenuWillShow(kFakeMenu1, kElementContext1);
+  element_tracker_->NotifyMenuItemShown(kFakeMenu1, kElementIdentifier1,
+                                        kScreenBounds1);
+  element_tracker_->NotifyMenuItemShown(kFakeMenu2, kElementIdentifier2,
+                                        kScreenBounds2);
+  element_tracker_->NotifyMenuItemHidden(kFakeMenu2, kElementIdentifier2);
+  element_tracker_->NotifyMenuItemHidden(kFakeMenu1, kElementIdentifier1);
+  element_tracker_->NotifyMenuItemActivated(kFakeMenu2, kElementIdentifier2);
+  element_tracker_->NotifyMenuDoneShowing(kFakeMenu1);
+}
+
 }  // namespace ui
