@@ -4,11 +4,11 @@
 
 #include "ui/base/metadata/base_type_conversion.h"
 
+#include <algorithm>
 #include <cmath>
 #include <string>
 
 #include "base/containers/fixed_flat_set.h"
-#include "base/cxx17_backports.h"
 #include "base/no_destructor.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
@@ -542,11 +542,11 @@ TypeConverter<SkColorUnique>::ParseHslString(
       (values.size() == 3 ||
        (base::StringToDouble(values[3], &a) && a >= 0.0 && a <= 1.0))) {
     SkScalar hsv[3];
-    hsv[0] = base::clamp(std::fmod(h, 360.0), 0.0, 360.0);
+    hsv[0] = std::clamp(std::fmod(h, 360.0), 0.0, 360.0);
     hsv[1] =
-        s > 1.0 ? base::clamp(s, 0.0, 100.0) / 100.0 : base::clamp(s, 0.0, 1.0);
+        s > 1.0 ? std::clamp(s, 0.0, 100.0) / 100.0 : std::clamp(s, 0.0, 1.0);
     hsv[2] =
-        v > 1.0 ? base::clamp(v, 0.0, 100.0) / 100.0 : base::clamp(v, 0.0, 1.0);
+        v > 1.0 ? std::clamp(v, 0.0, 100.0) / 100.0 : std::clamp(v, 0.0, 1.0);
     return absl::make_optional(
         SkHSVToColor(base::ClampRound<SkAlpha>(a * SK_AlphaOPAQUE), hsv));
   }
@@ -603,3 +603,9 @@ DEFINE_ENUM_CONVERTERS(
     {ui::MenuSeparatorType::SPACING_SEPARATOR, u"SPACING_SEPARATOR"},
     {ui::MenuSeparatorType::VERTICAL_SEPARATOR, u"VERTICAL_SEPARATOR"},
     {ui::MenuSeparatorType::PADDED_SEPARATOR, u"PADDED_SEPARATOR"})
+
+DEFINE_ENUM_CONVERTERS(ui::ButtonStyle,
+                       {ui::ButtonStyle::kDefault, u"kDefault"},
+                       {ui::ButtonStyle::kProminent, u"kProminent"},
+                       {ui::ButtonStyle::kTonal, u"kTonal"},
+                       {ui::ButtonStyle::kText, u"kText"})
