@@ -52,4 +52,19 @@ void ComboboxModel::RemoveObserver(ComboboxModelObserver* observer) {
   observers_.RemoveObserver(observer);
 }
 
+absl::optional<ui::ColorId> ComboboxModel::GetDropdownForegroundColorIdAt(
+    size_t index) const {
+  return absl::nullopt;
+}
+
+absl::optional<ui::ColorId> ComboboxModel::GetDropdownBackgroundColorIdAt(
+    size_t index) const {
+  return absl::nullopt;
+}
+
+absl::optional<ui::ColorId>
+ComboboxModel::GetDropdownSelectedBackgroundColorIdAt(size_t index) const {
+  return absl::nullopt;
+}
+
 }  // namespace ui
