@@ -154,11 +154,10 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
     virtual ~Delegate() = default;
   };
 
+  using LottieData = std::vector<uint8_t>;
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-  using LottieImageParseFunction =
-      gfx::ImageSkia (*)(const std::string& bytes_string);
-  using LottieThemedImageParseFunction =
-      ui::ImageModel (*)(const std::string& bytes_string);
+  using LottieImageParseFunction = gfx::ImageSkia (*)(LottieData);
+  using LottieThemedImageParseFunction = ui::ImageModel (*)(LottieData);
 #endif
 
   // Initialize the ResourceBundle for this process. Does not take ownership of
@@ -213,6 +212,10 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
       LottieThemedImageParseFunction parse_lottie_as_themed_still_image);
 #endif
 
+  // Exposed for testing, otherwise use GetSharedInstance().
+  explicit ResourceBundle(Delegate* delegate);
+  ~ResourceBundle();
+
   ResourceBundle(const ResourceBundle&) = delete;
   ResourceBundle& operator=(const ResourceBundle&) = delete;
 
@@ -300,6 +303,12 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
   // loading code of ResourceBundle.
   gfx::Image& GetNativeImageNamed(int resource_id);
 
+  // Loads a Lottie resource from `resource_id` and returns its decompressed
+  // contents. Returns `absl::nullopt` if `resource_id` does not index a
+  // Lottie resource. The output of this is suitable for passing to
+  // `SkottieWrapper`.
+  absl::optional<LottieData> GetLottieData(int resource_id) const;
+
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   // Gets a themed Lottie image (not animated) with the specified |resource_id|
   // from the current module data. |ResourceBundle| owns the result.
@@ -405,9 +414,6 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
   // Returns k100Percent if no resource is loaded.
   ResourceScaleFactor GetMaxResourceScaleFactor() const;
 
-  // Returns true if |scale_factor| is supported by this platform.
-  static bool IsScaleFactorSupported(ResourceScaleFactor scale_factor);
-
   // Checks whether overriding locale strings is supported. This will fail with
   // a DCHECK if the first string resource has already been queried.
   void CheckCanOverrideStringResources();
@@ -439,10 +445,6 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
 
   using IdToStringMap = std::unordered_map<int, std::u16string>;
 
-  // Ctor/dtor are private, since we're a singleton.
-  explicit ResourceBundle(Delegate* delegate);
-  ~ResourceBundle();
-
   // Shared initialization.
   static void InitSharedInstance(Delegate* delegate);
 
@@ -531,12 +533,6 @@ class COMPONENT_EXPORT(UI_BASE) ResourceBundle {
                         SkBitmap* bitmap,
                         bool* fell_back_to_1x);
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  // Creates the |bytes_string| from a Lottie asset, given the |resource_id|.
-  // Returns false if the resource is not a Lottie asset.
-  bool LoadLottieBytesString(int resource_id, std::string* bytes_string) const;
-#endif
-
   // Returns an empty image for when a resource cannot be loaded. This is a
   // bright red bitmap.
   gfx::Image& GetEmptyImage();
