@@ -10,13 +10,15 @@
 #include <memory>
 #include <unordered_set>
 
-#include "base/callback_forward.h"
 #include "base/containers/flat_map.h"
 #include "base/containers/flat_set.h"
+#include "base/functional/callback_forward.h"
+#include "base/functional/callback_helpers.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/observer_list.h"
 #include "base/power_monitor/power_observer.h"
+#include "base/scoped_observation_traits.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
@@ -46,6 +48,7 @@
 #include "ui/compositor/layer_animator_collection.h"
 #include "ui/compositor/throughput_tracker.h"
 #include "ui/compositor/throughput_tracker_host.h"
+#include "ui/display/types/display_constants.h"
 #include "ui/gfx/display_color_spaces.h"
 #include "ui/gfx/geometry/size.h"
 #include "ui/gfx/geometry/vector2d.h"
@@ -65,7 +68,7 @@ class LayerTreeDebugState;
 class LayerTreeFrameSink;
 class LayerTreeSettings;
 class TaskGraphRunner;
-}
+}  // namespace cc
 
 namespace gfx {
 namespace mojom {
@@ -74,7 +77,7 @@ class DelegatedInkPointRenderer;
 struct PresentationFeedback;
 class Rect;
 class Size;
-}
+}  // namespace gfx
 
 namespace gpu {
 class GpuMemoryBufferManager;
@@ -245,6 +248,12 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   void SetDisplayColorSpaces(
       const gfx::DisplayColorSpaces& display_color_spaces);
 
+#if BUILDFLAG(IS_MAC)
+  // Set the current CGDirectDisplayID and update the private client.
+  void SetVSyncDisplayID(const int64_t display_id);
+  int64_t display_id() const;
+#endif
+
   const gfx::DisplayColorSpaces& display_color_spaces() const {
     return display_color_spaces_;
   }
@@ -255,6 +264,19 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
     return host_->display_transform_hint();
   }
 
+  const viz::LocalSurfaceId& local_surface_id_from_parent() const {
+    return host_->local_surface_id_from_parent();
+  }
+
+  void SetLocalSurfaceIdFromParent(
+      const viz::LocalSurfaceId& local_surface_id_from_parent) {
+    host_->SetLocalSurfaceIdFromParent(local_surface_id_from_parent);
+  }
+
+  void SetExternalPageScaleFactor(float scale) {
+    host_->SetExternalPageScaleFactor(scale, false);
+  }
+
   // Returns the size of the widget that is being drawn to in pixel coordinates.
   const gfx::Size& size() const { return size_; }
 
@@ -283,6 +305,12 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   void AddVSyncParameterObserver(
       mojo::PendingRemote<viz::mojom::VSyncParameterObserver> observer);
 
+  // Sets and caches the maximum vsync interval, to be applied to the
+  // |display_private_| when possible, for use with variable refresh rates. An
+  // absent value indicates that VRR is not enabled.
+  void SetMaxVrrInterval(
+      const absl::optional<base::TimeDelta>& max_vrr_interval);
+
   // Sets the widget for the compositor to render into.
   void SetAcceleratedWidget(gfx::AcceleratedWidget widget);
   // Releases the widget previously set through SetAcceleratedWidget().
@@ -321,18 +349,30 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   std::unique_ptr<CompositorLock> GetCompositorLock(
       CompositorLockClient* client,
       base::TimeDelta timeout = base::Milliseconds(kCompositorLockTimeoutMs)) {
-    return lock_manager_.GetCompositorLock(client, timeout,
-                                           host_->DeferMainFrameUpdate());
+    return lock_manager_.GetCompositorLock(
+        client, timeout,
+        base::DoNothingWithBoundArgs(host_->DeferMainFrameUpdate()));
   }
 
+  // Registers a callback that is run when the presentation feedback for the
+  // next submitted frame is received (it's entirely possible some frames may be
+  // dropped between the time this is called and the callback is run).
+  // See ui/gfx/presentation_feedback.h for details on the args (TimeTicks is
+  // always non-zero).
+  // Note that since this might be called on failed presentations, it is
+  // deprecated in favor of `RequestSuccessfulPresentationTimeForNextFrame()`
+  // which will be called only after a successful presentation.
+  using PresentationTimeCallback =
+      base::OnceCallback<void(const gfx::PresentationFeedback&)>;
+  void RequestPresentationTimeForNextFrame(PresentationTimeCallback callback);
+
   // Registers a callback that is run when the next frame successfully makes it
   // to the screen (it's entirely possible some frames may be dropped between
   // the time this is called and the callback is run).
-  // See ui/gfx/presentation_feedback.h for details on the args (TimeTicks is
-  // always non-zero).
-  using PresentationTimeCallback =
-      base::OnceCallback<void(const gfx::PresentationFeedback&)>;
-  void RequestPresentationTimeForNextFrame(PresentationTimeCallback callback);
+  using SuccessfulPresentationTimeCallback =
+      base::OnceCallback<void(base::TimeTicks)>;
+  void RequestSuccessfulPresentationTimeForNextFrame(
+      SuccessfulPresentationTimeCallback callback);
 
   void IssueExternalBeginFrame(
       const viz::BeginFrameArgs& args,
@@ -359,7 +399,7 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
       bool,
       cc::PaintHoldingReason,
       absl::optional<cc::PaintHoldingCommitTrigger>) override {}
-  void OnPauseRenderingChanged(bool) override {}
+  void OnCommitRequested() override {}
   void WillUpdateLayers() override {}
   void DidUpdateLayers() override;
   void BeginMainFrame(const viz::BeginFrameArgs& args) override;
@@ -393,8 +433,6 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   void DidObserveFirstScrollDelay(
       base::TimeDelta first_scroll_delay,
       base::TimeTicks first_scroll_timestamp) override {}
-  void ReportEventLatency(
-      std::vector<cc::EventLatencyTracker::LatencyData> latencies) override;
 
   // cc::LayerTreeHostSingleThreadClient implementation.
   void DidSubmitCompositorFrame() override;
@@ -462,6 +500,7 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
  private:
   friend class base::RefCounted<Compositor>;
   friend class TotalAnimationThroughputReporter;
+  friend class TestCompositorHost;
 
   static void SendDamagedRectsRecursive(Layer* layer);
 
@@ -497,6 +536,11 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   // A sequence number of a current compositor frame for use with metrics.
   int activated_frame_count_ = 0;
 
+#if BUILDFLAG(IS_MAC)
+  // Current CGDirectDisplayID for the screen.
+  int64_t display_id_ = display::kInvalidDisplayId;
+#endif
+
   // Current vsync refresh rate per second. Initialized to 60hz as a reasonable
   // value until first begin frame arrives with the real refresh rate.
   float refresh_rate_ = 60.f;
@@ -516,6 +560,7 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
   base::TimeTicks vsync_timebase_;
   base::TimeDelta vsync_interval_ = viz::BeginFrameArgs::DefaultInterval();
   bool has_vsync_params_ = false;
+  absl::optional<base::TimeDelta> max_vrr_interval_ = absl::nullopt;
 
   const bool use_external_begin_frame_control_;
   const bool force_software_compositor_;
@@ -582,4 +627,21 @@ class COMPOSITOR_EXPORT Compositor : public base::PowerSuspendObserver,
 
 }  // namespace ui
 
+namespace base {
+
+template <>
+struct ScopedObservationTraits<ui::Compositor,
+                               ui::CompositorAnimationObserver> {
+  static void AddObserver(ui::Compositor* source,
+                          ui::CompositorAnimationObserver* observer) {
+    source->AddAnimationObserver(observer);
+  }
+  static void RemoveObserver(ui::Compositor* source,
+                             ui::CompositorAnimationObserver* observer) {
+    source->RemoveAnimationObserver(observer);
+  }
+};
+
+}  // namespace base
+
 #endif  // UI_COMPOSITOR_COMPOSITOR_H_
