@@ -103,6 +103,13 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
     sync_visibility_with_source_ = sync_visibility;
   }
 
+  // This method is relevant only if this layer is a mirror destination layer.
+  // Sets whether this mirror layer's rounded corners are synchronized with the
+  // source layer's rounded corners.
+  void set_sync_rounded_corners_with_source(bool sync_rounded_corners) {
+    sync_rounded_corners_with_source_ = sync_rounded_corners;
+  }
+
   // Sets up this layer to mirror output of |subtree_reflected_layer|, including
   // its entire hierarchy. |this| should be of type LAYER_SOLID_COLOR and should
   // not be a descendant of |subtree_reflected_layer|. This is achieved by using
@@ -174,7 +181,7 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   // The layer's animator is responsible for causing automatic animations when
   // properties are set. It also manages a queue of pending animations and
   // handles blending of animations. The layer takes ownership of the animator.
-  void SetAnimator(LayerAnimator* animator);
+  void SetAnimator(scoped_refptr<LayerAnimator> animator);
 
   // Returns the layer's animator. Creates a default animator of one has not
   // been set. Will not return NULL.
@@ -303,8 +310,16 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
 
   // Zoom the background by a factor of |zoom|. The effect is blended along the
   // edge across |inset| pixels.
+  // NOTE: Background zoom does not currently work with software compositing,
+  // see crbug.com/1451898. Usage should generally be limited to ash chrome,
+  // which does not rely on software compositing. Elsewhere, background zoom can
+  // still be set, but it will have no effect when software compositing is used
+  // (e.g. as a fallback when the GPU process has crashed too many times).
   void SetBackgroundZoom(float zoom, int inset);
 
+  // Applies an offset when drawing pixels for the layer background filter.
+  void SetBackgroundOffset(const gfx::Point& background_offset);
+
   // Set the shape of this layer.
   const ShapeRects* alpha_shape() const { return alpha_shape_.get(); }
   void SetAlphaShape(std::unique_ptr<ShapeRects> shape);
@@ -326,10 +341,10 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   Layer* layer_mask_layer() { return layer_mask_; }
   const Layer* layer_mask_layer() const { return layer_mask_; }
 
-  // Sets the visibility of the Layer. A Layer may be visible but not drawn.
-  // This happens if any ancestor of a Layer is not visible.
-  // Any changes made to this in the source layer will override the visibility
-  // of its mirror layer.
+  // Sets the visibility of the Layer. A Layer itself may be visible but not
+  // fully visible in the layer tree.  This happens if any ancestor of a
+  // Layer is not visible.  Any changes made to this in the source layer will
+  // override the visibility of its mirror layer.
   void SetVisible(bool visible);
   bool visible() const { return visible_; }
 
@@ -337,9 +352,9 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   // returns the current visibility.
   bool GetTargetVisibility() const;
 
-  // Returns true if this Layer is drawn. A Layer is drawn only if all ancestors
-  // are visible.
-  bool IsDrawn() const;
+  // Returns true if this Layer is visible. A Layer is visible only if
+  // all ancestors are visible.
+  bool IsVisible() const;
 
   // If set to true, this layer can receive hit test events, this property does
   // not affect the layer's descendants.
@@ -357,6 +372,7 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   const gfx::LinearGradient& gradient_mask() const {
     return cc_layer_->gradient_mask();
   }
+  bool HasGradientMask() { return !cc_layer_->gradient_mask().IsEmpty(); }
 
   // If set to true, this layer would not trigger a render surface (if possible)
   // due to having a rounded corner resulting in a better performance at the
@@ -398,7 +414,8 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   void SetName(const std::string& name);
 
   // Set new TransferableResource for this layer. This method only supports
-  // a gpu-backed |resource|.
+  // a gpu-backed |resource| which is assumed to have top-left origin. Clients
+  // should call SetTextureFlipped(true) for bottom-left origin resources.
   void SetTransferableResource(const viz::TransferableResource& resource,
                                viz::ReleaseCallback release_callback,
                                gfx::Size texture_size_in_dip);
@@ -572,6 +589,8 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
     compositor_ = compositor;
   }
 
+  void set_no_mutation(bool no_mutation) { no_mutation_ = no_mutation; }
+
  private:
   // TODO(https://crbug.com/1242749): temporary while tracking down crash.
   friend class Compositor;
@@ -712,11 +731,15 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   // layer's visibility are propagated to this mirror layer.
   bool sync_visibility_with_source_ = true;
 
+  // If true, and this is a destination mirror layer, changes in the rounded
+  // corners of the source layer are propagated to this mirror layer.
+  bool sync_rounded_corners_with_source_ = true;
+
   gfx::Rect bounds_;
 
   std::unique_ptr<SubpixelPositionOffsetCache> subpixel_position_offset_;
 
-  // Visibility of this layer. See SetVisible/IsDrawn for more details.
+  // Visibility of this layer. See SetVisible/IsVisible for more details.
   bool visible_;
 
   // Whether or not the layer wants to receive hit testing events. When set to
@@ -764,12 +787,15 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   // Width of the border in pixels, where the scaling is blended.
   int zoom_inset_;
 
+  // Offset to apply when drawing pixels for the layer background filter.
+  gfx::Point background_offset_;
+
   // Shape of the window.
   std::unique_ptr<ShapeRects> alpha_shape_;
 
   std::string name_;
 
-  raw_ptr<LayerDelegate> delegate_;
+  raw_ptr<LayerDelegate, DanglingUntriaged> delegate_;
 
   base::ObserverList<LayerObserver>::Unchecked observer_list_;
 
@@ -827,6 +853,7 @@ class COMPOSITOR_EXPORT Layer : public LayerAnimationDelegate,
   // TODO(https://crbug.com/1242749): temporary while tracking down crash.
   bool in_send_damaged_rects_ = false;
   bool sending_damaged_rects_for_descendants_ = false;
+  bool no_mutation_ = false;  // CHECK on Add/SetMakeLayer if true.
 
   base::WeakPtrFactory<Layer> weak_ptr_factory_{this};
 };
