@@ -63,13 +63,13 @@ LayerAnimator::~LayerAnimator() {
 }
 
 // static
-LayerAnimator* LayerAnimator::CreateDefaultAnimator() {
-  return new LayerAnimator(base::Milliseconds(0));
+scoped_refptr<LayerAnimator> LayerAnimator::CreateDefaultAnimator() {
+  return base::MakeRefCounted<LayerAnimator>(base::Milliseconds(0));
 }
 
 // static
-LayerAnimator* LayerAnimator::CreateImplicitAnimator() {
-  return new LayerAnimator(
+scoped_refptr<LayerAnimator> LayerAnimator::CreateImplicitAnimator() {
+  return base::MakeRefCounted<LayerAnimator>(
       base::Milliseconds(kLayerAnimatorDefaultTransitionDurationMs));
 }
 
