@@ -13,9 +13,9 @@
 #include <vector>
 
 #include "base/auto_reset.h"
-#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/containers/contains.h"
+#include "base/functional/bind.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/no_destructor.h"
@@ -26,12 +26,12 @@
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/system/sys_info.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
-#include "chromeos/system/devicemode.h"
 #include "chromeos/ui/base/display_util.h"
+#include "components/device_event_log/device_event_log.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/display/display.h"
 #include "ui/display/display_features.h"
@@ -41,10 +41,11 @@
 #include "ui/display/manager/display_change_observer.h"
 #include "ui/display/manager/display_configurator.h"
 #include "ui/display/manager/display_layout_store.h"
-#include "ui/display/manager/display_manager_util.h"
 #include "ui/display/manager/managed_display_info.h"
+#include "ui/display/manager/util/display_manager_util.h"
 #include "ui/display/screen.h"
 #include "ui/display/tablet_state.h"
+#include "ui/display/types/display_constants.h"
 #include "ui/display/types/display_snapshot.h"
 #include "ui/display/types/native_display_delegate.h"
 #include "ui/display/util/display_util.h"
@@ -65,17 +66,12 @@ const int kMinimumOverlapForInvalidOffset = 100;
 // The UMA histogram that logs the types of mirror mode.
 const char kMirrorModeTypesHistogram[] = "DisplayManager.MirrorModeTypes";
 
-// The UMA histogram that logs the range in which the number of connected
-// displays in mirror mode can reside.
-const char kMirroringDisplayCountRangesHistogram[] =
-    "DisplayManager.MirroringDisplayCountRanges";
-
 // The UMA histogram that logs whether mirroring is done in hardware or
 // software.
 const char kMirroringImplementationHistogram[] =
     "DisplayManager.MirroringImplementation";
 
-// The UMA historgram that logs the zoom percentage level of the intenral
+// The UMA histogram that logs the zoom percentage level of the internal
 // display.
 constexpr char kInternalDisplayZoomPercentageHistogram[] =
     "DisplayManager.InternalDisplayZoomPercentage";
@@ -333,7 +329,7 @@ DisplayManager::BeginEndNotifier::~BeginEndNotifier() {
 
 DisplayManager::DisplayManager(std::unique_ptr<Screen> screen)
     : screen_(std::move(screen)), layout_store_(new DisplayLayoutStore) {
-  configure_displays_ = chromeos::IsRunningAsSystemCompositor();
+  SetConfigureDisplays(base::SysInfo::IsRunningOnChromeOS());
   change_display_upon_host_resize_ = !configure_displays_;
   unified_desktop_enabled_ = base::CommandLine::ForCurrentProcess()->HasSwitch(
       ::switches::kEnableUnifiedDesktop);
@@ -346,6 +342,13 @@ DisplayManager::~DisplayManager() {
   on_display_zoom_modify_timeout_.Cancel();
 }
 
+void DisplayManager::SetConfigureDisplays(bool configure_displays) {
+  configure_displays_ = configure_displays;
+  if (display_configurator_) {
+    display_configurator_->SetConfigureDisplays(configure_displays);
+  }
+}
+
 bool DisplayManager::InitFromCommandLine() {
   DisplayInfoList info_list;
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
@@ -629,7 +632,9 @@ void DisplayManager::RegisterDisplayProperty(
     float device_scale_factor,
     float display_zoom_factor,
     float refresh_rate,
-    bool is_interlaced) {
+    bool is_interlaced,
+    VariableRefreshRateState variable_refresh_rate_state,
+    const absl::optional<uint16_t>& vsync_rate_min) {
   if (display_info_.find(display_id) == display_info_.end())
     display_info_[display_id] =
         ManagedDisplayInfo(display_id, std::string(), false);
@@ -649,6 +654,8 @@ void DisplayManager::RegisterDisplayProperty(
 
   info.set_refresh_rate(refresh_rate);
   info.set_is_interlaced(is_interlaced);
+  info.set_variable_refresh_rate_state(variable_refresh_rate_state);
+  info.set_vsync_rate_min(vsync_rate_min);
 
   if (!resolution_in_pixels.IsEmpty()) {
     DCHECK(!IsInternalDisplayId(display_id));
@@ -729,9 +736,9 @@ gfx::Insets DisplayManager::GetOverscanInsets(int64_t display_id) const {
 
 void DisplayManager::OnNativeDisplaysChanged(
     const DisplayInfoList& updated_displays) {
+  DISPLAY_LOG(EVENT) << "Displays updated, count:" << updated_displays.size()
+                     << " active:" << active_display_list_.size();
   if (updated_displays.empty()) {
-    VLOG(1) << __func__
-            << "(0): # of current displays=" << active_display_list_.size();
     // If the device is booted without display, or chrome is started
     // without --ash-host-window-bounds on linux desktop, use the
     // default display.
@@ -739,10 +746,12 @@ void DisplayManager::OnNativeDisplaysChanged(
       DisplayInfoList init_displays;
       init_displays.push_back(
           ManagedDisplayInfo::CreateFromSpec(std::string()));
+      init_displays[0].set_detected(false);
       MaybeInitInternalDisplay(&init_displays[0]);
       OnNativeDisplaysChanged(init_displays);
     } else {
-      // Otherwise don't update the displays when all displays are disconnected.
+      // Otherwise just update the displays' detected state when all displays
+      // are disconnected.
       // This happens when:
       // - the device is idle and powerd requested to turn off all displays.
       // - the device is suspended. (kernel turns off all displays)
@@ -752,15 +761,23 @@ void DisplayManager::OnNativeDisplaysChanged(
       //   disconnected.
       // The display will be updated when one of displays is turned on, and the
       // display list will be updated correctly.
+
+      for (auto& display : active_display_list_) {
+        if (display.detected()) {
+          ManagedDisplayInfo info = GetDisplayInfo(display.id());
+          info.set_detected(false);
+          display.set_detected(false);
+          InsertAndUpdateDisplayInfo(info);
+          NotifyMetricsChanged(display,
+                               DisplayObserver::DISPLAY_METRIC_DETECTED);
+        }
+      }
     }
     return;
   }
-  VLOG_IF(1, updated_displays.size() == 1)
-      << __func__ << "(1):" << updated_displays[0].ToString();
-  VLOG_IF(1, updated_displays.size() > 1)
-      << __func__ << "(" << updated_displays.size()
-      << ") [0]=" << updated_displays[0].ToString()
-      << ", [1]=" << updated_displays[1].ToString();
+  for (const auto& display : updated_displays) {
+    DISPLAY_LOG(EVENT) << display.ToString();
+  }
 
   first_display_id_ = updated_displays[0].id();
   std::map<gfx::Point, int64_t> origins;
@@ -982,6 +999,16 @@ void DisplayManager::UpdateDisplaysWith(
       if (current_display.label() != new_display.label())
         metrics |= DisplayObserver::DISPLAY_METRIC_LABEL;
 
+      if (current_display_info.variable_refresh_rate_state() !=
+              new_display_info.variable_refresh_rate_state() ||
+          current_display_info.vsync_rate_min() !=
+              new_display_info.vsync_rate_min()) {
+        metrics |= DisplayObserver::DISPLAY_METRIC_VRR;
+      }
+      if (current_display_info.detected() != new_display_info.detected()) {
+        metrics |= DisplayObserver::DISPLAY_METRIC_DETECTED;
+      }
+
       if (metrics != DisplayObserver::DISPLAY_METRIC_NONE) {
         display_changes.insert(
             std::pair<size_t, uint32_t>(new_displays.size(), metrics));
@@ -1004,12 +1031,23 @@ void DisplayManager::UpdateDisplaysWith(
       ++new_info_iter;
     }
   }
+
   Display old_primary;
-  if (delegate_)
-    old_primary = screen_->GetPrimaryDisplay();
+  if (delegate_) {
+    // Get old primary from current resolved layout, because we could be in the
+    // middle of updating the primary display, so screen_->GetPrimaryDisplay()
+    // may already point to the new primary.
+    if (current_resolved_layout_) {
+      Display* primary = FindDisplayForId(current_resolved_layout_->primary_id);
+      if (primary)
+        old_primary = *primary;
+    }
+    if (!old_primary.is_valid())
+      old_primary = screen_->GetPrimaryDisplay();
+  }
 
   // Clear focus if the display has been removed, but don't clear focus if
-  // the destkop has been moved from one display to another
+  // the desktop has been moved from one display to another
   // (mirror -> docked, docked -> single internal).
   bool clear_focus =
       !removed_displays.empty() &&
@@ -1030,6 +1068,13 @@ void DisplayManager::UpdateDisplaysWith(
     }
   }
 
+  if (new_displays != active_display_list_) {
+    DISPLAY_LOG(EVENT) << "Displays updated, count:" << new_displays.size();
+    for (const auto& display : new_displays) {
+      DISPLAY_LOG(EVENT) << display.ToString();
+    }
+  }
+
   active_display_list_ = new_displays;
   active_only_display_list_ = active_display_list_;
 
@@ -1068,13 +1113,17 @@ void DisplayManager::UpdateDisplaysWith(
   for (auto iter = display_changes.begin(); iter != display_changes.end();
        ++iter) {
     uint32_t metrics = iter->second;
-    const Display& updated_display = active_display_list_[iter->first];
+    Display& updated_display = active_display_list_[iter->first];
 
     if (notify_primary_change &&
         updated_display.id() == screen_->GetPrimaryDisplay().id()) {
       metrics |= DisplayObserver::DISPLAY_METRIC_PRIMARY;
       notify_primary_change = false;
     }
+    if (!updated_display.detected()) {
+      updated_display.set_detected(true);
+      metrics |= DisplayObserver::DISPLAY_METRIC_DETECTED;
+    }
     NotifyMetricsChanged(updated_display, metrics);
   }
 
@@ -1110,10 +1159,6 @@ void DisplayManager::UpdateDisplaysWith(
     delegate_->PostDisplayConfigurationChange();
 
   if (mirror_mode) {
-    UMA_HISTOGRAM_ENUMERATION(
-        kMirroringDisplayCountRangesHistogram,
-        GetDisplayCountRange(GetMirroringDestinationDisplayIdList().size() + 1),
-        DisplayCountRange::kCount);
     UMA_HISTOGRAM_ENUMERATION(kMirroringImplementationHistogram,
                               IsInSoftwareMirrorMode()
                                   ? MirroringImplementation::kSoftware
@@ -1155,7 +1200,7 @@ const Display& DisplayManager::GetFakePrimaryDisplay() {
     // https://crbug.com/1057501
     gfx::DisplayColorSpaces display_color_spaces(
         gfx::ColorSpace::CreateSRGB(), DisplaySnapshot::PrimaryFormat());
-    fake_display->set_color_spaces(display_color_spaces);
+    fake_display->SetColorSpaces(display_color_spaces);
   }
   return *fake_display;
 }
@@ -1294,7 +1339,8 @@ const Display DisplayManager::GetMirroringDisplayById(
     int64_t display_id) const {
   auto iter = base::ranges::find(software_mirroring_display_list_, display_id,
                                  &Display::id);
-  return iter == software_mirroring_display_list_.end() ? Display() : *iter;
+  return iter == software_mirroring_display_list_.end() ? GetInvalidDisplay()
+                                                        : *iter;
 }
 
 std::string DisplayManager::GetDisplayNameForId(int64_t id) const {
@@ -1436,6 +1482,7 @@ void DisplayManager::InitConfigurator(
   display_configurator_ = std::make_unique<display::DisplayConfigurator>();
   display_configurator_->Init(std::move(delegate),
                               false /* is_panel_fitting_enabled */);
+  display_configurator_->SetConfigureDisplays(configure_displays_);
 }
 
 void DisplayManager::ForceInitialConfigureWithObservers(
@@ -1557,7 +1604,7 @@ void DisplayManager::UpdateZoomFactor(int64_t display_id, float zoom_factor) {
     on_display_zoom_modify_timeout_.Cancel();
     on_display_zoom_modify_timeout_.Reset(
         base::BindOnce(&OnInternalDisplayZoomChanged, zoom_factor));
-    base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
         FROM_HERE, on_display_zoom_modify_timeout_.callback(),
         base::Seconds(kDisplayZoomModifyTimeoutSec));
   }
@@ -1637,7 +1684,7 @@ void DisplayManager::CreateMirrorWindowAsyncIfAny() {
   // ash::Shell::Init() will call this after the compositor is initialized.
   if (software_mirroring_display_list_.empty() || !delegate_)
     return;
-  base::ThreadTaskRunnerHandle::Get()->PostTask(
+  base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, base::BindOnce(&DisplayManager::CreateMirrorWindowIfAny,
                                 weak_ptr_factory_.GetWeakPtr()));
 }
@@ -2058,9 +2105,10 @@ Display DisplayManager::CreateDisplayFromDisplayInfoById(int64_t id) {
   new_display.set_panel_rotation(display_info.GetLogicalActiveRotation());
   new_display.set_touch_support(display_info.touch_support());
   new_display.set_maximum_cursor_size(display_info.maximum_cursor_size());
-  new_display.set_color_spaces(display_info.display_color_spaces());
+  new_display.SetColorSpaces(display_info.display_color_spaces());
   new_display.set_display_frequency(display_info.refresh_rate());
   new_display.set_label(display_info.name());
+  new_display.set_detected(display_info.detected());
 
   constexpr uint32_t kNormalBitDepthNumBitsPerChannel = 8u;
   if (display_info.bits_per_channel() > kNormalBitDepthNumBitsPerChannel) {
