@@ -14,6 +14,7 @@
 #include "base/gtest_prod_util.h"
 #include "base/memory/raw_ptr.h"
 #include "base/time/time.h"
+#include "build/build_config.h"
 #include "ui/events/event_constants.h"
 #include "ui/events/gesture_event_details.h"
 #include "ui/events/gestures/gesture_types.h"
@@ -69,7 +70,7 @@ class EVENTS_EXPORT Event {
     void set_time_stamp(base::TimeTicks time) { event_->time_stamp_ = time; }
 
    private:
-    raw_ptr<Event, DanglingUntriaged> event_;
+    raw_ptr<Event, AcrossTasksDanglingUntriaged> event_;
   };
 
   void SetNativeEvent(const PlatformEvent& event);
@@ -317,14 +318,11 @@ class EVENTS_EXPORT Event {
   LatencyInfo latency_;
   int flags_;
   PlatformEvent native_event_;
-  bool delete_native_event_ = false;
   bool cancelable_ = true;
   // Neither Event copy constructor nor the assignment operator copies
   // `target_`, as `target_` should be explicitly set so the setter will be
   // responsible for tracking it.
-  //
-  // TODO(crbug.com/1298696): Breaks events_unittests.
-  raw_ptr<EventTarget, DanglingUntriagedDegradeToNoOpWhenMTE> target_ = nullptr;
+  raw_ptr<EventTarget, AcrossTasksDanglingUntriaged> target_ = nullptr;
   EventPhase phase_ = EP_PREDISPATCH;
   EventResult result_ = ER_UNHANDLED;
 
@@ -668,7 +666,6 @@ class EVENTS_EXPORT TouchEvent : public LocatedEvent {
       : LocatedEvent(model, source, target),
         unique_event_id_(model.unique_event_id_),
         may_cause_scrolling_(model.may_cause_scrolling_),
-        hovering_(false),
         pointer_details_(model.pointer_details_) {}
 
   TouchEvent(EventType type,
@@ -709,6 +706,13 @@ class EVENTS_EXPORT TouchEvent : public LocatedEvent {
     return !!(result() & ER_DISABLE_SYNC_HANDLING);
   }
 
+  // Forces to process the gesture recognition even if the event is marked
+  // as `handled` or `stopped_propagation`.
+  void ForceProcessGesture();
+  bool force_process_gesture() const {
+    return !!(result() & ER_FORCE_PROCESS_GESTURE);
+  }
+
   const PointerDetails& pointer_details() const { return pointer_details_; }
   void SetPointerDetailsForTest(const PointerDetails& pointer_details);
 
@@ -766,7 +770,7 @@ class EVENTS_EXPORT TouchEvent : public LocatedEvent {
 //    Japanese, etc. all use VKEY_Q for the key beside Tab, while French uses
 //    VKEY_A. The stored key_code_ is non-located (e.g. VKEY_SHIFT rather than
 //    VKEY_LSHIFT, VKEY_1 rather than VKEY_NUMPAD1).
-// -- |uint32_t scan_code_| [USE_OZONE only] supports remapping of the top
+// -- |uint32_t scan_code_| [IS_OZONE only] supports remapping of the top
 //    function row based on a sysfs attribute provided by the kernel. This
 //    allows devices to have a custom top row layout and still be able to
 //    perform translation back and forth between F-Key and Action-Key. The
@@ -807,12 +811,12 @@ class EVENTS_EXPORT KeyEvent : public Event {
            base::TimeTicks time_stamp,
            bool is_char = false);
 
-  // Create a character event.
-  KeyEvent(char16_t character,
+  // Create an event with event type.
+  KeyEvent(EventType type,
            KeyboardCode key_code,
            DomCode code,
            int flags,
-           base::TimeTicks time_stamp = base::TimeTicks());
+           base::TimeTicks time_stamp);
 
   // Used for synthetic events with code of DOM KeyboardEvent (e.g. 'KeyA')
   // See also: ui/events/keycodes/dom/dom_values.txt
@@ -830,6 +834,13 @@ class EVENTS_EXPORT KeyEvent : public Event {
   // Sets whether to enable synthesizing key repeat in InitializeNative().
   static void SetSynthesizeKeyRepeatEnabled(bool enabled);
 
+  static ui::KeyEvent FromCharacter(
+      char16_t character,
+      KeyboardCode key_code,
+      DomCode code,
+      int flags,
+      base::TimeTicks time_stamp = base::TimeTicks());
+
   void InitializeNative();
 
   // This bypasses the normal mapping from keystroke events to characters,
@@ -869,13 +880,13 @@ class EVENTS_EXPORT KeyEvent : public Event {
   // events in an EventRewriter.
   void set_key_code(KeyboardCode key_code) { key_code_ = key_code; }
 
-#if defined(USE_OZONE)
+#if BUILDFLAG(IS_OZONE)
   // The scan code of the physical key. This is used to perform the mapping
   // of top row keys from Actions back to F-Keys on new Chrome OS keyboards
   // that supply the mapping via the kernel.
   uint32_t scan_code() const { return scan_code_; }
   void set_scan_code(uint32_t scan_code) { scan_code_ = scan_code; }
-#endif  // defined(USE_OZONE)
+#endif  // BUILDFLAG(IS_OZONE)
 
   // Returns the same value as key_code(), except that located codes are
   // returned in place of non-located ones (e.g. VKEY_LSHIFT or VKEY_RSHIFT
@@ -926,10 +937,10 @@ class EVENTS_EXPORT KeyEvent : public Event {
 
   KeyboardCode key_code_;
 
-#if defined(USE_OZONE)
+#if BUILDFLAG(IS_OZONE)
   // The scan code of the physical key on Chrome OS.
   uint32_t scan_code_ = 0;
-#endif  // defined(USE_OZONE)
+#endif  // BUILDFLAG(IS_OZONE)
 
   // DOM KeyboardEvent |code| (e.g. DomCode::US_A, DomCode::SPACE).
   // http://www.w3.org/TR/DOM-Level-3-Events-code/
@@ -956,7 +967,7 @@ class EVENTS_EXPORT KeyEvent : public Event {
   mutable DomKey key_ = DomKey::NONE;
 
   static KeyEvent* last_key_event_;
-#if defined(USE_OZONE)
+#if BUILDFLAG(IS_OZONE)
   static KeyEvent* last_ibus_key_event_;
 #endif
 
