@@ -7,6 +7,7 @@
 
 #include "base/memory/ref_counted.h"
 #include "ui/gfx/geometry/rect.h"
+#include "ui/gfx/video_types.h"
 
 namespace gl {
 
@@ -16,6 +17,8 @@ class DCOMPSurfaceProxy : public base::RefCounted<DCOMPSurfaceProxy> {
   virtual HANDLE GetSurfaceHandle() = 0;
   virtual void SetRect(const gfx::Rect& window_relative_rect) = 0;
   virtual void SetParentWindow(HWND parent) = 0;
+  virtual void SetProtectedVideoType(
+      gfx::ProtectedVideoType protected_video_type) = 0;
 
  protected:
   friend class base::RefCounted<DCOMPSurfaceProxy>;
