@@ -8,9 +8,9 @@
 
 #include "base/command_line.h"
 #include "base/logging.h"
+#include "base/ranges/algorithm.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
-
 #include "third_party/khronos/EGL/egl.h"
 #include "third_party/khronos/EGL/eglext.h"
 #include "ui/gl/egl_util.h"
@@ -20,6 +20,14 @@
 #include "ui/gl/gl_gl_api_implementation.h"
 #include "ui/gl/gl_surface_egl.h"
 
+#if BUILDFLAG(IS_MAC)
+#include "base/mac/mac_util.h"
+#endif
+
+#if BUILDFLAG(IS_WIN)
+#include "base/win/windows_version.h"
+#endif
+
 #ifndef EGL_CHROMIUM_create_context_bind_generates_resource
 #define EGL_CHROMIUM_create_context_bind_generates_resource 1
 #define EGL_CONTEXT_BIND_GENERATES_RESOURCE_CHROMIUM 0x33AD
@@ -43,8 +51,6 @@
 #ifndef EGL_ANGLE_external_context_and_surface
 #define EGL_ANGLE_external_context_and_surface 1
 #define EGL_EXTERNAL_CONTEXT_ANGLE 0x348E
-#define EGL_EXTERNAL_SURFACE_ANGLE 0x348F
-#define EGL_EXTERNAL_CONTEXT_SAVE_STATE_ANGLE 0x3490
 #endif /* EGL_ANGLE_external_context_and_surface */
 
 #ifndef EGL_ANGLE_create_context_client_arrays
@@ -86,11 +92,7 @@
 #define EGL_CONTEXT_VIRTUALIZATION_GROUP_ANGLE 0x3481
 #endif /* EGL_ANGLE_context_virtualization */
 
-#ifndef EGL_ANGLE_program_cache_control
-#define EGL_ANGLE_program_cache_control 1
-#define EGL_CONTEXT_PROGRAM_BINARY_CACHE_ENABLED_ANGLE 0x3459
-#endif /* EGL_ANGLE_program_cache_control */
-
+using ui::GetEGLErrorString;
 using ui::GetLastEGLErrorString;
 
 namespace gl {
@@ -103,8 +105,7 @@ namespace {
 bool ChangeContextAttributes(std::vector<EGLint>& context_attributes,
                              EGLint attribute,
                              EGLint value) {
-  auto iter = std::find(context_attributes.begin(), context_attributes.end(),
-                        attribute);
+  auto iter = base::ranges::find(context_attributes, attribute);
   if (iter != context_attributes.end()) {
     ++iter;
     if (iter != context_attributes.end()) {
@@ -116,6 +117,23 @@ bool ChangeContextAttributes(std::vector<EGLint>& context_attributes,
   return false;
 }
 
+bool IsARMSwiftShaderPlatform() {
+#if BUILDFLAG(IS_MAC)
+  return base::mac::GetCPUType() == base::mac::CPUType::kArm;
+#elif BUILDFLAG(IS_IOS)
+  return true;
+#elif BUILDFLAG(IS_WIN)
+  base::win::OSInfo::WindowsArchitecture windows_architecture =
+      base::win::OSInfo::GetInstance()->GetArchitecture();
+  base::win::OSInfo* os_info = base::win::OSInfo::GetInstance();
+  return windows_architecture == base::win::OSInfo::ARM64_ARCHITECTURE ||
+         os_info->IsWowX86OnARM64() || os_info->IsWowAMD64OnARM64();
+#else
+  // SwiftShader is not used on Android
+  return false;
+#endif
+}
+
 }  // namespace
 
 GLContextEGL::GLContextEGL(GLShareGroup* share_group)
@@ -180,6 +198,16 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
 
   bool is_swangle = IsSoftwareGLImplementation(GetGLImplementationParts());
 
+  if (attribs.webgl_compatibility_context && is_swangle &&
+      IsARMSwiftShaderPlatform()) {
+    // crbug.com/1378476: LLVM 10 is used as the JIT compiler for SwiftShader,
+    // which doesn't fully support ARM. Disable Swiftshader on ARM CPUs for
+    // WebGL until LLVM is upgraded.
+    DVLOG(1) << __FUNCTION__
+             << ": Software WebGL contexts are not supported on ARM CPUs.";
+    return false;
+  }
+
   if (gl_display_->ext->b_EGL_EXT_create_context_robustness || is_swangle) {
     DVLOG(1) << "EGL_EXT_create_context_robustness supported.";
     context_attributes.push_back(EGL_CONTEXT_OPENGL_ROBUST_ACCESS_EXT);
@@ -255,9 +283,11 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
   }
 
   if (gl_display_->ext->b_EGL_ANGLE_create_context_client_arrays) {
-    // Disable client arrays if the context supports it
     context_attributes.push_back(EGL_CONTEXT_CLIENT_ARRAYS_ENABLED_ANGLE);
-    context_attributes.push_back(EGL_FALSE);
+    context_attributes.push_back(
+        attribs.angle_create_context_client_arrays ? EGL_TRUE : EGL_FALSE);
+  } else {
+    DCHECK(!attribs.angle_create_context_client_arrays);
   }
 
   if (gl_display_->ext->b_EGL_ANGLE_robust_resource_initialization ||
@@ -302,10 +332,6 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
       context_attributes.push_back(EGL_EXTERNAL_CONTEXT_ANGLE);
       context_attributes.push_back(EGL_TRUE);
     }
-    if (attribs.angle_restore_external_context_state) {
-      context_attributes.push_back(EGL_EXTERNAL_CONTEXT_SAVE_STATE_ANGLE);
-      context_attributes.push_back(EGL_TRUE);
-    }
   }
 
   if (gl_display_->ext->b_EGL_ANGLE_context_virtualization) {
@@ -314,14 +340,6 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
         static_cast<EGLint>(attribs.angle_context_virtualization_group_number));
   }
 
-  // Skia manages program cache by itself.
-  // For WebGL program, it should manage program by itself too.
-  if (gl_display_->ext->b_EGL_ANGLE_program_cache_control) {
-    context_attributes.push_back(
-        EGL_CONTEXT_PROGRAM_BINARY_CACHE_ENABLED_ANGLE);
-    context_attributes.push_back(EGL_FALSE);
-  }
-
   // Append final EGL_NONE to signal the context attributes are finished
   context_attributes.push_back(EGL_NONE);
   context_attributes.push_back(EGL_NONE);
@@ -330,12 +348,16 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
       eglCreateContext(gl_display_->GetDisplay(), config_,
                        share_group() ? share_group()->GetHandle() : nullptr,
                        context_attributes.data());
+  if (context_) {
+    return true;
+  }
 
   // If EGL_KHR_no_config_context is in use and context creation failed,
   // it might indicate that an unsupported ES version was requested. Try
   // falling back to a lower version.
-  if (!context_ && gl_display_->ext->b_EGL_KHR_no_config_context &&
-      eglGetError() == EGL_BAD_MATCH) {
+  GLint error = eglGetError();
+  if (gl_display_->ext->b_EGL_KHR_no_config_context &&
+      (error == EGL_BAD_MATCH || error == EGL_BAD_ATTRIBUTE)) {
     // Set up the list of versions to try: 3.1 -> 3.0 -> 2.0
     std::vector<std::pair<EGLint, EGLint>> candidate_versions;
     if (context_client_major_version == 3 &&
@@ -361,18 +383,16 @@ bool GLContextEGL::Initialize(GLSurface* compatible_surface,
                            context_attributes.data());
       // Stop searching as soon as a context is successfully created.
       if (context_) {
-        break;
+        return true;
+      } else {
+        error = eglGetError();
       }
     }
   }
 
-  if (!context_) {
-    LOG(ERROR) << "eglCreateContext failed with error "
-               << GetLastEGLErrorString();
-    return false;
-  }
-
-  return true;
+  LOG(ERROR) << "eglCreateContext failed with error "
+             << GetEGLErrorString(error);
+  return false;
 }
 
 void GLContextEGL::Destroy() {
@@ -532,6 +552,12 @@ bool GLContextEGL::IsCurrent(GLSurface* surface) {
       return false;
   }
 
+  if (gl_display_) {
+    if (gl_display_->GetDisplay() != eglGetCurrentDisplay()) {
+      return false;
+    }
+  }
+
   return true;
 }
 
