@@ -24,19 +24,15 @@
 #include "ui/gl/gl_bindings.h"
 #include "ui/gl/gl_context_egl.h"
 #include "ui/gl/gl_display_egl_util.h"
+#include "ui/gl/gl_features.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_surface.h"
 
-#if defined(USE_GLX)
-#include "ui/gl/glx_util.h"
-#endif  // defined(USE_GLX)
-
-#if defined(USE_OZONE)
+#if BUILDFLAG(IS_OZONE)
 #include "ui/ozone/buildflags.h"
-#endif  // defined(USE_OZONE)
+#endif  // BUILDFLAG(IS_OZONE)
 
 #if BUILDFLAG(IS_ANDROID)
-#include <android/native_window_jni.h>
 #include "base/android/build_info.h"
 #endif
 
@@ -125,6 +121,7 @@
 #define EGL_ANGLE_platform_angle_device_id
 #define EGL_PLATFORM_ANGLE_DEVICE_ID_HIGH_ANGLE 0x34D6
 #define EGL_PLATFORM_ANGLE_DEVICE_ID_LOW_ANGLE 0x34D7
+#define EGL_PLATFORM_ANGLE_DISPLAY_KEY_ANGLE 0x34DC
 #endif /* EGL_ANGLE_platform_angle_device_id */
 
 // From ANGLE's egl/eglext.h.
@@ -147,6 +144,21 @@ namespace gl {
 
 namespace {
 
+void AdjustAngleFeaturesFromChromeFeatures(
+    std::vector<std::string>& enabled_angle_features,
+    std::vector<std::string>& disabled_angle_features) {
+#if BUILDFLAG(IS_MAC)
+  if (base::FeatureList::IsEnabled(features::kWriteMetalShaderCacheToDisk)) {
+    disabled_angle_features.push_back("enableParallelMtlLibraryCompilation");
+    enabled_angle_features.push_back("compileMetalShaders");
+    enabled_angle_features.push_back("disableProgramCaching");
+  }
+  if (base::FeatureList::IsEnabled(features::kUseBuiltInMetalShaderCache)) {
+    enabled_angle_features.push_back("loadMetalShadersFromBlobCache");
+  }
+#endif
+}
+
 std::vector<const char*> GetAttribArrayFromStringVector(
     const std::vector<std::string>& strings) {
   std::vector<const char*> attribs;
@@ -256,7 +268,8 @@ EGLDisplay GetDisplayFromType(
     const std::vector<std::string>& enabled_angle_features,
     const std::vector<std::string>& disabled_angle_features,
     bool disable_all_angle_features,
-    uint64_t system_device_id) {
+    uint64_t system_device_id,
+    DisplayKey display_key) {
   std::vector<EGLAttrib> extra_display_attribs;
   if (disable_all_angle_features) {
     extra_display_attribs.push_back(EGL_FEATURE_ALL_DISABLED_ANGLE);
@@ -272,6 +285,10 @@ EGLDisplay GetDisplayFromType(
     extra_display_attribs.push_back(EGL_PLATFORM_ANGLE_DEVICE_ID_HIGH_ANGLE);
     extra_display_attribs.push_back(high_part);
   }
+  if (display_key != DisplayKey::kDefault) {
+    extra_display_attribs.push_back(EGL_PLATFORM_ANGLE_DISPLAY_KEY_ANGLE);
+    extra_display_attribs.push_back(static_cast<EGLint>(display_key));
+  }
   EGLNativeDisplayType display = native_display.GetDisplay();
   switch (display_type) {
     case DEFAULT:
@@ -359,14 +376,14 @@ EGLDisplay GetDisplayFromType(
       extra_display_attribs.push_back(EGL_PLATFORM_ANGLE_DEVICE_TYPE_ANGLE);
       extra_display_attribs.push_back(
           EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE);
-#if defined(USE_OZONE)
+#if BUILDFLAG(IS_OZONE)
 #if BUILDFLAG(IS_CHROMEOS) && BUILDFLAG(OZONE_PLATFORM_X11)
       extra_display_attribs.push_back(
           EGL_PLATFORM_ANGLE_NATIVE_PLATFORM_TYPE_ANGLE);
       extra_display_attribs.push_back(
           EGL_PLATFORM_VULKAN_DISPLAY_MODE_HEADLESS_ANGLE);
 #endif  // BUILDFLAG(OZONE_PLATFORM_X11)
-#endif  // defined(USE_OZONE)
+#endif  // BUILDFLAG(IS_OZONE)
       return GetPlatformANGLEDisplay(
           display, EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE, enabled_angle_features,
           disabled_angle_features, extra_display_attribs);
@@ -397,9 +414,11 @@ ANGLEImplementation GetANGLEImplementationFromDisplayType(
     case ANGLE_D3D11on12:
       return ANGLEImplementation::kD3D11;
     case ANGLE_OPENGL:
+    case ANGLE_OPENGL_EGL:
     case ANGLE_OPENGL_NULL:
       return ANGLEImplementation::kOpenGL;
     case ANGLE_OPENGLES:
+    case ANGLE_OPENGLES_EGL:
     case ANGLE_OPENGLES_NULL:
       return ANGLEImplementation::kOpenGLES;
     case ANGLE_NULL:
@@ -461,13 +480,6 @@ const char* DisplayTypeString(DisplayType display_type) {
   }
 }
 
-void AddInitDisplay(std::vector<DisplayType>* init_displays,
-                    DisplayType display_type) {
-  // Make sure to not add the same display type twice.
-  if (!base::Contains(*init_displays, display_type))
-    init_displays->push_back(display_type);
-}
-
 const char* GetDebugMessageTypeString(EGLint source) {
   switch (source) {
     case EGL_DEBUG_MSG_CRITICAL_KHR:
@@ -511,159 +523,34 @@ void EGLAPIENTRY LogEGLDebugMessage(EGLenum error,
   }
 }
 
-void GetEGLInitDisplays(bool supports_angle_d3d,
-                        bool supports_angle_opengl,
-                        bool supports_angle_null,
-                        bool supports_angle_vulkan,
-                        bool supports_angle_swiftshader,
-                        bool supports_angle_egl,
-                        bool supports_angle_metal,
-                        const base::CommandLine* command_line,
-                        std::vector<DisplayType>* init_displays) {
-  // If we're already requesting software GL, make sure we don't fallback to the
-  // GPU
-  bool forceSoftwareGL = IsSoftwareGLImplementation(GetGLImplementationParts());
+void SetEglDebugMessageControl() {
+  static bool egl_debug_message_control_is_set = false;
+  if (!egl_debug_message_control_is_set) {
+    EGLAttrib controls[] = {
+        EGL_DEBUG_MSG_CRITICAL_KHR,
+        EGL_TRUE,
+        EGL_DEBUG_MSG_ERROR_KHR,
+        EGL_TRUE,
+        EGL_DEBUG_MSG_WARN_KHR,
+        EGL_TRUE,
+        EGL_DEBUG_MSG_INFO_KHR,
+        EGL_TRUE,
+        EGL_NONE,
+        EGL_NONE,
+    };
 
-  std::string requested_renderer =
-      forceSoftwareGL ? kANGLEImplementationSwiftShaderName
-                      : command_line->GetSwitchValueASCII(switches::kUseANGLE);
-
-  bool use_angle_default =
-      !forceSoftwareGL &&
-      (!command_line->HasSwitch(switches::kUseANGLE) ||
-       requested_renderer == kANGLEImplementationDefaultName);
-
-  if (supports_angle_null &&
-      requested_renderer == kANGLEImplementationNullName) {
-    AddInitDisplay(init_displays, ANGLE_NULL);
-    return;
-  }
-
-  // If no display has been explicitly requested and the DefaultANGLEOpenGL
-  // experiment is enabled, try creating OpenGL displays first.
-  // TODO(oetuaho@nvidia.com): Only enable this path on specific GPUs with a
-  // blocklist entry. http://crbug.com/693090
-  if (supports_angle_opengl && use_angle_default &&
-      base::FeatureList::IsEnabled(features::kDefaultANGLEOpenGL)) {
-    AddInitDisplay(init_displays, ANGLE_OPENGL);
-    AddInitDisplay(init_displays, ANGLE_OPENGLES);
-  }
-
-  if (supports_angle_metal && use_angle_default &&
-      base::FeatureList::IsEnabled(features::kDefaultANGLEMetal)) {
-    AddInitDisplay(init_displays, ANGLE_METAL);
-  }
-
-  if (supports_angle_vulkan && use_angle_default &&
-      features::IsDefaultANGLEVulkan()) {
-    AddInitDisplay(init_displays, ANGLE_VULKAN);
-  }
-
-  if (supports_angle_d3d) {
-    if (use_angle_default) {
-      // Default mode for ANGLE - try D3D11, else try D3D9
-      if (!command_line->HasSwitch(switches::kDisableD3D11)) {
-        AddInitDisplay(init_displays, ANGLE_D3D11);
-      }
-      AddInitDisplay(init_displays, ANGLE_D3D9);
-    } else {
-      if (requested_renderer == kANGLEImplementationD3D11Name) {
-        AddInitDisplay(init_displays, ANGLE_D3D11);
-      } else if (requested_renderer == kANGLEImplementationD3D9Name) {
-        AddInitDisplay(init_displays, ANGLE_D3D9);
-      } else if (requested_renderer == kANGLEImplementationD3D11NULLName) {
-        AddInitDisplay(init_displays, ANGLE_D3D11_NULL);
-      } else if (requested_renderer == kANGLEImplementationD3D11on12Name) {
-        AddInitDisplay(init_displays, ANGLE_D3D11on12);
-      }
-    }
-  }
-
-  if (supports_angle_opengl) {
-    if (use_angle_default && !supports_angle_d3d) {
-#if BUILDFLAG(IS_ANDROID)
-      // Don't request desktopGL on android
-      AddInitDisplay(init_displays, ANGLE_OPENGLES);
-#else
-      AddInitDisplay(init_displays, ANGLE_OPENGL);
-      AddInitDisplay(init_displays, ANGLE_OPENGLES);
-#endif  // BUILDFLAG(IS_ANDROID)
-    } else {
-      if (requested_renderer == kANGLEImplementationOpenGLName) {
-        AddInitDisplay(init_displays, ANGLE_OPENGL);
-      } else if (requested_renderer == kANGLEImplementationOpenGLESName) {
-        AddInitDisplay(init_displays, ANGLE_OPENGLES);
-      } else if (requested_renderer == kANGLEImplementationOpenGLNULLName) {
-        AddInitDisplay(init_displays, ANGLE_OPENGL_NULL);
-      } else if (requested_renderer == kANGLEImplementationOpenGLESNULLName) {
-        AddInitDisplay(init_displays, ANGLE_OPENGLES_NULL);
-      } else if (requested_renderer == kANGLEImplementationOpenGLEGLName &&
-                 supports_angle_egl) {
-        AddInitDisplay(init_displays, ANGLE_OPENGL_EGL);
-      } else if (requested_renderer == kANGLEImplementationOpenGLESEGLName &&
-                 supports_angle_egl) {
-        AddInitDisplay(init_displays, ANGLE_OPENGLES_EGL);
-      }
-    }
-  }
-
-  if (supports_angle_vulkan) {
-    if (use_angle_default) {
-      if (!supports_angle_d3d && !supports_angle_opengl) {
-        AddInitDisplay(init_displays, ANGLE_VULKAN);
-      }
-    } else if (requested_renderer == kANGLEImplementationVulkanName) {
-      AddInitDisplay(init_displays, ANGLE_VULKAN);
-    } else if (requested_renderer == kANGLEImplementationVulkanNULLName) {
-      AddInitDisplay(init_displays, ANGLE_VULKAN_NULL);
-    }
-  }
-
-  if (supports_angle_swiftshader) {
-    if (requested_renderer == kANGLEImplementationSwiftShaderName ||
-        requested_renderer == kANGLEImplementationSwiftShaderForWebGLName) {
-      AddInitDisplay(init_displays, ANGLE_SWIFTSHADER);
-    }
-  }
-
-  if (supports_angle_metal) {
-    if (use_angle_default) {
-      if (!supports_angle_opengl) {
-        AddInitDisplay(init_displays, ANGLE_METAL);
-      }
-    } else if (requested_renderer == kANGLEImplementationMetalName) {
-      AddInitDisplay(init_displays, ANGLE_METAL);
-    } else if (requested_renderer == kANGLEImplementationMetalNULLName) {
-      AddInitDisplay(init_displays, ANGLE_METAL_NULL);
-    }
-  }
-
-  // If no displays are available due to missing angle extensions or invalid
-  // flags, request the default display.
-  if (init_displays->empty()) {
-    init_displays->push_back(DEFAULT);
+    eglDebugMessageControlKHR(&LogEGLDebugMessage, controls);
   }
 }
 
 }  // namespace
 
-void GetEGLInitDisplaysForTesting(bool supports_angle_d3d,
-                                  bool supports_angle_opengl,
-                                  bool supports_angle_null,
-                                  bool supports_angle_vulkan,
-                                  bool supports_angle_swiftshader,
-                                  bool supports_angle_egl,
-                                  bool supports_angle_metal,
-                                  const base::CommandLine* command_line,
-                                  std::vector<DisplayType>* init_displays) {
-  GetEGLInitDisplays(supports_angle_d3d, supports_angle_opengl,
-                     supports_angle_null, supports_angle_vulkan,
-                     supports_angle_swiftshader, supports_angle_egl,
-                     supports_angle_metal, command_line, init_displays);
-}
-
-GLDisplay::GLDisplay(uint64_t system_device_id, DisplayPlatform type)
-    : system_device_id_(system_device_id), type_(type) {}
+GLDisplay::GLDisplay(uint64_t system_device_id,
+                     DisplayKey display_key,
+                     DisplayPlatform type)
+    : system_device_id_(system_device_id),
+      display_key_(display_key),
+      type_(type) {}
 
 GLDisplay::~GLDisplay() = default;
 
@@ -680,7 +567,6 @@ GLDisplayPlatform* GLDisplay::GetAs() {
       type_checked = std::is_same<GLDisplayPlatform, GLDisplayEGL>::value;
 #endif  // defined(USE_EGL)
       break;
-
     case X11:
 #if defined(USE_GLX)
       type_checked = std::is_same<GLDisplayPlatform, GLDisplayX11>::value;
@@ -725,12 +611,17 @@ void GLDisplayEGL::EGLGpuSwitchingObserver::OnGpuSwitched(
   eglHandleGPUSwitchANGLE(display_);
 }
 
-GLDisplayEGL::GLDisplayEGL(uint64_t system_device_id)
-    : GLDisplay(system_device_id, EGL), display_(EGL_NO_DISPLAY) {
+// Because on Apple platforms there is a member variable of a type (ObjCStorage)
+// that is defined in gl_display_egl.mm, the constructor/destructor also have to
+// be there. If making changes to this copy, be sure to adjust the other.
+#if !BUILDFLAG(IS_APPLE)
+GLDisplayEGL::GLDisplayEGL(uint64_t system_device_id, DisplayKey display_key)
+    : GLDisplay(system_device_id, display_key, EGL) {
   ext = std::make_unique<DisplayExtensionsEGL>();
 }
 
 GLDisplayEGL::~GLDisplayEGL() = default;
+#endif
 
 EGLDisplay GLDisplayEGL::GetDisplay() const {
   return display_;
@@ -754,6 +645,10 @@ void GLDisplayEGL::Shutdown() {
   egl_surfaceless_context_supported_ = false;
   egl_context_priority_supported_ = false;
   egl_android_native_fence_sync_supported_ = false;
+
+#if BUILDFLAG(IS_APPLE)
+  CleanupMetalSharedEvent();
+#endif
 }
 
 bool GLDisplayEGL::IsInitialized() const {
@@ -764,7 +659,6 @@ void GLDisplayEGL::SetDisplay(EGLDisplay display) {
   display_ = display;
 }
 
-#if !defined(TOOLKIT_QT) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_WIN)
 EGLDisplayPlatform GLDisplayEGL::GetNativeDisplay() const {
   return native_display_;
 }
@@ -795,27 +689,54 @@ bool GLDisplayEGL::IsANGLEExternalContextAndSurfaceSupported() {
   return this->ext->b_EGL_ANGLE_external_context_and_surface;
 }
 
-bool GLDisplayEGL::Initialize(EGLDisplayPlatform native_display) {
+bool GLDisplayEGL::Initialize(bool supports_angle,
+                              std::vector<DisplayType> init_displays,
+                              EGLDisplayPlatform native_display) {
   if (display_ != EGL_NO_DISPLAY)
     return true;
 
-  if (!InitializeDisplay(native_display))
+  if (!InitializeDisplay(supports_angle, init_displays, native_display,
+                         /*existing_display=*/nullptr)) {
     return false;
-  InitializeCommon();
-
-  if (ext->b_EGL_ANGLE_power_preference) {
-    gpu_switching_observer_ =
-        std::make_unique<EGLGpuSwitchingObserver>(display_);
-    ui::GpuSwitchingManager::GetInstance()->AddObserver(
-        gpu_switching_observer_.get());
   }
+  InitializeCommon(/*for_testing=*/false);
+
+  return true;
+}
+
+bool GLDisplayEGL::Initialize(GLDisplay* other_display) {
+  DCHECK(other_display);
+  DCHECK_EQ(display_, EGL_NO_DISPLAY);
+  DCHECK_NE(display_key_, other_display->display_key());
+  GLDisplayEGL* other_display_egl = other_display->GetAs<GLDisplayEGL>();
+  if (other_display_egl == nullptr || !other_display_egl->IsInitialized()) {
+    return false;
+  }
+
+  // Only allow initialization from a display from the same device.
+  if (other_display_egl->system_device_id() != system_device_id_) {
+    return false;
+  }
+
+  auto gl_implementation = GetGLImplementationParts();
+  bool supports_angle = (gl_implementation.gl == kGLImplementationEGLANGLE);
+  std::vector<DisplayType> init_displays;
+  init_displays.push_back(other_display_egl->GetDisplayType());
+  if (!InitializeDisplay(supports_angle, init_displays,
+                         other_display_egl->GetNativeDisplay(),
+                         other_display_egl)) {
+    return false;
+  }
+
+  InitializeCommon(/*for_testing=*/false);
+
   return true;
 }
 
 void GLDisplayEGL::InitializeForTesting() {
   display_ = eglGetCurrentDisplay();
   ext->InitializeExtensionSettings(display_);
-  InitializeCommon();
+  InitializeCommon(/*for_testing=*/true);
 }
 
 bool GLDisplayEGL::InitializeExtensionSettings() {
@@ -827,7 +748,10 @@ bool GLDisplayEGL::InitializeExtensionSettings() {
 
 // InitializeDisplay is necessary because the static binding code
 // needs a full Display init before it can query the Display extensions.
-bool GLDisplayEGL::InitializeDisplay(EGLDisplayPlatform native_display) {
+bool GLDisplayEGL::InitializeDisplay(bool supports_angle,
+                                     std::vector<DisplayType> init_displays,
+                                     EGLDisplayPlatform native_display,
+                                     gl::GLDisplayEGL* existing_display) {
   if (display_ != EGL_NO_DISPLAY)
     return true;
 
@@ -835,57 +759,10 @@ bool GLDisplayEGL::InitializeDisplay(EGLDisplayPlatform native_display) {
 
   bool supports_egl_debug = g_driver_egl.client_ext.b_EGL_KHR_debug;
   if (supports_egl_debug) {
-    EGLAttrib controls[] = {
-        EGL_DEBUG_MSG_CRITICAL_KHR,
-        EGL_TRUE,
-        EGL_DEBUG_MSG_ERROR_KHR,
-        EGL_TRUE,
-        EGL_DEBUG_MSG_WARN_KHR,
-        EGL_TRUE,
-        EGL_DEBUG_MSG_INFO_KHR,
-        EGL_TRUE,
-        EGL_NONE,
-        EGL_NONE,
-    };
-
-    eglDebugMessageControlKHR(&LogEGLDebugMessage, controls);
+    SetEglDebugMessageControl();
   }
 
-  bool supports_angle_d3d = false;
-  bool supports_angle_opengl = false;
-  bool supports_angle_null = false;
-  bool supports_angle_vulkan = false;
-  bool supports_angle_swiftshader = false;
-  bool supports_angle_egl = false;
-  bool supports_angle_metal = false;
-  // Check for availability of ANGLE extensions.
-  if (g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle) {
-    supports_angle_d3d = g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle_d3d;
-    supports_angle_opengl =
-        g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle_opengl;
-    supports_angle_null =
-        g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle_null;
-    supports_angle_vulkan =
-        g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle_vulkan;
-    supports_angle_swiftshader =
-        g_driver_egl.client_ext
-            .b_EGL_ANGLE_platform_angle_device_type_swiftshader;
-    supports_angle_egl = g_driver_egl.client_ext
-                             .b_EGL_ANGLE_platform_angle_device_type_egl_angle;
-    supports_angle_metal =
-        g_driver_egl.client_ext.b_EGL_ANGLE_platform_angle_metal;
-  }
-
-  bool supports_angle = supports_angle_d3d || supports_angle_opengl ||
-                        supports_angle_null || supports_angle_vulkan ||
-                        supports_angle_swiftshader || supports_angle_metal;
-
-  std::vector<DisplayType> init_displays;
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
-  GetEGLInitDisplays(supports_angle_d3d, supports_angle_opengl,
-                     supports_angle_null, supports_angle_vulkan,
-                     supports_angle_swiftshader, supports_angle_egl,
-                     supports_angle_metal, command_line, &init_displays);
 
   std::vector<std::string> enabled_angle_features =
       GetStringVectorFromCommandLine(command_line,
@@ -894,35 +771,42 @@ bool GLDisplayEGL::InitializeDisplay(EGLDisplayPlatform native_display) {
       GetStringVectorFromCommandLine(command_line,
                                      switches::kDisableANGLEFeatures);
 
+  AdjustAngleFeaturesFromChromeFeatures(enabled_angle_features,
+                                        disabled_angle_features);
+
   bool disable_all_angle_features =
       command_line->HasSwitch(switches::kDisableGpuDriverBugWorkarounds);
 
   for (size_t disp_index = 0; disp_index < init_displays.size(); ++disp_index) {
     DisplayType display_type = init_displays[disp_index];
-    EGLDisplay display = GetDisplayFromType(
-        display_type, native_display, enabled_angle_features,
-        disabled_angle_features, disable_all_angle_features, system_device_id_);
+    EGLDisplay display =
+        GetDisplayFromType(display_type, native_display, enabled_angle_features,
+                           disabled_angle_features, disable_all_angle_features,
+                           system_device_id_, display_key_);
     if (display == EGL_NO_DISPLAY) {
-      LOG(ERROR) << "EGL display query failed with error "
-                 << GetLastEGLErrorString();
+      // Assume this is not an error, so don't verbosely report it;
+      // simply try the next display type.
+      continue;
     }
 
-    // Init ANGLE platform now that we have the global display.
-    if (supports_angle) {
-      if (!angle::InitializePlatform(display)) {
-        LOG(ERROR) << "ANGLE Platform initialization failed.";
+    if (!existing_display) {
+      // Init ANGLE platform now that we have the global display.
+      if (supports_angle) {
+        if (!angle::InitializePlatform(display)) {
+          LOG(ERROR) << "ANGLE Platform initialization failed.";
+        }
+
+        SetANGLEImplementation(
+            GetANGLEImplementationFromDisplayType(display_type));
       }
 
-      SetANGLEImplementation(
-          GetANGLEImplementationFromDisplayType(display_type));
-    }
-
-    // The platform may need to unset its platform specific display env in case
-    // of vulkan if the platform doesn't support Vulkan surface.
-    absl::optional<base::ScopedEnvironmentVariableOverride> unset_display;
-    if (display_type == ANGLE_VULKAN) {
-      unset_display = GLDisplayEglUtil::GetInstance()
-                          ->MaybeGetScopedDisplayUnsetForVulkan();
+      // The platform may need to unset its platform specific display env in
+      // case of vulkan if the platform doesn't support Vulkan surface.
+      absl::optional<base::ScopedEnvironmentVariableOverride> unset_display;
+      if (display_type == ANGLE_VULKAN) {
+        unset_display = GLDisplayEglUtil::GetInstance()
+                            ->MaybeGetScopedDisplayUnsetForVulkan();
+      }
     }
 
     if (!eglInitialize(display, nullptr, nullptr)) {
@@ -934,30 +818,40 @@ bool GLDisplayEGL::InitializeDisplay(EGLDisplayPlatform native_display) {
       continue;
     }
 
-    std::ostringstream display_type_string;
-    auto gl_implementation = GetGLImplementationParts();
-    display_type_string << GetGLImplementationGLName(gl_implementation);
-    if (gl_implementation.gl == kGLImplementationEGLANGLE) {
-      display_type_string << ":" << DisplayTypeString(display_type);
+    if (!existing_display) {
+      std::ostringstream display_type_string;
+      auto gl_implementation = GetGLImplementationParts();
+      display_type_string << GetGLImplementationGLName(gl_implementation);
+      if (gl_implementation.gl == kGLImplementationEGLANGLE) {
+        display_type_string << ":" << DisplayTypeString(display_type);
+      }
+
+      static auto* egl_display_type_key = base::debug::AllocateCrashKeyString(
+          "egl-display-type", base::debug::CrashKeySize::Size32);
+      base::debug::SetCrashKeyString(egl_display_type_key,
+                                     display_type_string.str());
+
+      UMA_HISTOGRAM_ENUMERATION("GPU.EGLDisplayType", display_type,
+                                DISPLAY_TYPE_MAX);
     }
-
-    static auto* egl_display_type_key = base::debug::AllocateCrashKeyString(
-        "egl-display-type", base::debug::CrashKeySize::Size32);
-    base::debug::SetCrashKeyString(egl_display_type_key,
-                                   display_type_string.str());
-
-    UMA_HISTOGRAM_ENUMERATION("GPU.EGLDisplayType", display_type,
-                              DISPLAY_TYPE_MAX);
     display_ = display;
     display_type_ = display_type;
-    ext->InitializeExtensionSettings(display);
+    if (!existing_display) {
+      ext->InitializeExtensionSettings(display);
+    } else {
+      type_ = existing_display->type();
+      ext =
+          std::make_unique<DisplayExtensionsEGL>(*existing_display->ext.get());
+    }
     return true;
   }
 
+  LOG(ERROR) << "Initialization of all EGL display types failed.";
+
   return false;
 }
 
-void GLDisplayEGL::InitializeCommon() {
+void GLDisplayEGL::InitializeCommon(bool for_testing) {
   // According to https://source.android.com/compatibility/android-cdd.html the
   // EGL_IMG_context_priority extension is mandatory for Virtual Reality High
   // Performance support, but due to a bug in Android Nougat the extension
@@ -1029,13 +923,21 @@ void GLDisplayEGL::InitializeCommon() {
     egl_android_native_fence_sync_supported_ = true;
   }
 #endif  // BUILDFLAG(IS_ANDROID)
+
+  if (!for_testing) {
+    if (ext->b_EGL_ANGLE_power_preference) {
+      gpu_switching_observer_ =
+          std::make_unique<EGLGpuSwitchingObserver>(display_);
+      ui::GpuSwitchingManager::GetInstance()->AddObserver(
+          gpu_switching_observer_.get());
+    }
+  }
 }
 #endif  // defined(USE_EGL)
-#endif // !defined(TOOLKIT_QT) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_WIN)
 
 #if defined(USE_GLX)
-GLDisplayX11::GLDisplayX11(uint64_t system_device_id)
-    : GLDisplay(system_device_id, X11) {}
+GLDisplayX11::GLDisplayX11(uint64_t system_device_id, DisplayKey display_key)
+    : GLDisplay(system_device_id, display_key, X11) {}
 
 GLDisplayX11::~GLDisplayX11() = default;
 
@@ -1048,6 +950,10 @@ void GLDisplayX11::Shutdown() {}
 bool GLDisplayX11::IsInitialized() const {
   return true;
 }
+
+bool GLDisplayX11::Initialize(gl::GLDisplay*) {
+    // FIXME?
+}
 #endif  // defined(USE_GLX)
 
 #if BUILDFLAG(IS_WIN)
@@ -1093,8 +999,8 @@ LRESULT CALLBACK IntermediateWindowProc(HWND window,
 }
 }  // namespace
 
-GLDisplayWGL::GLDisplayWGL(uint64_t system_device_id)
-    : GLDisplay(system_device_id, WGL),
+GLDisplayWGL::GLDisplayWGL(uint64_t system_device_id, DisplayKey display_key)
+    : GLDisplay(system_device_id, display_key, WGL),
       module_handle_(0),
       window_class_(0),
       window_handle_(0),
@@ -1199,6 +1105,10 @@ bool GLDisplayWGL::IsInitialized() const
 void GLDisplayWGL::Shutdown()
 {
 }
+bool GLDisplayWGL::Initialize(gl::GLDisplay*) {
+  // FIXME?
+  return false;
+}
 #endif  // BUILDFLAG(IS_WIN)
 
 }  // namespace gl
