@@ -7,6 +7,9 @@
 
 #include "base/feature_list.h"
 #include "build/build_config.h"
+#include "build/chromecast_buildflags.h"
+#include "build/chromeos_buildflags.h"
+#include "ui/gl/buildflags.h"
 #include "ui/gl/gl_export.h"
 
 namespace features {
@@ -14,14 +17,20 @@ namespace features {
 // Controls if GPU should synchronize presentation with vsync.
 GL_EXPORT bool UseGpuVsync();
 
-#if BUILDFLAG(IS_ANDROID)
-// Use new Android 13 API to obtain and target a frame deadline.
-GL_EXPORT BASE_DECLARE_FEATURE(kAndroidFrameDeadline);
-#endif
-
+#if BUILDFLAG(ENABLE_VALIDATING_COMMAND_DECODER)
 // All features in alphabetical order. The features should be documented
 // alongside the definition of their values in the .cc file.
 GL_EXPORT BASE_DECLARE_FEATURE(kDefaultPassthroughCommandDecoder);
+#endif
+
+#if BUILDFLAG(IS_MAC)
+GL_EXPORT BASE_DECLARE_FEATURE(kWriteMetalShaderCacheToDisk);
+GL_EXPORT BASE_DECLARE_FEATURE(kUseBuiltInMetalShaderCache);
+#endif
+
+#if BUILDFLAG(IS_WIN)
+GL_EXPORT BASE_DECLARE_FEATURE(kUsePrimaryMonitorVSyncIntervalOnSV3);
+#endif  // BUILDFLAG(IS_WIN)
 
 GL_EXPORT bool IsAndroidFrameDeadlineEnabled();
 
