@@ -17,7 +17,7 @@
 #include "ui/gl/gl_version_info.h"
 #include "ui/gl/progress_reporter.h"
 
-#if BUILDFLAG(IS_APPLE)
+#if BUILDFLAG(IS_MAC)
 #include "base/mac/mac_util.h"
 #endif
 
@@ -169,7 +169,7 @@ struct Slow {};
 // Call needs to be wrapped with glFlush call, used on MacOS.
 struct NeedFlush {};
 
-#if BUILDFLAG(IS_APPLE)
+#if BUILDFLAG(IS_MAC)
 using SlowOnMac = Slow;
 using NeedFlushOnMac = NeedFlush;
 #else
@@ -190,7 +190,9 @@ struct BindWithTraits {
     constexpr bool need_flush =
         base::trait_helpers::HasTrait<NeedFlush, Traits...>();
 
-#if BUILDFLAG(IS_APPLE)
+#if BUILDFLAG(IS_MAC)
+    // If running on Apple silicon, regardless of the architecture, don't
+    // perform this workaround. See https://crbug.com/1131312.
     if (need_flush && base::mac::GetCPUType() == base::mac::CPUType::kIntel &&
         !is_angle) {
       return bind_impl<droppable, slow, /*need_flush=*/true>(func,
@@ -226,10 +228,12 @@ const char* kBlocklistExtensions[] = {
     "GL_ARB_multi_draw_indirect",
     "GL_ARB_sample_shading",
     "GL_ARB_texture_barrier",
+    "GL_CHROMIUM_framebuffer_mixed_samples",
     "GL_EXT_direct_state_access",
     "GL_EXT_multi_draw_indirect",
     "GL_EXT_raster_multisample",
     "GL_NV_bindless_texture",
+    "GL_NV_framebuffer_mixed_samples",
     "GL_NV_texture_barrier",
     "GL_OES_sample_shading",
     "GL_EXT_draw_instanced",
@@ -541,8 +545,6 @@ sk_sp<GrGLInterface> CreateGrGLInterface(
                  RenderbufferStorageMultisampleEXT, NeedFlushOnMac);
   BIND(BlitFramebuffer, NeedFlushOnMac);
 
-  BIND_EXTENSION(CoverageModulation, CoverageModulationNV);
-
   BIND_EXTENSION(InsertEventMarker, InsertEventMarkerEXT);
   BIND_EXTENSION(PushGroupMarker, PushGroupMarkerEXT);
   BIND_EXTENSION(PopGroupMarker, PopGroupMarkerEXT);
