@@ -32,6 +32,7 @@
 #include "ui/base/ime/linux/linux_input_method_context.h"
 #include "ui/color/color_id.h"
 #include "ui/color/color_provider.h"
+#include "ui/color/color_provider_key.h"
 #include "ui/color/color_provider_manager.h"
 #include "ui/display/display.h"
 #include "ui/events/keycodes/dom/dom_code.h"
@@ -67,15 +68,10 @@
 #include "ui/native_theme/native_theme.h"
 #include "ui/ozone/buildflags.h"
 #include "ui/ozone/public/ozone_platform.h"
-#include "ui/shell_dialogs/select_file_dialog_linux.h"
+#include "ui/shell_dialogs/select_file_dialog.h"
 #include "ui/shell_dialogs/select_file_policy.h"
-#include "ui/shell_dialogs/shell_dialog_linux.h"
 #include "ui/views/window/window_button_order_provider.h"
 
-#if defined(USE_GIO)
-#include "ui/gtk/settings_provider_gsettings.h"
-#endif
-
 #if BUILDFLAG(OZONE_PLATFORM_WAYLAND)
 #define USE_WAYLAND
 #endif
@@ -106,16 +102,6 @@ int indicators_count;
 // The unknown content type.
 const char kUnknownContentType[] = "application/octet-stream";
 
-std::unique_ptr<SettingsProvider> CreateSettingsProvider(GtkUi* gtk_ui) {
-  if (GtkCheckVersion(3, 14))
-    return std::make_unique<SettingsProviderGtk>(gtk_ui);
-#if defined(USE_GIO)
-  return std::make_unique<SettingsProviderGSettings>(gtk_ui);
-#else
-  return nullptr;
-#endif
-}
-
 // Returns a gfx::FontRenderParams corresponding to GTK's configuration.
 gfx::FontRenderParams GetGtkFontRenderParams() {
   GtkSettings* gtk_settings = gtk_settings_get_default();
@@ -165,23 +151,6 @@ gfx::FontRenderParams GetGtkFontRenderParams() {
   return params;
 }
 
-ui::LinuxUiTheme::WindowFrameAction GetDefaultMiddleClickAction() {
-  if (GtkCheckVersion(3, 14))
-    return ui::LinuxUiTheme::WindowFrameAction::kNone;
-  std::unique_ptr<base::Environment> env(base::Environment::Create());
-  switch (base::nix::GetDesktopEnvironment(env.get())) {
-    case base::nix::DESKTOP_ENVIRONMENT_KDE4:
-    case base::nix::DESKTOP_ENVIRONMENT_KDE5:
-      // Starting with KDE 4.4, windows' titlebars can be dragged with the
-      // middle mouse button to create tab groups. We don't support that in
-      // Chrome, but at least avoid lowering windows in response to middle
-      // clicks to avoid surprising users who expect the KDE behavior.
-      return ui::LinuxUiTheme::WindowFrameAction::kNone;
-    default:
-      return ui::LinuxUiTheme::WindowFrameAction::kLower;
-  }
-}
-
 std::unique_ptr<GtkUiPlatform> CreateGtkUiPlatform(ui::LinuxUiBackend backend) {
   switch (backend) {
     case ui::LinuxUiBackend::kStub:
@@ -200,6 +169,23 @@ std::unique_ptr<GtkUiPlatform> CreateGtkUiPlatform(ui::LinuxUiBackend backend) {
   }
 }
 
+double FontScale() {
+  double resolution = 0;
+  if (GtkCheckVersion(4)) {
+    auto* settings = gtk_settings_get_default();
+    int dpi = 0;
+    g_object_get(settings, "gtk-xft-dpi", &dpi, nullptr);
+    resolution = dpi / 1024.0;
+  } else {
+    GdkScreen* screen = gdk_screen_get_default();
+    resolution = gdk_screen_get_resolution(screen);
+  }
+  const double font_scale = resolution > 0 ? resolution / kDefaultDPI : 1.0;
+  // Round to the nearest 1/64th so that UI can losslessly multiply and divide
+  // the scale factor.
+  return std::round(font_scale * 64) / 64;
+}
+
 }  // namespace
 
 GtkUi::GtkUi() : window_frame_actions_() {
@@ -210,8 +196,6 @@ GtkUi::GtkUi() : window_frame_actions_() {
 GtkUi::~GtkUi() {
   DCHECK_EQ(g_gtk_ui, this);
   g_gtk_ui = nullptr;
-
-  shell_dialog_linux::Finalize();
 }
 
 // static
@@ -221,8 +205,9 @@ GtkUiPlatform* GtkUi::GetPlatform() {
 }
 
 bool GtkUi::Initialize() {
-  if (!LoadGtk())
+  if (!LoadGtk() || !GtkCheckVersion(3, 20)) {
     return false;
+  }
 
   auto* delegate = ui::LinuxUiDelegate::GetInstance();
   DCHECK(delegate);
@@ -234,19 +219,16 @@ bool GtkUi::Initialize() {
   env->SetVar("NO_AT_BRIDGE", "1");
   // gtk_init_check() modifies argv, so make a copy first.
   CmdLineArgs cmd_line = CopyCmdLine(*base::CommandLine::ForCurrentProcess());
-  if (!GtkInitFromCommandLine(&cmd_line.argc, cmd_line.argv.data()))
+  if (!GtkInitFromCommandLine(&cmd_line.argc, cmd_line.argv.data())) {
     return false;
+  }
   native_theme_ = NativeThemeGtk::instance();
 
-  // This creates an extra thread that may race against GtkInitFromCommandLine,
-  // so this must be done after to avoid the race condition.
-  shell_dialog_linux::Initialize();
-
-  using Action = ui::LinuxUiTheme::WindowFrameAction;
-  using ActionSource = ui::LinuxUiTheme::WindowFrameActionSource;
+  using Action = ui::LinuxUi::WindowFrameAction;
+  using ActionSource = ui::LinuxUi::WindowFrameActionSource;
   window_frame_actions_ = {
       {ActionSource::kDoubleClick, Action::kToggleMaximize},
-      {ActionSource::kMiddleClick, GetDefaultMiddleClickAction()},
+      {ActionSource::kMiddleClick, Action::kNone},
       {ActionSource::kRightClick, Action::kMenu}};
 
   GtkSettings* settings = gtk_settings_get_default();
@@ -270,16 +252,27 @@ bool GtkUi::Initialize() {
     g_signal_connect_after(screen, "notify::resolution", dpi_callback, this);
   }
 
-  // Listen for scale factor changes.  We would prefer to listen on
-  // a GdkScreen, but there is no scale-factor property, so use an
-  // unmapped window instead.
-  g_signal_connect(GetDummyWindow(), "notify::scale-factor",
-                   G_CALLBACK(OnDeviceScaleFactorMaybeChangedThunk), this);
+  // Listen for scale factor changes.
+  GdkDisplay* display = gdk_display_get_default();
+  if (GtkCheckVersion(4)) {
+    GListModel* monitors = gdk_display_get_monitors(display);
+    g_signal_connect_after(monitors, "items-changed",
+                           G_CALLBACK(OnMonitorsChangedThunk), this);
+    const guint n_monitors = g_list_model_get_n_items(monitors);
+    OnMonitorsChanged(monitors, 0, 0, n_monitors);
+  } else {
+    g_signal_connect_after(display, "monitor-added",
+                           G_CALLBACK(OnMonitorAddedThunk), this);
+    const int n_monitors = gdk_display_get_n_monitors(display);
+    for (int i = 0; i < n_monitors; i++) {
+      TrackMonitor(gdk_display_get_monitor(display, i));
+    }
+  }
 
   LoadGtkValues();
 
   // We must build this after GTK gets initialized.
-  settings_provider_ = CreateSettingsProvider(this);
+  settings_provider_ = std::make_unique<SettingsProviderGtk>(this);
 
   indicators_count = 0;
 
@@ -314,24 +307,24 @@ bool GtkUi::GetDisplayProperty(int id, int* result) const {
   return false;
 }
 
-SkColor GtkUi::GetFocusRingColor() const {
-  return focus_ring_color_;
+void GtkUi::GetFocusRingColor(SkColor* color) const {
+  *color = focus_ring_color_;
 }
 
-SkColor GtkUi::GetActiveSelectionBgColor() const {
-  return active_selection_bg_color_;
+void GtkUi::GetActiveSelectionBgColor(SkColor* color) const {
+  *color = active_selection_bg_color_;
 }
 
-SkColor GtkUi::GetActiveSelectionFgColor() const {
-  return active_selection_fg_color_;
+void GtkUi::GetActiveSelectionFgColor(SkColor* color) const {
+  *color = active_selection_fg_color_;
 }
 
-SkColor GtkUi::GetInactiveSelectionBgColor() const {
-  return inactive_selection_bg_color_;
+void GtkUi::GetInactiveSelectionBgColor(SkColor* color) const {
+  *color = inactive_selection_bg_color_;
 }
 
-SkColor GtkUi::GetInactiveSelectionFgColor() const {
-  return inactive_selection_fg_color_;
+void GtkUi::GetInactiveSelectionFgColor(SkColor* color) const {
+  *color = inactive_selection_fg_color_;
 }
 
 base::TimeDelta GtkUi::GetCursorBlinkInterval() const {
@@ -382,8 +375,9 @@ gfx::Image GtkUi::GetIconForContentType(const std::string& content_type,
       auto icon_paintable = Gtk4IconThemeLookupByGicon(
           theme, icon.get(), size, scale_int, GTK_TEXT_DIR_NONE,
           static_cast<GtkIconLookupFlags>(0));
-      if (!icon_paintable)
+      if (!icon_paintable) {
         continue;
+      }
 
       auto* paintable = GlibCast<GdkPaintable>(icon_paintable.get(),
                                                gdk_paintable_get_type());
@@ -402,12 +396,14 @@ gfx::Image GtkUi::GetIconForContentType(const std::string& content_type,
       auto icon_info = Gtk3IconThemeLookupByGiconForScale(
           theme, icon.get(), size, scale_int,
           static_cast<GtkIconLookupFlags>(GTK_ICON_LOOKUP_FORCE_SIZE));
-      if (!icon_info)
+      if (!icon_info) {
         continue;
+      }
       auto* surface =
           gtk_icon_info_load_surface(icon_info.get(), nullptr, nullptr);
-      if (!surface)
+      if (!surface) {
         continue;
+      }
       DCHECK_EQ(cairo_surface_get_type(surface), CAIRO_SURFACE_TYPE_IMAGE);
       DCHECK_EQ(cairo_image_surface_get_format(surface), CAIRO_FORMAT_ARGB32);
 
@@ -440,8 +436,9 @@ void GtkUi::SetWindowButtonOrdering(
   views::WindowButtonOrderProvider::GetInstance()->SetWindowButtonOrder(
       leading_buttons, trailing_buttons);
 
-  for (auto& observer : window_button_order_observer_list())
+  for (auto& observer : window_button_order_observer_list_) {
     observer.OnWindowButtonOrderingChange();
+  }
 }
 
 void GtkUi::SetWindowFrameAction(WindowFrameActionSource source,
@@ -479,7 +476,7 @@ ui::SelectFileDialog* GtkUi::CreateSelectFileDialog(
       std::move(policy));
 }
 
-ui::LinuxUiTheme::WindowFrameAction GtkUi::GetWindowFrameAction(
+ui::LinuxUi::WindowFrameAction GtkUi::GetWindowFrameAction(
     WindowFrameActionSource source) {
   return window_frame_actions_[source];
 }
@@ -491,6 +488,14 @@ bool GtkUi::PreferDarkTheme() const {
   return dark;
 }
 
+void GtkUi::SetDarkTheme(bool dark) {
+  auto* settings = gtk_settings_get_default();
+  g_object_set(settings, "gtk-application-prefer-dark-theme", dark, nullptr);
+  // OnThemeChanged() will be called via the
+  // notify::gtk-application-prefer-dark-theme handler to update the native
+  // theme.
+}
+
 bool GtkUi::AnimationsEnabled() const {
   gboolean animations_enabled = false;
   g_object_get(gtk_settings_get_default(), "gtk-enable-animations",
@@ -498,19 +503,26 @@ bool GtkUi::AnimationsEnabled() const {
   return animations_enabled;
 }
 
+void GtkUi::AddWindowButtonOrderObserver(
+    ui::WindowButtonOrderObserver* observer) {
+  window_button_order_observer_list_.AddObserver(observer);
+}
+
+void GtkUi::RemoveWindowButtonOrderObserver(
+    ui::WindowButtonOrderObserver* observer) {
+  window_button_order_observer_list_.RemoveObserver(observer);
+}
+
 std::unique_ptr<ui::NavButtonProvider> GtkUi::CreateNavButtonProvider() {
-  if (GtkCheckVersion(3, 14))
-    return std::make_unique<gtk::NavButtonProviderGtk>();
-  return nullptr;
+  return std::make_unique<gtk::NavButtonProviderGtk>();
 }
 
 ui::WindowFrameProvider* GtkUi::GetWindowFrameProvider(bool solid_frame) {
-  if (!GtkCheckVersion(3, 14))
-    return nullptr;
   auto& provider =
       solid_frame ? solid_frame_provider_ : transparent_frame_provider_;
-  if (!provider)
+  if (!provider) {
     provider = std::make_unique<gtk::WindowFrameProviderGtk>(solid_frame);
+  }
   return provider.get();
 }
 
@@ -529,8 +541,9 @@ base::flat_map<std::string, std::string> GtkUi::GetKeyboardLayoutMap() {
   GdkKeymap* keymap = nullptr;
   if (!GtkCheckVersion(4)) {
     keymap = gdk_keymap_get_for_display(display);
-    if (!keymap)
+    if (!keymap) {
       return {};
+    }
   }
 
   auto layouts = std::make_unique<ui::DomKeyboardLayoutManager>();
@@ -560,12 +573,14 @@ base::flat_map<std::string, std::string> GtkUi::GetKeyboardLayoutMap() {
           uint16_t unicode = gdk_keyval_to_unicode(keyvals[i]);
           if (unicode == 0) {
             for (const auto& i_dead : kDeadKeyMapping) {
-              if (keyvals[i] == i_dead.gdk_key)
+              if (keyvals[i] == i_dead.gdk_key) {
                 unicode = i_dead.unicode;
+              }
             }
           }
-          if (unicode != 0)
+          if (unicode != 0) {
             layouts->GetLayout(keys[i].group)->AddKeyMapping(domcode, unicode);
+          }
         }
       }
     }
@@ -598,19 +613,22 @@ bool GtkUi::GetTextEditCommandsForEvent(
     const ui::Event& event,
     std::vector<ui::TextEditCommandAuraLinux>* commands) {
   // GTK4 dropped custom key bindings.
-  if (GtkCheckVersion(4))
+  if (GtkCheckVersion(4)) {
     return false;
+  }
 
   // TODO(crbug.com/963419): Use delegate's |GetGdkKeymap| here to
   // determine if GtkUi's key binding handling implementation is used or not.
   // Ozone/Wayland was unintentionally using GtkUi for keybinding handling, so
   // early out here, for now, until a proper solution for ozone is implemented.
-  if (!platform_->GetGdkKeymap())
+  if (!platform_->GetGdkKeymap()) {
     return false;
+  }
 
   // Ensure that we have a keyboard handler.
-  if (!key_bindings_handler_)
+  if (!key_bindings_handler_) {
     key_bindings_handler_ = std::make_unique<GtkKeyBindingsHandler>();
+  }
 
   return key_bindings_handler_->MatchEvent(event, commands);
 }
@@ -638,25 +656,44 @@ void GtkUi::OnThemeChanged(GtkSettings* settings, GtkParamSpec* param) {
 void GtkUi::OnCursorThemeNameChanged(GtkSettings* settings,
                                      GtkParamSpec* param) {
   std::string cursor_theme_name = GetCursorThemeName();
-  if (cursor_theme_name.empty())
+  if (cursor_theme_name.empty()) {
     return;
-  for (auto& observer : cursor_theme_observers())
+  }
+  for (auto& observer : cursor_theme_observers()) {
     observer.OnCursorThemeNameChanged(cursor_theme_name);
+  }
 }
 
 void GtkUi::OnCursorThemeSizeChanged(GtkSettings* settings,
                                      GtkParamSpec* param) {
   int cursor_theme_size = GetCursorThemeSize();
-  if (!cursor_theme_size)
+  if (!cursor_theme_size) {
     return;
-  for (auto& observer : cursor_theme_observers())
+  }
+  for (auto& observer : cursor_theme_observers()) {
     observer.OnCursorThemeSizeChanged(cursor_theme_size);
+  }
 }
 
 void GtkUi::OnDeviceScaleFactorMaybeChanged(void*, GParamSpec*) {
   UpdateDeviceScaleFactor();
 }
 
+void GtkUi::OnMonitorAdded(GdkDisplay* display, GdkMonitor* monitor) {
+  TrackMonitor(monitor);
+  UpdateDeviceScaleFactor();
+}
+
+void GtkUi::OnMonitorsChanged(GListModel* monitors,
+                              guint position,
+                              guint removed,
+                              guint added) {
+  for (size_t i = position; i < position + added; i++) {
+    TrackMonitor(static_cast<GdkMonitor*>(g_list_model_get_item(monitors, i)));
+  }
+  UpdateDeviceScaleFactor();
+}
+
 void GtkUi::LoadGtkValues() {
   // TODO(thomasanderson): GtkThemeService had a comment here about having to
   // muck with the raw Prefs object to remove prefs::kCurrentThemeImages or else
@@ -679,31 +716,30 @@ void GtkUi::UpdateColors() {
   const auto* color_provider =
       ui::ColorProviderManager::Get().GetColorProviderFor(
           {(color_scheme == ui::NativeTheme::ColorScheme::kDark)
-               ? ui::ColorProviderManager::ColorMode::kDark
-               : ui::ColorProviderManager::ColorMode::kLight,
+               ? ui::ColorProviderKey::ColorMode::kDark
+               : ui::ColorProviderKey::ColorMode::kLight,
            (color_scheme == ui::NativeTheme::ColorScheme::kPlatformHighContrast)
-               ? ui::ColorProviderManager::ContrastMode::kHigh
-               : ui::ColorProviderManager::ContrastMode::kNormal,
+               ? ui::ColorProviderKey::ContrastMode::kHigh
+               : ui::ColorProviderKey::ContrastMode::kNormal,
            ui::SystemTheme::kGtk,
            // Some theme colors, e.g. COLOR_NTP_LINK, are derived from color
            // provider colors. We assume that those sources' colors won't change
            // with frame type.
-           ui::ColorProviderManager::FrameType::kChromium});
+           ui::ColorProviderKey::FrameType::kChromium});
 
-  SkColor location_bar_border = GetBorderColor("GtkEntry#entry");
-  if (SkColorGetA(location_bar_border))
+  SkColor location_bar_border = GetBorderColor("entry");
+  if (SkColorGetA(location_bar_border)) {
     colors_[ThemeProperties::COLOR_LOCATION_BAR_BORDER] = location_bar_border;
+  }
 
   inactive_selection_bg_color_ = GetSelectionBgColor(
-      GtkCheckVersion(3, 20) ? "GtkTextView#textview.view:backdrop "
-                               "#text:backdrop #selection:backdrop"
-                             : "GtkTextView.view:selected:backdrop");
-  inactive_selection_fg_color_ =
-      GetFgColor(GtkCheckVersion(3, 20) ? "GtkTextView#textview.view:backdrop "
-                                          "#text:backdrop #selection:backdrop"
-                                        : "GtkTextView.view:selected:backdrop");
+      "textview.view:backdrop "
+      "text:backdrop selection:backdrop");
+  inactive_selection_fg_color_ = GetFgColor(
+      "textview.view:backdrop "
+      "text:backdrop selection:backdrop");
 
-  SkColor tab_border = GetBorderColor("GtkButton#button");
+  SkColor tab_border = GetBorderColor("button");
   // Separates the toolbar from the bookmark bar or butter bars.
   colors_[ThemeProperties::COLOR_TOOLBAR_CONTENT_AREA_SEPARATOR] = tab_border;
   // Separates entries in the downloads bar.
@@ -713,10 +749,9 @@ void GtkUi::UpdateColors() {
       color_provider->GetColor(ui::kColorTextfieldBackground);
   colors_[ThemeProperties::COLOR_NTP_TEXT] =
       color_provider->GetColor(ui::kColorTextfieldForeground);
-  colors_[ThemeProperties::COLOR_NTP_HEADER] =
-      GetBorderColor("GtkButton#button");
+  colors_[ThemeProperties::COLOR_NTP_HEADER] = GetBorderColor("button");
 
-  SkColor tab_text_color = GetFgColor("GtkLabel#label");
+  SkColor tab_text_color = GetFgColor("label");
   colors_[ThemeProperties::COLOR_TOOLBAR_BUTTON_ICON] = tab_text_color;
   colors_[ThemeProperties::COLOR_TOOLBAR_BUTTON_ICON_HOVERED] = tab_text_color;
   colors_[ThemeProperties::COLOR_TOOLBAR_BUTTON_ICON_PRESSED] = tab_text_color;
@@ -742,7 +777,7 @@ void GtkUi::UpdateColors() {
     ColorMap& color_map =
         custom_frame ? custom_frame_colors_ : native_frame_colors_;
     const std::string header_selector =
-        custom_frame ? "#headerbar.header-bar.titlebar" : "GtkMenuBar#menubar";
+        custom_frame ? "headerbar.header-bar.titlebar" : "menubar";
     const std::string header_selector_inactive = header_selector + ":backdrop";
     const SkColor frame_color =
         SkColorSetA(GetBgColor(header_selector), SK_AlphaOPAQUE);
@@ -767,9 +802,9 @@ void GtkUi::UpdateColors() {
         tab_color;
 
     const SkColor background_tab_text_color =
-        GetFgColor(header_selector + " GtkLabel#label.title");
+        GetFgColor(header_selector + " label.title");
     const SkColor background_tab_text_color_inactive =
-        GetFgColor(header_selector_inactive + " GtkLabel#label.title");
+        GetFgColor(header_selector_inactive + " label.title");
 
     color_map[ThemeProperties::COLOR_TAB_FOREGROUND_INACTIVE_FRAME_ACTIVE] =
         background_tab_text_color;
@@ -784,11 +819,10 @@ void GtkUi::UpdateColors() {
 
     // These colors represent the border drawn around tabs and between
     // the tabstrip and toolbar.
-    SkColor toolbar_top_separator = GetBorderColor(
-        header_selector + " GtkSeparator#separator.vertical.titlebutton");
-    SkColor toolbar_top_separator_inactive =
-        GetBorderColor(header_selector +
-                       ":backdrop GtkSeparator#separator.vertical.titlebutton");
+    SkColor toolbar_top_separator =
+        GetBorderColor(header_selector + " separator.vertical.titlebutton");
+    SkColor toolbar_top_separator_inactive = GetBorderColor(
+        header_selector + ":backdrop separator.vertical.titlebutton");
 
     auto toolbar_top_separator_has_good_contrast = [&]() {
       // This constant is copied from chrome/browser/themes/theme_service.cc.
@@ -805,10 +839,9 @@ void GtkUi::UpdateColors() {
     };
 
     if (!toolbar_top_separator_has_good_contrast()) {
-      toolbar_top_separator =
-          GetBorderColor(header_selector + " GtkButton#button");
+      toolbar_top_separator = GetBorderColor(header_selector + " button");
       toolbar_top_separator_inactive =
-          GetBorderColor(header_selector + ":backdrop GtkButton#button");
+          GetBorderColor(header_selector + ":backdrop button");
     }
 
     // If we can't get a contrasting stroke from the theme, have ThemeService
@@ -827,7 +860,7 @@ void GtkUi::UpdateColors() {
 }
 
 void GtkUi::UpdateDefaultFont() {
-  gfx::SetFontRenderParamsDeviceScaleFactor(device_scale_factor_);
+  gfx::SetFontRenderParamsDeviceScaleFactor(display_config().primary_scale);
 
   auto fake_label = TakeGObject(gtk_label_new(nullptr));
   PangoContext* pc = gtk_widget_get_pango_context(fake_label);
@@ -840,65 +873,101 @@ void GtkUi::UpdateDefaultFont() {
       base::SplitString(pango_font_description_get_family(desc), ",",
                         base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
 
+  constexpr double kPangoScale = PANGO_SCALE;
+  double size_pixels;
   if (pango_font_description_get_size_is_absolute(desc)) {
     // If the size is absolute, it's specified in Pango units. There are
     // PANGO_SCALE Pango units in a device unit (pixel).
-    const int size_pixels = pango_font_description_get_size(desc) / PANGO_SCALE;
-    default_font_size_pixels_ = size_pixels;
-    query.pixel_size = size_pixels;
+    size_pixels = pango_font_description_get_size(desc) / kPangoScale;
+    query.pixel_size = std::round(size_pixels);
   } else {
     // Non-absolute sizes are in points (again scaled by PANGO_SIZE).
     // Round the value when converting to pixels to match GTK's logic.
-    const double size_points = pango_font_description_get_size(desc) /
-                               static_cast<double>(PANGO_SCALE);
-    default_font_size_pixels_ =
-        static_cast<int>(kDefaultDPI / 72.0 * size_points + 0.5);
-    query.point_size = static_cast<int>(size_points);
+    const double size_points =
+        pango_font_description_get_size(desc) / kPangoScale;
+    size_pixels = kDefaultDPI / 72.0 * size_points;
+    query.point_size = std::round(size_points);
   }
+  if (!platform_->IncludeFontScaleInDeviceScale()) {
+    size_pixels *= FontScale();
+  }
+  default_font_size_pixels_ = std::round(size_pixels);
 
   query.style = gfx::Font::NORMAL;
   query.weight =
       static_cast<gfx::Font::Weight>(pango_font_description_get_weight(desc));
   // TODO(davemoore): What about PANGO_STYLE_OBLIQUE?
-  if (pango_font_description_get_style(desc) == PANGO_STYLE_ITALIC)
+  if (pango_font_description_get_style(desc) == PANGO_STYLE_ITALIC) {
     query.style |= gfx::Font::ITALIC;
+  }
 
   default_font_render_params_ =
       gfx::GetFontRenderParams(query, &default_font_family_);
   default_font_style_ = query.style;
 }
 
-float GtkUi::GetRawDeviceScaleFactor() {
-  if (display::Display::HasForceDeviceScaleFactor())
-    return display::Display::GetForcedDeviceScaleFactor();
+void GtkUi::TrackMonitor(GdkMonitor* monitor) {
+  auto* callback = G_CALLBACK(OnDeviceScaleFactorMaybeChangedThunk);
+  g_signal_connect_after(monitor, "notify::geometry", callback, this);
+  g_signal_connect_after(monitor, "notify::scale-factor", callback, this);
+}
 
-  float scale = gtk_widget_get_scale_factor(GetDummyWindow());
-  DCHECK_GT(scale, 0.0);
-
-  double resolution = 0;
-  if (GtkCheckVersion(4)) {
-    auto* settings = gtk_settings_get_default();
-    int dpi = 0;
-    g_object_get(settings, "gtk-xft-dpi", &dpi, nullptr);
-    resolution = dpi / 1024.0;
-  } else {
-    GdkScreen* screen = gdk_screen_get_default();
-    resolution = gdk_screen_get_resolution(screen);
+ui::DisplayConfig GtkUi::GetDisplayConfig() const {
+  ui::DisplayConfig config;
+  if (display::Display::HasForceDeviceScaleFactor()) {
+    config.primary_scale = display::Display::GetForcedDeviceScaleFactor();
+    return config;
   }
-  if (resolution > 0)
-    scale *= resolution / kDefaultDPI;
 
-  // Round to the nearest 64th so that UI can losslessly multiply and divide
-  // the scale factor.
-  scale = roundf(scale * 64) / 64;
+  const double font_scale =
+      platform_->IncludeFontScaleInDeviceScale() ? FontScale() : 1.0;
 
-  return scale;
+  GdkDisplay* display = gdk_display_get_default();
+  GdkMonitor* primary = nullptr;
+  std::vector<GdkMonitor*> monitors;
+  if (GtkCheckVersion(4)) {
+    GListModel* list = gdk_display_get_monitors(display);
+    auto n_monitors = g_list_model_get_n_items(list);
+    if (!n_monitors) {
+      return config;
+    }
+    primary = static_cast<GdkMonitor*>(g_list_model_get_item(list, 0));
+    monitors.reserve(n_monitors);
+    for (unsigned int i = 0; i < n_monitors; ++i) {
+      monitors.push_back(
+          static_cast<GdkMonitor*>(g_list_model_get_item(list, i)));
+    }
+  } else {
+    primary = gdk_display_get_primary_monitor(display);
+    const int n_monitors = gdk_display_get_n_monitors(display);
+    monitors.reserve(n_monitors);
+    for (int i = 0; i < n_monitors; i++) {
+      monitors.push_back(gdk_display_get_monitor(display, i));
+    }
+  }
+  if (!primary) {
+    return config;
+  }
+  config.primary_scale =
+      std::max(1, gdk_monitor_get_scale_factor(primary)) * font_scale;
+  config.display_geometries.reserve(monitors.size());
+  for (GdkMonitor* monitor : monitors) {
+    GdkRectangle geometry;
+    gdk_monitor_get_geometry(monitor, &geometry);
+    int monitor_scale = std::max(1, gdk_monitor_get_scale_factor(monitor));
+    config.display_geometries.emplace_back(
+        gfx::Rect(monitor_scale * geometry.x, monitor_scale * geometry.y,
+                  monitor_scale * geometry.width,
+                  monitor_scale * geometry.height),
+        monitor_scale * font_scale);
+  }
+  return config;
 }
 
 void GtkUi::UpdateDeviceScaleFactor() {
-  float old_device_scale_factor = device_scale_factor_;
-  device_scale_factor_ = GetRawDeviceScaleFactor();
-  if (device_scale_factor_ != old_device_scale_factor) {
+  auto new_config = GetDisplayConfig();
+  if (display_config() != new_config) {
+    display_config() = std::move(new_config);
     for (ui::DeviceScaleFactorObserver& observer :
          device_scale_factor_observer_list()) {
       observer.OnDeviceScaleFactorChanged();
@@ -907,8 +976,4 @@ void GtkUi::UpdateDeviceScaleFactor() {
   UpdateDefaultFont();
 }
 
-float GtkUi::GetDeviceScaleFactor() const {
-  return device_scale_factor_;
-}
-
 }  // namespace gtk
