@@ -15,6 +15,7 @@
 #include "base/test/simple_test_tick_clock.h"
 #include "cc/paint/display_item_list.h"
 #include "cc/paint/paint_op_buffer.h"
+#include "cc/paint/paint_op_buffer_iterator.h"
 #include "cc/paint/paint_record.h"
 #include "cc/paint/record_paint_canvas.h"
 #include "cc/paint/skottie_frame_data.h"
@@ -22,6 +23,8 @@
 #include "cc/paint/skottie_resource_metadata.h"
 #include "cc/paint/skottie_wrapper.h"
 #include "cc/test/lottie_test_data.h"
+#include "cc/test/paint_image_matchers.h"
+#include "cc/test/paint_op_matchers.h"
 #include "cc/test/skia_common.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -36,6 +39,8 @@
 namespace lottie {
 namespace {
 
+using ::cc::PaintOpIs;
+using ::testing::AllOf;
 using ::testing::ElementsAre;
 using ::testing::Eq;
 using ::testing::FieldsAre;
@@ -44,8 +49,8 @@ using ::testing::FloatNear;
 using ::testing::IsEmpty;
 using ::testing::NotNull;
 using ::testing::Pair;
+using ::testing::ResultOf;
 using ::testing::SizeIs;
-using ::testing::UnorderedElementsAre;
 
 // A skottie animation with solid green color for the first 2.5 seconds and then
 // a solid blue color for the next 2.5 seconds.
@@ -303,13 +308,6 @@ class AnimationTest : public testing::Test {
 
 class AnimationWithImageAssetsTest : public AnimationTest {
  protected:
-  AnimationWithImageAssetsTest()
-      : display_list_(base::MakeRefCounted<cc::DisplayItemList>(
-            cc::DisplayItemList::kToBeReleasedAsPaintOpBuffer)),
-        record_canvas_(display_list_.get(),
-                       SkRect::MakeIWH(cc::kLottieDataWith2AssetsWidth,
-                                       cc::kLottieDataWith2AssetsHeight)) {}
-
   void SetUp() override {
     canvas_ = std::make_unique<gfx::Canvas>(&record_canvas_, kCanvasImageScale);
     skottie_ = cc::CreateSkottieFromString(cc::kLottieDataWith2Assets);
@@ -327,7 +325,6 @@ class AnimationWithImageAssetsTest : public AnimationTest {
         .quality = cc::PaintFlags::FilterQuality::kHigh};
   }
 
-  const scoped_refptr<cc::DisplayItemList> display_list_;
   cc::RecordPaintCanvas record_canvas_;
   TestSkottieFrameDataProvider frame_data_provider_;
   raw_ptr<TestSkottieFrameDataProvider::ImageAssetImpl> asset_0_;
@@ -1339,32 +1336,31 @@ TEST_F(AnimationWithImageAssetsTest, PaintsAnimationImagesToCanvas) {
   asset_0->set_current_frame_data(frame_0);
   asset_1->set_current_frame_data(frame_1);
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
-  sk_sp<cc::PaintRecord> paint_record = display_list_->ReleaseAsRecord();
-  ASSERT_THAT(paint_record, NotNull());
-  ASSERT_THAT(paint_record->size(), Eq(1u));
-  const cc::DrawSkottieOp* op =
-      paint_record->GetOpAtForTesting<cc::DrawSkottieOp>(0);
-  ASSERT_THAT(op, NotNull());
-  EXPECT_THAT(op->images, UnorderedElementsAre(Pair(
-                              cc::HashSkottieResourceId("image_0"), frame_0)));
+  cc::PaintRecord paint_record = record_canvas_.ReleaseAsRecord();
+  EXPECT_THAT(paint_record,
+              ElementsAre(AllOf(
+                  PaintOpIs<cc::DrawSkottieOp>(),
+                  ResultOf(
+                      [](const cc::PaintOp& op) {
+                        return static_cast<const cc::DrawSkottieOp&>(op).images;
+                      },
+                      ElementsAre(cc::SkottieImageIs("image_0", frame_0))))));
 
   AdvanceClock(animation_->GetAnimationDuration() * .75);
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
-  paint_record = display_list_->ReleaseAsRecord();
-  ASSERT_THAT(paint_record, NotNull());
-  ASSERT_THAT(paint_record->size(), Eq(1u));
-  op = paint_record->GetOpAtForTesting<cc::DrawSkottieOp>(0);
-  ASSERT_THAT(op, NotNull());
-  EXPECT_THAT(op->images, UnorderedElementsAre(Pair(
-                              cc::HashSkottieResourceId("image_1"), frame_1)));
+  paint_record = record_canvas_.ReleaseAsRecord();
+  EXPECT_THAT(paint_record,
+              ElementsAre(AllOf(
+                  PaintOpIs<cc::DrawSkottieOp>(),
+                  ResultOf(
+                      [](const cc::PaintOp& op) {
+                        return static_cast<const cc::DrawSkottieOp&>(op).images;
+                      },
+                      ElementsAre(cc::SkottieImageIs("image_1", frame_1))))));
 }
 
 TEST_F(AnimationWithImageAssetsTest, GracefullyHandlesNullImages) {
@@ -1372,20 +1368,18 @@ TEST_F(AnimationWithImageAssetsTest, GracefullyHandlesNullImages) {
 
   animation_->Start(Animation::PlaybackConfig::CreateWithStyle(
       Animation::Style::kLoop, *animation_));
-
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
-  sk_sp<cc::PaintRecord> paint_record = display_list_->ReleaseAsRecord();
-  ASSERT_THAT(paint_record, NotNull());
-  ASSERT_THAT(paint_record->size(), Eq(1u));
-  const cc::DrawSkottieOp* op =
-      paint_record->GetOpAtForTesting<cc::DrawSkottieOp>(0);
-  ASSERT_THAT(op, NotNull());
-  EXPECT_THAT(op->images,
-              UnorderedElementsAre(Pair(cc::HashSkottieResourceId("image_0"),
-                                        cc::SkottieFrameData())));
+  cc::PaintRecord paint_record = record_canvas_.ReleaseAsRecord();
+  EXPECT_THAT(paint_record,
+              ElementsAre(AllOf(
+                  PaintOpIs<cc::DrawSkottieOp>(),
+                  ResultOf(
+                      [](const cc::PaintOp& op) {
+                        return static_cast<const cc::DrawSkottieOp&>(op).images;
+                      },
+                      ElementsAre(cc::SkottieImageIs(
+                          "image_0", cc::SkottieFrameData()))))));
 }
 
 TEST_F(AnimationWithImageAssetsTest, LoadsCorrectFrameTimestamp) {
@@ -1397,9 +1391,7 @@ TEST_F(AnimationWithImageAssetsTest, LoadsCorrectFrameTimestamp) {
   asset_0_->set_current_frame_data(CreateHighQualityTestFrameData());
   asset_1_->set_current_frame_data(CreateHighQualityTestFrameData());
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
   ASSERT_TRUE(asset_0_->last_frame_t().has_value());
   EXPECT_THAT(asset_0_->last_frame_t().value(), FloatEq(0));
@@ -1408,9 +1400,7 @@ TEST_F(AnimationWithImageAssetsTest, LoadsCorrectFrameTimestamp) {
       animation_->GetAnimationDuration() * .75;
   AdvanceClock(three_quarter_duration);
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
   // The timestamp is "relative to the image layer timeline origin" (see
   // SkResources.h). The test animation used in this case has 2 layers for the
@@ -1425,9 +1415,7 @@ TEST_F(AnimationWithImageAssetsTest, LoadsCorrectFrameTimestamp) {
 
   AdvanceClock(half_duration);
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
   base::TimeDelta quarter_duration = animation_->GetAnimationDuration() / 4;
   ASSERT_TRUE(asset_0_->last_frame_t().has_value());
@@ -1444,9 +1432,7 @@ TEST_F(AnimationWithImageAssetsTest, LoadsCorrectImageScale) {
 
   asset_0_->set_current_frame_data(CreateHighQualityTestFrameData());
 
-  display_list_->StartPaint();
   animation_->Paint(canvas(), NowTicks(), animation_->GetOriginalSize());
-  display_list_->EndPaintOfUnpaired(gfx::Rect(animation_->GetOriginalSize()));
 
   ASSERT_TRUE(asset_0_->last_frame_scale_factor().has_value());
   EXPECT_THAT(asset_0_->last_frame_scale_factor().value(),
@@ -1490,7 +1476,7 @@ class AnimationRestarter : public AnimationObserver {
   }
 
  private:
-  const base::raw_ptr<Animation> animation_;
+  const raw_ptr<Animation> animation_;
   base::ScopedObservation<Animation, AnimationObserver> observation_{this};
 };
 
