@@ -10,9 +10,9 @@
 #include <vector>
 
 #include "base/auto_reset.h"
-#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/containers/contains.h"
+#include "base/functional/bind.h"
 #include "base/observer_list.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/string_util.h"
@@ -34,6 +34,17 @@
 #endif
 
 namespace message_center {
+namespace {
+
+bool IsNotificationsGroupingEnabled() {
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  return true;
+#else
+  return false;
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+}
+
+}  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
 // MessageCenterImpl
@@ -42,12 +53,9 @@ MessageCenterImpl::MessageCenterImpl(
     std::unique_ptr<LockScreenController> lock_screen_controller)
     : lock_screen_controller_(std::move(lock_screen_controller)),
       popup_timers_controller_(std::make_unique<PopupTimersController>(this)),
+      notifications_grouping_enabled_(IsNotificationsGroupingEnabled()),
       stats_collector_(this) {
   notification_list_ = std::make_unique<NotificationList>(this);
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  notifications_grouping_enabled_ =
-      ash::features::IsNotificationsRefreshEnabled();
-#endif
 }
 
 MessageCenterImpl::~MessageCenterImpl() = default;
@@ -64,21 +72,25 @@ void MessageCenterImpl::RemoveObserver(MessageCenterObserver* observer) {
 
 void MessageCenterImpl::AddNotificationBlocker(NotificationBlocker* blocker) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (base::Contains(blockers_, blocker))
+  if (base::Contains(blockers_, blocker)) {
     return;
+  }
 
   blocker->AddObserver(this);
   blockers_.push_back(blocker);
+  OnBlockingStateChanged(blocker);
 }
 
 void MessageCenterImpl::RemoveNotificationBlocker(
     NotificationBlocker* blocker) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   auto iter = base::ranges::find(blockers_, blocker);
-  if (iter == blockers_.end())
+  if (iter == blockers_.end()) {
     return;
+  }
   blocker->RemoveObserver(this);
   blockers_.erase(iter);
+  OnBlockingStateChanged(blocker);
 }
 
 void MessageCenterImpl::OnBlockingStateChanged(NotificationBlocker* blocker) {
@@ -91,11 +103,13 @@ void MessageCenterImpl::OnBlockingStateChanged(NotificationBlocker* blocker) {
       notification_list_->GetVisibleNotifications(blockers_);
 
   for (const std::string& notification_id : blocked) {
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnNotificationUpdated(notification_id);
+    }
   }
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnBlockingStateChanged(blocker);
+  }
 }
 
 void MessageCenterImpl::SetVisibility(Visibility visibility) {
@@ -107,16 +121,19 @@ void MessageCenterImpl::SetVisibility(Visibility visibility) {
     notification_list_->SetNotificationsShown(blockers_, &updated_ids);
 
     for (const auto& id : updated_ids) {
-      for (MessageCenterObserver& observer : observer_list_)
+      for (MessageCenterObserver& observer : observer_list_) {
         observer.OnNotificationUpdated(id);
+      }
     }
 
-    for (Notification* notification : GetPopupNotifications())
+    for (Notification* notification : GetPopupNotifications()) {
       MarkSinglePopupAsShown(notification->id(), false);
+    }
   }
 
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnCenterVisibilityChanged(visibility);
+  }
 }
 
 bool MessageCenterImpl::IsMessageCenterVisible() const {
@@ -124,6 +141,28 @@ bool MessageCenterImpl::IsMessageCenterVisible() const {
   return visible_;
 }
 
+ExpandState MessageCenterImpl::GetNotificationExpandState(
+    const std::string& id) {
+  return notification_list_->GetNotificationExpandState(id);
+}
+
+void MessageCenterImpl::SetNotificationExpandState(
+    const std::string& id,
+    const ExpandState expand_state) {
+  DCHECK(FindVisibleNotificationById(id));
+
+  notification_list_->SetNotificationExpandState(id, expand_state);
+}
+
+void MessageCenterImpl::OnSetExpanded(const std::string& id, bool expanded) {
+  scoped_refptr<NotificationDelegate> delegate =
+      notification_list_->GetNotificationDelegate(id);
+
+  if (delegate) {
+    delegate->ExpandStateChanged(expanded);
+  }
+}
+
 void MessageCenterImpl::SetHasMessageCenterView(bool has_message_center_view) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   has_message_center_view_ = has_message_center_view;
@@ -165,15 +204,21 @@ Notification* MessageCenterImpl::FindParentNotification(
   // the same website for the same user. Also make sure to only group
   // notifications from web pages with valid origin urls. For system
   // notifications, currently we only group privacy indicators notification.
+  // For ARC notifications, only group them when the flag
+  // IsRenderArcNotificationsByChromeEnabled() is enabled.
   bool is_privacy_indicators_notification = false;
+  bool render_arc_notifications_by_chrome = false;
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   is_privacy_indicators_notification =
       notification->notifier_id().id == ash::kPrivacyIndicatorsNotifierId;
+  render_arc_notifications_by_chrome =
+      ash::features::IsRenderArcNotificationsByChromeEnabled();
 #endif
 
   if (!is_privacy_indicators_notification &&
       (notification->origin_url().is_empty() ||
-       notification->notifier_id().type != NotifierType::WEB_PAGE)) {
+       notification->notifier_id().type != NotifierType::WEB_PAGE) &&
+      notification->notifier_id().type != NotifierType::ARC_APPLICATION) {
     return nullptr;
   }
 
@@ -181,9 +226,45 @@ Notification* MessageCenterImpl::FindParentNotification(
       notification_list_->GetNotificationsByNotifierId(
           notification->notifier_id());
 
-  // `notifications` keeps notifications ordered with the most recent one in the
-  // front. If we have notifications for this notifier_id we return the last
-  // notification..
+  // Handle ARC notification grouping in Chrome
+  if (notification->notifier_id().type == NotifierType::ARC_APPLICATION) {
+    // If render_arc_notifications_by_chrome flag is not enabled,
+    // use Android grouping and do not apply grouping rules from the chrome
+    // side.
+    if (!render_arc_notifications_by_chrome) {
+      return nullptr;
+    }
+
+    // To stay consistent with Android, ARC notifications with group key
+    // are grouped using notifier_id() where id and group keys are checked.
+    // For ARC notifications without a group key,
+    // only group them when there are more than 4 notifications
+    if (!notification->notifier_id().group_key.has_value()) {
+      if (notifications.size() < 4) {
+        return nullptr;
+      }
+      for (auto* n : notifications) {
+        if (n->group_parent() || n->group_child()) {
+          continue;
+        }
+        n->SetGroupChild();
+      }
+    }
+  }
+
+  auto parent_notification_it = base::ranges::find_if(
+      notifications,
+      [](Notification* notification) { return notification->group_parent(); });
+
+  // If there's already a notification assigned to be the group parent,
+  // returns that notification immediately.
+  if (parent_notification_it != notifications.cend()) {
+    return *parent_notification_it;
+  }
+
+  // Otherwise, the parent notification should be the oldest one. Since
+  // `notifications` keeps notifications ordered with the most recent one in
+  // the front, the oldest one should be the last in the list.
   return notifications.size() ? *notifications.rbegin() : nullptr;
 }
 
@@ -201,8 +282,9 @@ Notification* MessageCenterImpl::FindVisibleNotificationById(
 
   const auto& notifications = GetVisibleNotifications();
   for (Notification* notification : notifications) {
-    if (notification->id() == id)
+    if (notification->id() == id) {
       return notification;
+    }
   }
 
   return nullptr;
@@ -225,6 +307,13 @@ MessageCenterImpl::GetVisibleNotifications() {
   return visible_notifications_;
 }
 
+NotificationList::Notifications
+MessageCenterImpl::GetVisibleNotificationsWithoutBlocker(
+    const NotificationBlocker* blocker) const {
+  return notification_list_->GetVisibleNotificationsWithoutBlocker(blockers_,
+                                                                   blocker);
+}
+
 NotificationList::PopupNotifications
 MessageCenterImpl::GetPopupNotifications() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
@@ -249,13 +338,8 @@ void MessageCenterImpl::AddNotification(
   notification->set_allow_group(notifications_grouping_enabled_);
 
   const std::string id = notification->id();
-  for (NotificationBlocker* blocker : blockers_)
+  for (NotificationBlocker* blocker : blockers_) {
     blocker->CheckState();
-
-  auto* parent = FindParentNotification(notification.get());
-  if (notification->allow_group() && parent && !notification->group_parent()) {
-    parent->SetGroupParent();
-    notification->SetGroupChild();
   }
 
   // Sometimes the notification can be added with the same id and the
@@ -267,19 +351,39 @@ void MessageCenterImpl::AddNotification(
     return;
   }
 
+  auto* parent = FindParentNotification(notification.get());
+  if (notification->allow_group() && parent && !notification->group_parent()) {
+    parent->SetGroupParent();
+    notification->SetGroupChild();
+  }
+
   notification_list_->AddNotification(std::move(notification));
   visible_notifications_ =
       notification_list_->GetVisibleNotifications(blockers_);
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnNotificationAdded(id);
+  }
 }
 
 void MessageCenterImpl::UpdateNotification(
     const std::string& old_id,
     std::unique_ptr<Notification> new_notification) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  for (NotificationBlocker* blocker : blockers_)
+  for (NotificationBlocker* blocker : blockers_) {
     blocker->CheckState();
+  }
+
+  auto* old_notification = notification_list_->GetNotificationById(old_id);
+  if (old_notification &&
+      old_notification->notifier_id() == new_notification->notifier_id()) {
+    // Copy grouping metadata to the new notification.
+    if (old_notification->group_parent()) {
+      new_notification->SetGroupParent();
+    }
+    if (old_notification->group_child()) {
+      new_notification->SetGroupChild();
+    }
+  }
 
   std::string new_id = new_notification->id();
   notification_list_->UpdateNotificationMessage(old_id,
@@ -301,8 +405,9 @@ void MessageCenterImpl::RemoveNotification(const std::string& id,
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
   Notification* notification = notification_list_->GetNotificationById(id);
-  if (!notification)
+  if (!notification) {
     return;
+  }
 
   if (by_user && notification->pinned()) {
     // When pinned, a popup will not be removed completely but moved into the
@@ -323,13 +428,15 @@ void MessageCenterImpl::RemoveNotification(const std::string& id,
   // RemoveNotification reentrantly.
   notification_list_->RemoveNotification(copied_id);
 
-  if (delegate.get())
+  if (delegate.get()) {
     delegate->Close(by_user);
+  }
 
   visible_notifications_ =
       notification_list_->GetVisibleNotifications(blockers_);
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnNotificationRemoved(copied_id, by_user);
+  }
 }
 
 void MessageCenterImpl::RemoveNotificationsForNotifierId(
@@ -337,8 +444,9 @@ void MessageCenterImpl::RemoveNotificationsForNotifierId(
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   NotificationList::Notifications notifications =
       notification_list_->GetNotificationsByNotifierId(notifier_id);
-  for (Notification* notification : notifications)
+  for (Notification* notification : notifications) {
     RemoveNotification(notification->id(), false);
+  }
   if (!notifications.empty()) {
     visible_notifications_ =
         notification_list_->GetVisibleNotifications(blockers_);
@@ -357,8 +465,9 @@ void MessageCenterImpl::RemoveAllNotifications(bool by_user, RemoveType type) {
       notification_list_->GetVisibleNotifications(blockers);
   std::set<std::string> ids;
   for (Notification* notification : notifications) {
-    if (!remove_pinned && notification->pinned())
+    if (!remove_pinned && notification->pinned()) {
       continue;
+    }
 
     ids.insert(notification->id());
     scoped_refptr<NotificationDelegate> delegate = notification->delegate();
@@ -367,8 +476,9 @@ void MessageCenterImpl::RemoveAllNotifications(bool by_user, RemoveType type) {
     // RemoveNotification reentrantly.
     notification_list_->RemoveNotification(notification->id());
 
-    if (delegate.get())
+    if (delegate.get()) {
       delegate->Close(by_user);
+    }
   }
 
   if (!ids.empty()) {
@@ -376,8 +486,9 @@ void MessageCenterImpl::RemoveAllNotifications(bool by_user, RemoveType type) {
         notification_list_->GetVisibleNotifications(blockers_);
   }
   for (const auto& id : ids) {
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnNotificationRemoved(id, by_user);
+    }
   }
 }
 
@@ -386,8 +497,9 @@ void MessageCenterImpl::SetNotificationIcon(const std::string& notification_id,
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
   if (notification_list_->SetNotificationIcon(notification_id, image)) {
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnNotificationUpdated(notification_id);
+    }
   }
 }
 
@@ -395,15 +507,17 @@ void MessageCenterImpl::SetNotificationImage(const std::string& notification_id,
                                              const gfx::Image& image) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   if (notification_list_->SetNotificationImage(notification_id, image)) {
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnNotificationUpdated(notification_id);
+    }
   }
 }
 
 void MessageCenterImpl::ClickOnNotification(const std::string& id) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (!FindVisibleNotificationById(id))
+  if (!FindVisibleNotificationById(id)) {
     return;
+  }
 
   lock_screen_controller_->DismissLockScreenThenExecute(
       base::BindOnce(&MessageCenterImpl::ClickOnNotificationUnlocked,
@@ -414,8 +528,9 @@ void MessageCenterImpl::ClickOnNotification(const std::string& id) {
 void MessageCenterImpl::ClickOnNotificationButton(const std::string& id,
                                                   int button_index) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (!FindVisibleNotificationById(id))
+  if (!FindVisibleNotificationById(id)) {
     return;
+  }
 
   lock_screen_controller_->DismissLockScreenThenExecute(
       base::BindOnce(&MessageCenterImpl::ClickOnNotificationUnlocked,
@@ -428,8 +543,9 @@ void MessageCenterImpl::ClickOnNotificationButtonWithReply(
     int button_index,
     const std::u16string& reply) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (!FindVisibleNotificationById(id))
+  if (!FindVisibleNotificationById(id)) {
     return;
+  }
 
   lock_screen_controller_->DismissLockScreenThenExecute(
       base::BindOnce(&MessageCenterImpl::ClickOnNotificationUnlocked,
@@ -447,18 +563,28 @@ void MessageCenterImpl::ClickOnNotificationUnlocked(
   DCHECK(!lock_screen_controller_->IsScreenLocked());
 
   // Ensure the notification is still visible.
-  if (!FindVisibleNotificationById(id))
+  if (!FindVisibleNotificationById(id)) {
     return;
+  }
 
-  if (HasMessageCenterView() && HasPopupNotifications())
+  if (HasMessageCenterView() && HasPopupNotifications()) {
     MarkSinglePopupAsShown(id, true);
-  for (MessageCenterObserver& observer : observer_list_)
+  }
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnNotificationClicked(id, button_index, reply);
+  }
 
   scoped_refptr<NotificationDelegate> delegate =
       notification_list_->GetNotificationDelegate(id);
-  if (delegate)
+  if (delegate) {
     delegate->Click(button_index, reply);
+  }
+
+  if (const Notification* notification =
+          notification_list_->GetNotificationById(id);
+      notification && notification->rich_notification_data().remove_on_click) {
+    RemoveNotification(id, /*by_user=*/true);
+  }
 }
 
 void MessageCenterImpl::ClickOnSettingsButton(const std::string& id) {
@@ -469,11 +595,24 @@ void MessageCenterImpl::ClickOnSettingsButton(const std::string& id) {
       notification &&
       (notification->rich_notification_data().settings_button_handler ==
        SettingsButtonHandler::DELEGATE);
-  if (handled_by_delegate)
+  if (handled_by_delegate) {
     notification->delegate()->SettingsClick();
+  }
 
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnNotificationSettingsClicked(handled_by_delegate);
+  }
+}
+
+void MessageCenterImpl::ClickOnSnoozeButton(const std::string& id) {
+  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  Notification* notification = notification_list_->GetNotificationById(id);
+
+  bool handled_by_delegate =
+      notification && notification_list_->GetNotificationDelegate(id);
+  if (handled_by_delegate) {
+    notification->delegate()->SnoozeButtonClicked();
+  }
 }
 
 void MessageCenterImpl::DisableNotification(const std::string& id) {
@@ -489,8 +628,9 @@ void MessageCenterImpl::DisableNotification(const std::string& id) {
 void MessageCenterImpl::MarkSinglePopupAsShown(const std::string& id,
                                                bool mark_notification_as_read) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (!FindVisibleNotificationById(id))
+  if (!FindNotificationById(id)) {
     return;
+  }
 
   if (HasMessageCenterView()) {
     notification_list_->MarkSinglePopupAsShown(id, mark_notification_as_read);
@@ -526,23 +666,27 @@ void MessageCenterImpl::DisplayedNotification(const std::string& id,
   // This method may be called from the handlers, so we shouldn't manipulate
   // notifications in this method.
 
-  if (!FindVisibleNotificationById(id))
+  if (!FindVisibleNotificationById(id)) {
     return;
+  }
 
-  if (HasPopupNotifications())
+  if (HasPopupNotifications()) {
     notification_list_->MarkSinglePopupAsDisplayed(id);
+  }
   scoped_refptr<NotificationDelegate> delegate =
       notification_list_->GetNotificationDelegate(id);
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnNotificationDisplayed(id, source);
+  }
 }
 
 void MessageCenterImpl::SetQuietMode(bool in_quiet_mode) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   if (in_quiet_mode != notification_list_->quiet_mode()) {
     notification_list_->SetQuietMode(in_quiet_mode);
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnQuietModeChanged(in_quiet_mode);
+    }
   }
   quiet_mode_timer_.Stop();
 }
@@ -557,8 +701,9 @@ void MessageCenterImpl::EnterQuietModeWithExpire(
 
   if (!quiet_mode_timer_.IsRunning()) {
     notification_list_->SetQuietMode(true);
-    for (MessageCenterObserver& observer : observer_list_)
+    for (MessageCenterObserver& observer : observer_list_) {
       observer.OnQuietModeChanged(true);
+    }
   }
 
   // This will restart the timer if it is already running.
@@ -569,14 +714,16 @@ void MessageCenterImpl::EnterQuietModeWithExpire(
 
 void MessageCenterImpl::RestartPopupTimers() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (popup_timers_controller_)
+  if (popup_timers_controller_) {
     popup_timers_controller_->StartAll();
+  }
 }
 
 void MessageCenterImpl::PausePopupTimers() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  if (popup_timers_controller_)
+  if (popup_timers_controller_) {
     popup_timers_controller_->PauseAll();
+  }
 }
 
 const std::u16string& MessageCenterImpl::GetSystemNotificationAppName() const {
@@ -590,8 +737,9 @@ void MessageCenterImpl::SetSystemNotificationAppName(
 
 void MessageCenterImpl::OnMessageViewHovered(
     const std::string& notification_id) {
-  for (MessageCenterObserver& observer : observer_list_)
+  for (MessageCenterObserver& observer : observer_list_) {
     observer.OnMessageViewHovered(notification_id);
+  }
 }
 
 void MessageCenterImpl::DisableTimersForTest() {
