@@ -12,7 +12,7 @@ namespace message_center {
 enum NotificationType {
   NOTIFICATION_TYPE_SIMPLE = 0,
   DEPRECATED_NOTIFICATION_TYPE_BASE_FORMAT =
-      1,  // Use NOTIFICATION_TYPE_SIMPLE instead.
+      1,  // Use `NOTIFICATION_TYPE_SIMPLE` instead.
   NOTIFICATION_TYPE_IMAGE = 2,
   NOTIFICATION_TYPE_MULTIPLE = 3,
   NOTIFICATION_TYPE_PROGRESS = 4,  // Notification with progress bar.
@@ -25,14 +25,16 @@ enum NotificationType {
 enum NotificationPriority {
   MIN_PRIORITY = -2,
   LOW_PRIORITY = -1,
+  // In ChromeOS, if priority < `DEFAULT_PRIORITY`, the notification will be
+  // silently added to the tray (no pop-up will be shown).
   DEFAULT_PRIORITY = 0,
-  // Priorities > |DEFAULT_PRIORITY| have the capability to wake the display up
+  // Priorities > `DEFAULT_PRIORITY` have the capability to wake the display up
   // if it was off.
   HIGH_PRIORITY = 1,
   MAX_PRIORITY = 2,
 
   // Top priority for system-level notifications.. This can't be set from
-  // kPriorityKey, instead you have to call SetSystemPriority() of
+  // `kPriorityKey`, instead you have to call `SetSystemPriority()` of
   // Notification object.
   SYSTEM_PRIORITY = 3,
 };
