@@ -39,42 +39,13 @@
 #include "base/time/time.h"
 #endif
 
-#if BUILDFLAG(IS_WIN)
-#include "ui/base/win/shell.h"
-#endif
-
 namespace message_center {
 
 namespace {
 
-// Creates a text for spoken feedback from the data contained in the
-// notification.
-std::u16string CreateAccessibleName(const Notification& notification) {
-  if (!notification.accessible_name().empty())
-    return notification.accessible_name();
-
-  // Fall back to a text constructed from the notification.
-  // Add non-empty elements.
-
-  std::vector<std::u16string> accessible_lines;
-  if (!notification.title().empty())
-    accessible_lines.push_back(notification.title());
-
-  if (!notification.message().empty())
-    accessible_lines.push_back(notification.message());
-
-  if (!notification.context_message().empty())
-    accessible_lines.push_back(notification.context_message());
-  std::vector<NotificationItem> items = notification.items();
-  for (size_t i = 0; i < items.size() && i < kNotificationMaximumItems; ++i) {
-    accessible_lines.push_back(items[i].title + u" " + items[i].message);
-  }
-  return base::JoinString(accessible_lines, u"\n");
-}
-
 bool ShouldShowAeroShadowBorder() {
 #if BUILDFLAG(IS_WIN)
-  return ui::win::IsAeroGlassEnabled();
+  return true;
 #else
   return false;
 #endif
@@ -115,7 +86,7 @@ MessageView::MessageView(const Notification& notification)
     const auto& shadow = gfx::ShadowDetails::Get(2, 0);
     gfx::Insets ninebox_insets = gfx::ShadowValue::GetBlurRegion(shadow.values);
     SetBorder(views::CreateBorderPainter(
-        views::Painter::CreateImagePainter(shadow.ninebox_image,
+        views::Painter::CreateImagePainter(shadow.nine_patch_image,
                                            ninebox_insets),
         -gfx::ShadowValue::GetMargin(shadow.values)));
   }
@@ -131,13 +102,35 @@ views::View* MessageView::FindGroupNotificationView(
   return nullptr;
 }
 
+// Creates text for spoken feedback from the data contained in the
+// notification.
+std::u16string MessageView::CreateAccessibleName(
+    const Notification& notification) {
+  if (!notification.accessible_name().empty())
+    return notification.accessible_name();
+
+  // Fall back to text constructed from the notification.
+  // Add non-empty elements.
+
+  std::vector<std::u16string> accessible_lines;
+  if (!notification.title().empty())
+    accessible_lines.push_back(notification.title());
+
+  if (!notification.message().empty())
+    accessible_lines.push_back(notification.message());
+
+  if (!notification.context_message().empty())
+    accessible_lines.push_back(notification.context_message());
+  std::vector<NotificationItem> items = notification.items();
+  for (size_t i = 0; i < items.size() && i < kNotificationMaximumItems; ++i) {
+    accessible_lines.push_back(items[i].title + u" " + items[i].message);
+  }
+  return base::JoinString(accessible_lines, u"\n");
+}
+
 void MessageView::UpdateWithNotification(const Notification& notification) {
   pinned_ = notification.pinned();
-  std::u16string new_accessible_name = CreateAccessibleName(notification);
-  if (new_accessible_name != accessible_name_) {
-    accessible_name_ = new_accessible_name;
-    NotifyAccessibilityEvent(ax::mojom::Event::kTextChanged, true);
-  }
+  SetAccessibleName(CreateAccessibleName(notification));
   slide_out_controller_.set_slide_mode(CalculateSlideMode());
 }
 
@@ -168,7 +161,7 @@ void MessageView::SlideOutAndClose(int direction) {
 }
 
 void MessageView::SetExpanded(bool expanded) {
-  // Not implemented by default.
+  MessageCenter::Get()->OnSetExpanded(notification_id_, expanded);
 }
 
 bool MessageView::IsExpanded() const {
@@ -186,7 +179,7 @@ bool MessageView::IsManuallyExpandedOrCollapsed() const {
   return false;
 }
 
-void MessageView::SetManuallyExpandedOrCollapsed(bool value) {
+void MessageView::SetManuallyExpandedOrCollapsed(ExpandState state) {
   // Not implemented by default.
 }
 
@@ -231,10 +224,11 @@ void MessageView::GetAccessibleNodeData(ui::AXNodeData* node_data) {
       ax::mojom::StringAttribute::kRoleDescription,
       l10n_util::GetStringUTF8(IDS_MESSAGE_NOTIFICATION_ACCESSIBLE_NAME));
 
-  if (accessible_name_.empty())
+  if (GetAccessibleName().empty()) {
     node_data->SetNameFrom(ax::mojom::NameFrom::kAttributeExplicitlyEmpty);
+  }
 
-  node_data->SetName(accessible_name_);
+  node_data->SetNameChecked(GetAccessibleName());
 }
 
 bool MessageView::OnMousePressed(const ui::MouseEvent& event) {
@@ -358,11 +352,10 @@ ui::Layer* MessageView::GetSlideOutLayer() {
   // If a message view is contained in a parent message view it should give up
   // slide behavior to the parent message view when the parent view is
   // collapsed.
-  auto* nested_layer =
-      (parent_message_view_ && !parent_message_view_->IsExpanded())
-          ? parent_message_view_->layer()
-          : layer();
-  bool is_nested = (parent_message_view_ && !parent_message_view_->IsExpanded())
+  auto* nested_layer = (ShouldParentHandleSlide() && parent_message_view_)
+                           ? parent_message_view_->layer()
+                           : layer();
+  bool is_nested = (ShouldParentHandleSlide() && parent_message_view_)
                        ? parent_message_view_->is_nested()
                        : is_nested_;
   return is_nested ? nested_layer : GetWidget()->GetLayer();
@@ -407,6 +400,9 @@ void MessageView::RemoveObserver(MessageView::Observer* observer) {
 }
 
 void MessageView::OnSlideOut() {
+  if (ShouldParentHandleSlide() && parent_message_view_)
+    return parent_message_view_->OnSlideOut();
+
   // The notification will be deleted after slide out, so give observers a
   // chance to handle the notification before fulling sliding out.
   for (auto& observer : observers_)
@@ -464,6 +460,13 @@ MessageView::Mode MessageView::GetMode() const {
 }
 
 float MessageView::GetSlideAmount() const {
+  if (slide_out_controller_.mode() ==
+      views::SlideOutController::SlideMode::kNone) {
+    // The return value of this method is used by NotificationSwipeControlView
+    // to determine visibility of the setting button. Returning 0 not to show
+    // the setting button with SlideMode::kNone.
+    return 0.f;
+  }
   return slide_out_controller_.gesture_amount();
 }
 
@@ -504,6 +507,8 @@ void MessageView::OnSettingsButtonPressed(const ui::Event& event) {
 void MessageView::OnSnoozeButtonPressed(const ui::Event& event) {
   for (auto& observer : observers_)
     observer.OnSnoozeButtonPressed(notification_id_);
+
+  MessageCenter::Get()->ClickOnSnoozeButton(notification_id());
 }
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -528,6 +533,13 @@ bool MessageView::ShouldShowControlButtons() const {
 #endif
 }
 
+bool MessageView::ShouldParentHandleSlide() const {
+  if (!parent_message_view_)
+    return false;
+
+  return !parent_message_view_->IsExpanded();
+}
+
 void MessageView::UpdateBackgroundPainter() {
   const auto* color_provider = GetColorProvider();
   SkColor background_color = color_provider->GetColor(
@@ -542,12 +554,13 @@ void MessageView::UpdateBackgroundPainter() {
 void MessageView::UpdateNestedBorder() {
   if (!is_nested_ || !GetWidget())
     return;
-  SkColor border_color =
-      GetColorProvider()->GetColor(ui::kColorFocusableBorderUnfocused);
 
+  SkColor border_color;
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-  if (ash::features::IsNotificationsRefreshEnabled())
-    border_color = SK_ColorTRANSPARENT;
+  border_color = SK_ColorTRANSPARENT;
+#else
+  border_color =
+      GetColorProvider()->GetColor(ui::kColorFocusableBorderUnfocused);
 #endif
 
   SetBorder(views::CreateRoundedRectBorder(
