@@ -4,6 +4,8 @@
 
 #include "ui/ozone/platform/wayland/host/wayland_data_drag_controller.h"
 
+#include <viewporter-client-protocol.h>
+
 #include <bitset>
 #include <cstdint>
 #include <memory>
@@ -11,8 +13,6 @@
 #include "base/check.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/notreached.h"
-#include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/clipboard/clipboard_constants.h"
 #include "ui/base/dragdrop/drag_drop_types.h"
 #include "ui/base/dragdrop/mojom/drag_drop_types.mojom.h"
@@ -21,7 +21,9 @@
 #include "ui/events/event_constants.h"
 #include "ui/events/platform/platform_event_source.h"
 #include "ui/events/platform/scoped_event_dispatcher.h"
+#include "ui/gfx/image/image_skia_rep.h"
 #include "ui/ozone/platform/wayland/common/wayland_util.h"
+#include "ui/ozone/platform/wayland/host/dump_util.h"
 #include "ui/ozone/platform/wayland/host/wayland_connection.h"
 #include "ui/ozone/platform/wayland/host/wayland_data_device_manager.h"
 #include "ui/ozone/platform/wayland/host/wayland_data_offer.h"
@@ -59,27 +61,29 @@ DragOperation DndActionToDragOperation(uint32_t action) {
 
 int DndActionsToDragOperations(uint32_t actions) {
   int operations = DragDropTypes::DRAG_NONE;
-  if (actions & WL_DATA_DEVICE_MANAGER_DND_ACTION_COPY)
+  if (actions & WL_DATA_DEVICE_MANAGER_DND_ACTION_COPY) {
     operations |= DragDropTypes::DRAG_COPY;
-  if (actions & WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE)
+  }
+  if (actions & WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE) {
     operations |= DragDropTypes::DRAG_MOVE;
+  }
+  if (actions & WL_DATA_DEVICE_MANAGER_DND_ACTION_ASK) {
+    operations |= DragDropTypes::DRAG_LINK;
+  }
   return operations;
 }
 
 uint32_t DragOperationsToDndActions(int operations) {
   uint32_t dnd_actions = WL_DATA_DEVICE_MANAGER_DND_ACTION_NONE;
-  if (operations & DragDropTypes::DRAG_COPY)
+  if (operations & DragDropTypes::DRAG_COPY) {
     dnd_actions |= WL_DATA_DEVICE_MANAGER_DND_ACTION_COPY;
-  if (operations & DragDropTypes::DRAG_MOVE)
+  }
+  if (operations & DragDropTypes::DRAG_MOVE) {
     dnd_actions |= WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE;
+  }
   return dnd_actions;
 }
 
-const SkBitmap* GetDragImage(const OSExchangeData& data) {
-  const SkBitmap* icon_bitmap = data.provider().GetDragImage().bitmap();
-  return icon_bitmap && !icon_bitmap->empty() ? icon_bitmap : nullptr;
-}
-
 }  // namespace
 
 WaylandDataDragController::WaylandDataDragController(
@@ -90,7 +94,7 @@ WaylandDataDragController::WaylandDataDragController(
     : connection_(connection),
       data_device_manager_(data_device_manager),
       data_device_(data_device_manager->GetDevice()),
-      window_manager_(connection->wayland_window_manager()),
+      window_manager_(connection->window_manager()),
       pointer_delegate_(pointer_delegate),
       touch_delegate_(touch_delegate) {
   DCHECK(connection_);
@@ -108,10 +112,9 @@ bool WaylandDataDragController::StartSession(const OSExchangeData& data,
   DCHECK(!origin_window_);
   DCHECK(!icon_surface_);
 
-  WaylandWindow* origin_window =
-      source == DragEventSource::kTouch
-          ? window_manager_->GetCurrentTouchFocusedWindow()
-          : window_manager_->GetCurrentPointerFocusedWindow();
+  auto* origin_window = source == DragEventSource::kTouch
+                            ? window_manager_->GetCurrentTouchFocusedWindow()
+                            : window_manager_->GetCurrentPointerFocusedWindow();
   if (!origin_window) {
     LOG(ERROR) << "Failed to get focused window. source=" << source;
     return false;
@@ -121,7 +124,7 @@ bool WaylandDataDragController::StartSession(const OSExchangeData& data,
   // by the time "start drag" gets processed by Ozone/Wayland, the origin
   // pointer event (touch or mouse) has already been released. In this case,
   // make sure the flow bails earlier, otherwise the drag loop keeps running,
-  // causing hangs as observerd in crbug.com/1209269.
+  // causing hangs as observed in crbug.com/1209269.
   auto serial = GetAndValidateSerialForDrag(source);
   if (!serial.has_value()) {
     LOG(ERROR) << "Invalid state when trying to start drag. source=" << source;
@@ -135,32 +138,33 @@ bool WaylandDataDragController::StartSession(const OSExchangeData& data,
   data_source_->SetDndActions(DragOperationsToDndActions(operations));
 
   // Create drag icon surface (if any) and store the data to be exchanged.
-  icon_bitmap_ = GetDragImage(data);
-  if (icon_bitmap_) {
+  icon_image_ = data.provider().GetDragImage();
+  if (!icon_image_.isNull()) {
     icon_surface_ = std::make_unique<WaylandSurface>(connection_, nullptr);
     if (icon_surface_->Initialize()) {
-      // Corresponds to actual scale factor of the origin surface.
-      icon_surface_->set_surface_buffer_scale(origin_window->window_scale());
+      // Corresponds to actual scale factor of the origin surface. Use the
+      // latched state as that is what is currently displayed to the user and
+      // used as buffers in these surfaces.
+      icon_surface_buffer_scale_ = origin_window->applied_state().window_scale;
+      icon_surface_->set_surface_buffer_scale(icon_surface_buffer_scale_);
       // Icon surface do not need input.
       const gfx::Rect empty_region_px;
-      icon_surface_->set_input_region(&empty_region_px);
+      icon_surface_->set_input_region(empty_region_px);
       icon_surface_->ApplyPendingState();
 
       auto icon_offset = -data.provider().GetDragImageOffset();
-      icon_offset_ =
-          gfx::ScaleToRoundedPoint({icon_offset.x(), icon_offset.y()},
-                                   1.0f / origin_window->window_scale());
+      pending_icon_offset_ = {icon_offset.x(), icon_offset.y()};
+      current_icon_offset_ = {0, 0};
     } else {
       LOG(ERROR) << "Failed to create drag icon surface.";
       icon_surface_.reset();
+      icon_surface_buffer_scale_ = 1.0f;
     }
   }
 
   // Starts the wayland drag session setting |this| object as delegate.
   state_ = State::kStarted;
-  drag_source_ = serial->type == wl::SerialType::kTouchPress
-                     ? DragSource::kTouch
-                     : DragSource::kMouse;
+  drag_source_ = source;
   data_device_->StartDrag(*data_source_, *origin_window, serial->value,
                           icon_surface_ ? icon_surface_->surface() : nullptr,
                           this);
@@ -176,6 +180,21 @@ bool WaylandDataDragController::StartSession(const OSExchangeData& data,
   return true;
 }
 
+void WaylandDataDragController::UpdateDragImage(const gfx::ImageSkia& image,
+                                                const gfx::Vector2d& offset) {
+  icon_image_ = image;
+
+  if (icon_surface_ && window_) {
+    icon_surface_buffer_scale_ = window_->applied_state().window_scale;
+    icon_surface_->set_surface_buffer_scale(icon_surface_buffer_scale_);
+    icon_surface_->ApplyPendingState();
+  }
+
+  pending_icon_offset_ = {-offset.x(), -offset.y()};
+
+  DrawIconInternal();
+}
+
 bool WaylandDataDragController::ShouldReleaseCaptureForDrag(
     ui::OSExchangeData* data) const {
   DCHECK(data);
@@ -184,6 +203,25 @@ bool WaylandDataDragController::ShouldReleaseCaptureForDrag(
   return !IsWindowDraggingSession(*data);
 }
 
+void WaylandDataDragController::DumpState(std::ostream& out) const {
+  constexpr auto kStateToString = base::MakeFixedFlatMap<State, const char*>(
+      {{State::kIdle, "idle"},
+       {State::kStarted, "started"},
+       {State::kTransferring, "transferring"}});
+  out << "WaylandDataDragController: state="
+      << GetMapValueOrDefault(kStateToString, state_)
+      << ", drag_source=" << !!drag_source_
+      << ", data_source=" << !!data_source_
+      << ", origin_window=" << GetWindowName(origin_window_)
+      << ", current_window=" << GetWindowName(window_)
+      << ", unprocessed_mime_types=" << ListToString(unprocessed_mime_types_)
+      << ", last_drag_location=" << last_drag_location_.ToString()
+      << ", is_leave_pending=" << is_leave_pending_
+      << ", icon_surface_bufer_scale=" << icon_surface_buffer_scale_
+      << ", pending_icon_offset=" << pending_icon_offset_.ToString()
+      << ", current_icon_offset=" << current_icon_offset_.ToString();
+}
+
 // Sessions initiated from Chromium, will have |data_source_| set. In which
 // case, |offered_exchange_data_provider_| is expected to be non-null as well.
 bool WaylandDataDragController::IsDragSource() const {
@@ -192,8 +230,9 @@ bool WaylandDataDragController::IsDragSource() const {
 }
 
 void WaylandDataDragController::DrawIcon() {
-  if (!icon_surface_ || !icon_bitmap_)
+  if (!icon_surface_ || icon_image_.isNull()) {
     return;
+  }
 
   static const wl_callback_listener kFrameListener{
       .done = WaylandDataDragController::OnDragSurfaceFrame};
@@ -220,27 +259,62 @@ void WaylandDataDragController::OnDragSurfaceFrame(void* data,
   self->connection_->Flush();
 }
 
-void WaylandDataDragController::DrawIconInternal() {
-  if (!icon_surface_ || !icon_bitmap_)
-    return;
+SkBitmap WaylandDataDragController::GetIconBitmap() {
+  return icon_image_.GetRepresentation(icon_surface_buffer_scale_).GetBitmap();
+}
 
-  DCHECK(!icon_bitmap_->empty());
-  gfx::Size size(icon_bitmap_->width(), icon_bitmap_->height());
+void WaylandDataDragController::DrawIconInternal() {
+  // If there was a drag icon before but now there isn't, attach a null buffer
+  // to the icon surface to hide it.
+  if (icon_surface_ && icon_image_.isNull()) {
+    auto* const surface = icon_surface_->surface();
+    wl_surface_attach(surface, nullptr, 0, 0);
+    wl_surface_commit(surface);
+  }
+
+  if (!icon_surface_ || icon_image_.isNull()) {
+    return;
+  }
+
+  auto icon_bitmap = GetIconBitmap();
+  CHECK(!icon_bitmap.drawsNothing());
+  // The protocol expects the attached buffer to have a pixel size that is a
+  // multiple of the surface's scale factor. Some compositors (eg. Wlroots) will
+  // refuse to attach the buffer if this condition is not met.
+  const gfx::Size size_dip =
+      gfx::ScaleToCeiledSize({icon_bitmap.width(), icon_bitmap.height()},
+                             1.0f / icon_surface_buffer_scale_);
+  const gfx::Size size_px =
+      gfx::ScaleToCeiledSize(size_dip, icon_surface_buffer_scale_);
 
   icon_buffer_ = std::make_unique<WaylandShmBuffer>(
-      connection_->wayland_buffer_factory(), size);
+      connection_->buffer_factory(), size_px);
   if (!icon_buffer_->IsValid()) {
     LOG(ERROR) << "Failed to create drag icon buffer.";
     return;
   }
 
-  DVLOG(3) << "Drawing drag icon. size=" << size.ToString();
-  wl::DrawBitmap(*icon_bitmap_, icon_buffer_.get());
+  DVLOG(3) << "Drawing drag icon. size_px=" << size_px.ToString();
+  wl::DrawBitmap(icon_bitmap, icon_buffer_.get());
   auto* const surface = icon_surface_->surface();
-  wl_surface_attach(surface, icon_buffer_->get(), icon_offset_.x(),
-                    icon_offset_.y());
-  wl_surface_damage(surface, 0, 0, size.width(), size.height());
+  if (wl::get_version_of_object(surface) < WL_SURFACE_OFFSET_SINCE_VERSION) {
+    wl_surface_attach(surface, icon_buffer_->get(),
+                      pending_icon_offset_.x() - current_icon_offset_.x(),
+                      pending_icon_offset_.y() - current_icon_offset_.y());
+  } else {
+    wl_surface_attach(surface, icon_buffer_->get(), 0, 0);
+    wl_surface_offset(surface,
+                      pending_icon_offset_.x() - current_icon_offset_.x(),
+                      pending_icon_offset_.y() - current_icon_offset_.y());
+  }
+  if (connection_->UseViewporterSurfaceScaling() && icon_surface_->viewport()) {
+    wp_viewport_set_destination(icon_surface_->viewport(), size_dip.width(),
+                                size_dip.height());
+  }
+  wl_surface_damage(surface, 0, 0, size_px.width(), size_px.height());
   wl_surface_commit(surface);
+
+  current_icon_offset_ = pending_icon_offset_;
 }
 
 void WaylandDataDragController::OnDragOffer(
@@ -262,9 +336,11 @@ void WaylandDataDragController::OnDragEnter(WaylandWindow* window,
     data_offer_->Accept(serial, mime);
   }
 
+  // Update the focused window to ensure the window under the cursor receives
+  // drag motion events.
   if (pointer_grabber_for_window_drag_) {
     DCHECK(drag_source_.has_value());
-    if (*drag_source_ == DragSource::kMouse) {
+    if (*drag_source_ == DragEventSource::kMouse) {
       pointer_delegate_->OnPointerFocusChanged(
           window, location, wl::EventDispatchPolicy::kImmediate);
     } else {
@@ -286,7 +362,7 @@ void WaylandDataDragController::OnDragEnter(WaylandWindow* window,
                              offered_exchange_data_provider_->Clone()));
   } else {
     // Otherwise, we are about to accept data dragged from another application.
-    // Reading the data may take some time so set |state_| to |kTrasferring|,
+    // Reading the data may take some time so set |state_| to |kTransferring|,
     // which will defer sending OnDragEnter to the client until the data
     // is ready.
     state_ = State::kTransferring;
@@ -298,8 +374,9 @@ void WaylandDataDragController::OnDragEnter(WaylandWindow* window,
 }
 
 void WaylandDataDragController::OnDragMotion(const gfx::PointF& location) {
-  if (!window_)
+  if (!window_) {
     return;
+  }
 
   if (state_ == State::kTransferring) {
     last_drag_location_ = location;
@@ -320,8 +397,9 @@ void WaylandDataDragController::OnDragLeave() {
     return;
   }
 
-  if (window_)
+  if (window_) {
     window_->OnDragLeave();
+  }
 
   window_ = nullptr;
   data_offer_.reset();
@@ -329,8 +407,9 @@ void WaylandDataDragController::OnDragLeave() {
 }
 
 void WaylandDataDragController::OnDragDrop() {
-  if (!window_)
+  if (!window_) {
     return;
+  }
 
   window_->OnDragDrop();
 
@@ -350,8 +429,9 @@ void WaylandDataDragController::OnDataSourceFinish(bool completed) {
                   : DragOperation::kNone);
     // DnD handlers expect DragLeave to be sent for drag sessions that end up
     // with no data transfer (wl_data_source::cancelled event).
-    if (!completed)
+    if (!completed) {
       origin_window_->OnDragLeave();
+    }
     origin_window_ = nullptr;
   }
 
@@ -362,15 +442,17 @@ void WaylandDataDragController::OnDataSourceFinish(bool completed) {
   // Dispatch this after calling WaylandWindow::OnDragSessionClose(), else the
   // extra leave event that is dispatched if |completed| is false may cause
   // problems.
-  if (pointer_grabber_for_window_drag_)
+  if (pointer_grabber_for_window_drag_) {
     DispatchPointerRelease();
+  }
 
   window_manager_->RemoveObserver(this);
   data_source_.reset();
   data_offer_.reset();
   icon_buffer_.reset();
   icon_surface_.reset();
-  icon_bitmap_ = nullptr;
+  icon_surface_buffer_scale_ = 1.0f;
+  icon_image_ = gfx::ImageSkia();
   icon_frame_callback_.reset();
   offered_exchange_data_provider_.reset();
   data_device_->ResetDragDelegate();
@@ -392,14 +474,17 @@ void WaylandDataDragController::OnDataSourceSend(const std::string& mime_type,
 }
 
 void WaylandDataDragController::OnWindowRemoved(WaylandWindow* window) {
-  if (window == window_)
+  if (window == window_) {
     window_ = nullptr;
+  }
 
-  if (window == origin_window_)
+  if (window == origin_window_) {
     origin_window_ = nullptr;
+  }
 
-  if (window == pointer_grabber_for_window_drag_)
+  if (window == pointer_grabber_for_window_drag_) {
     pointer_grabber_for_window_drag_ = nullptr;
+  }
 }
 
 // Asynchronously requests and reads data for every negotiated/supported mime
@@ -442,8 +527,9 @@ void WaylandDataDragController::OnDataTransferFinished(
     base::TimeTicks start_time,
     std::unique_ptr<OSExchangeData> received_data) {
   unprocessed_mime_types_.clear();
-  if (state_ == State::kIdle)
+  if (state_ == State::kIdle) {
     return;
+  }
 
   state_ = State::kIdle;
 
@@ -457,7 +543,7 @@ void WaylandDataDragController::OnDataTransferFinished(
       data_offer_.reset();
     }
     offered_exchange_data_provider_.reset();
-    data_device_->ResetDragDelegate();
+    data_device_->ResetDragDelegateIfNotDragSource();
     is_leave_pending_ = false;
     return;
   }
@@ -486,12 +572,17 @@ std::string WaylandDataDragController::GetNextUnprocessedMimeType() {
 void WaylandDataDragController::PropagateOnDragEnter(
     const gfx::PointF& location,
     std::unique_ptr<OSExchangeData> data) {
-  DCHECK(window_);
-  {
-    window_->OnDragEnter(
-        location, std::move(data),
-        DndActionsToDragOperations(data_offer_->source_actions()));
+  // |data_offer_| may have already been destroyed at this point if, for
+  // example, the drop event comes in while the data fetching was ongoing and no
+  // subsequent 'leave' is received, so just early-out in this case.
+  if (!data_offer_) {
+    return;
   }
+
+  DCHECK(window_);
+  window_->OnDragEnter(
+      location, std::move(data),
+      DndActionsToDragOperations(data_offer_->source_actions()));
   OnDragMotion(location);
 }
 
@@ -535,8 +626,9 @@ bool WaylandDataDragController::IsWindowDraggingSession(
 
 void WaylandDataDragController::SetUpWindowDraggingSessionIfNeeded(
     const ui::OSExchangeData& data) {
-  if (!IsWindowDraggingSession(data))
+  if (!IsWindowDraggingSession(data)) {
     return;
+  }
 
   DCHECK(origin_window_);
   pointer_grabber_for_window_drag_ = origin_window_;
@@ -546,7 +638,8 @@ void WaylandDataDragController::DispatchPointerRelease() {
   DCHECK(pointer_grabber_for_window_drag_);
   pointer_delegate_->OnPointerButtonEvent(
       ET_MOUSE_RELEASED, EF_LEFT_MOUSE_BUTTON, pointer_grabber_for_window_drag_,
-      wl::EventDispatchPolicy::kImmediate);
+      wl::EventDispatchPolicy::kImmediate,
+      /*allow_release_of_unpressed_button=*/true);
   pointer_grabber_for_window_drag_ = nullptr;
 }
 
@@ -565,8 +658,9 @@ uint32_t WaylandDataDragController::DispatchEvent(const PlatformEvent& event) {
   // to be notified about it (eg: an error event), and the only way of detecting
   // that, for now, is to monitor wl_pointer events here and abort the session
   // if it comes in.
-  if (event->type() == ET_MOUSE_RELEASED)
+  if (event->type() == ET_MOUSE_RELEASED) {
     OnDataSourceFinish(/*completed=*/false);
+  }
 
   return POST_DISPATCH_PERFORM_DEFAULT;
 }
