@@ -4,17 +4,19 @@
 
 #include "ui/ozone/platform/wayland/host/wayland_event_source.h"
 
+#include <functional>
 #include <memory>
 
-#include "base/bind.h"
 #include "base/check.h"
 #include "base/containers/cxx20_erase.h"
+#include "base/functional/bind.h"
 #include "base/logging.h"
 #include "base/memory/raw_ptr.h"
 #include "base/ranges/algorithm.h"
 #include "base/time/time.h"
 #include "build/chromeos_buildflags.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
+#include "ui/base/dragdrop/mojom/drag_drop_types.mojom.h"
 #include "ui/events/base_event_utils.h"
 #include "ui/events/event_utils.h"
 #include "ui/events/keycodes/dom/dom_code.h"
@@ -22,23 +24,53 @@
 #include "ui/events/keycodes/dom/keycode_converter.h"
 #include "ui/events/keycodes/keyboard_code_conversion.h"
 #include "ui/events/keycodes/keyboard_codes.h"
+#include "ui/events/ozone/events_ozone.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine.h"
 #include "ui/events/ozone/layout/keyboard_layout_engine_manager.h"
+#include "ui/events/platform/wayland/wayland_event_watcher.h"
 #include "ui/events/pointer_details.h"
 #include "ui/events/types/event_type.h"
 #include "ui/gfx/geometry/point_f.h"
 #include "ui/gfx/geometry/vector2d_f.h"
+#include "ui/ozone/platform/wayland/host/dump_util.h"
 #include "ui/ozone/platform/wayland/host/wayland_connection.h"
 #include "ui/ozone/platform/wayland/host/wayland_cursor_position.h"
-#include "ui/ozone/platform/wayland/host/wayland_event_watcher.h"
 #include "ui/ozone/platform/wayland/host/wayland_keyboard.h"
 #include "ui/ozone/platform/wayland/host/wayland_window.h"
+#include "ui/ozone/platform/wayland/host/wayland_window_drag_controller.h"
 #include "ui/ozone/platform/wayland/host/wayland_window_manager.h"
 
 namespace ui {
 
 namespace {
 
+constexpr auto kPointerToStringMap = base::MakeFixedFlatMap<int, const char*>({
+    {EF_LEFT_MOUSE_BUTTON, "Left"},
+    {EF_MIDDLE_MOUSE_BUTTON, "Middle"},
+    {EF_RIGHT_MOUSE_BUTTON, "Right"},
+    {EF_BACK_MOUSE_BUTTON, "Back"},
+    {EF_FORWARD_MOUSE_BUTTON, "Forward"},
+});
+
+constexpr auto kModifierToStringMap = base::MakeFixedFlatMap<int, const char*>({
+    {ui::EF_SHIFT_DOWN, "Shift"},
+    {ui::EF_CONTROL_DOWN, "Control"},
+    {ui::EF_ALT_DOWN, "Alt"},
+    {ui::EF_COMMAND_DOWN, "Command"},
+    {ui::EF_ALTGR_DOWN, "AltGr"},
+    {ui::EF_MOD3_DOWN, "Mod3"},
+    {ui::EF_CAPS_LOCK_ON, "CapsLock"},
+    {ui::EF_NUM_LOCK_ON, "NumLock"},
+});
+
+std::string ToPointerFlagsString(int flags) {
+  return ToMatchingKeyMaskString(flags, kPointerToStringMap);
+}
+
+std::string ToKeyboardModifierStrings(int modifiers) {
+  return ToMatchingKeyMaskString(modifiers, kModifierToStringMap);
+}
+
 bool HasAnyPointerButtonFlag(int flags) {
   return (flags & (EF_LEFT_MOUSE_BUTTON | EF_MIDDLE_MOUSE_BUTTON |
                    EF_RIGHT_MOUSE_BUTTON | EF_BACK_MOUSE_BUTTON |
@@ -91,7 +123,10 @@ void SetRootLocation(LocatedEvent* event) {
 constexpr int kGestureScrollFingerCount = 2;
 
 // Maximum size of the latest pointer scroll data set to be stored.
-constexpr int kPointerScrollDataSetMaxSize = 20;
+constexpr int kPointerScrollDataSetMaxSize = 3;
+
+// Maximum time delta between last scroll event and lifting of fingers.
+constexpr int kFlingStartTimeoutMs = 200;
 
 }  // namespace
 
@@ -99,7 +134,7 @@ struct WaylandEventSource::TouchPoint {
   TouchPoint(gfx::PointF location, WaylandWindow* current_window);
   ~TouchPoint() = default;
 
-  raw_ptr<WaylandWindow> window;
+  raw_ptr<WaylandWindow, DanglingUntriaged> window;
   gfx::PointF last_known_location;
 };
 
@@ -123,6 +158,14 @@ WaylandEventSource::PointerScrollData::operator=(const PointerScrollData&) =
 WaylandEventSource::PointerScrollData&
 WaylandEventSource::PointerScrollData::operator=(PointerScrollData&&) = default;
 
+void WaylandEventSource::PointerScrollData::DumpState(std::ostream& out) const {
+  if (axis_source) {
+    out << "axis_source=" << *axis_source;
+  }
+  out << ", d=(" << dx << ", " << dy << "), dt=" << dt
+      << ", is_axis_stop=" << ToBoolString(is_axis_stop);
+}
+
 // WaylandEventSource::FrameData implementation
 WaylandEventSource::FrameData::FrameData(const Event& e,
                                          base::OnceCallback<void()> cb)
@@ -130,6 +173,11 @@ WaylandEventSource::FrameData::FrameData(const Event& e,
 
 WaylandEventSource::FrameData::~FrameData() = default;
 
+void WaylandEventSource::FrameData::DumpState(std::ostream& out) const {
+  out << "event=" << (event ? event->ToString() : "none")
+      << ", callback=" << !!completion_cb;
+}
+
 // WaylandEventSource implementation
 
 // static
@@ -145,12 +193,14 @@ void WaylandEventSource::ConvertEventToTarget(const EventTarget* new_target,
 WaylandEventSource::WaylandEventSource(wl_display* display,
                                        wl_event_queue* event_queue,
                                        WaylandWindowManager* window_manager,
-                                       WaylandConnection* connection)
+                                       WaylandConnection* connection,
+                                       bool use_threaded_polling)
     : window_manager_(window_manager),
       connection_(connection),
-      event_watcher_(
-          WaylandEventWatcher::CreateWaylandEventWatcher(display,
-                                                         event_queue)) {
+      event_watcher_(WaylandEventWatcher::CreateWaylandEventWatcher(
+          display,
+          event_queue,
+          use_threaded_polling)) {
   DCHECK(window_manager_);
 
   // Observes remove changes to know when touch points can be removed.
@@ -240,8 +290,7 @@ uint32_t WaylandEventSource::OnKeyboardKeyEvent(
 
   if (kind == WaylandKeyboard::KeyEventKind::kKey) {
     // Mark that this is the key event which IME did not consume.
-    properties.emplace(kPropertyKeyboardImeFlag,
-                       std::vector<uint8_t>{kPropertyKeyboardImeIgnoredFlag});
+    SetKeyboardImeFlagProperty(&properties, kPropertyKeyboardImeIgnoredFlag);
   }
   event.SetProperties(properties);
   return DispatchEvent(&event);
@@ -288,9 +337,25 @@ void WaylandEventSource::OnPointerButtonEvent(
     int changed_button,
     WaylandWindow* window,
     wl::EventDispatchPolicy dispatch_policy) {
+  OnPointerButtonEvent(type, changed_button, window, dispatch_policy, false);
+}
+
+void WaylandEventSource::OnPointerButtonEvent(
+    EventType type,
+    int changed_button,
+    WaylandWindow* window,
+    wl::EventDispatchPolicy dispatch_policy,
+    bool allow_release_of_unpressed_button) {
   DCHECK(type == ET_MOUSE_PRESSED || type == ET_MOUSE_RELEASED);
   DCHECK(HasAnyPointerButtonFlag(changed_button));
 
+  // Ignore release events for buttons that aren't currently pressed. Such
+  // events should never happen, but there have been compositor bugs before
+  // (e.g. crbug.com/1376393).
+  if (!allow_release_of_unpressed_button && type == ET_MOUSE_RELEASED &&
+      (pointer_flags_ & changed_button) == 0)
+    return;
+
   WaylandWindow* prev_focused_window =
       window_manager_->GetCurrentPointerFocusedWindow();
   if (window)
@@ -331,7 +396,7 @@ void WaylandEventSource::OnPointerButtonEventInternal(WaylandWindow* window,
     window_manager_->SetPointerFocusedWindow(window);
 
   if (type == ET_MOUSE_RELEASED)
-    last_pointer_stylus_tool_.reset();
+    last_pointer_stylus_data_.reset();
 }
 
 void WaylandEventSource::OnPointerMotionEvent(
@@ -369,6 +434,38 @@ void WaylandEventSource::RoundTripQueue() {
   event_watcher_->RoundTripQueue();
 }
 
+void WaylandEventSource::DumpState(std::ostream& out) const {
+  out << "WaylandEventSource: " << std::endl;
+  out << "  pointer_location=" << pointer_location_.ToString()
+      << ", flags=" << ToPointerFlagsString(pointer_flags_)
+      << ", last button pressed=" << last_pointer_button_pressed_
+      << ", keyboard modifiers="
+      << ToKeyboardModifierStrings(keyboard_modifiers_) << std::endl;
+  if (relative_pointer_location_) {
+    out << "  relative_poniter_location="
+        << relative_pointer_location_->ToString() << std::endl;
+  }
+
+  size_t i = 0;
+  for (const auto& frame_data : pointer_frames_) {
+    out << "  pointer_frame[" << i++ << "]=";
+    frame_data->DumpState(out);
+    out << std::endl;
+  }
+  i = 0;
+  for (const auto& frame_data : touch_frames_) {
+    out << "  touch_frame[" << i++ << "]=";
+    frame_data->DumpState(out);
+    out << std::endl;
+  }
+  i = 0;
+  for (const auto& scroll_data : pointer_scroll_data_set_) {
+    out << "  point_scroll_data[" << i++ << "]=";
+    scroll_data.DumpState(out);
+    out << std::endl;
+  }
+}
+
 const gfx::PointF& WaylandEventSource::GetPointerLocation() const {
   return pointer_location_;
 }
@@ -592,6 +689,16 @@ void WaylandEventSource::OnTouchFrame() {
 }
 
 void WaylandEventSource::OnTouchFocusChanged(WaylandWindow* window) {
+  // If a window dragging session is active (and touch-based), transfer the
+  // touch points to it.
+  auto drag_source = connection_->window_drag_controller()->drag_source();
+  if (drag_source && window) {
+    DCHECK_EQ(*drag_source, mojom::DragEventSource::kTouch);
+    for (auto& touch_point : touch_points_) {
+      touch_point.second->window = window;
+    }
+  }
+
   window_manager_->SetTouchFocusedWindow(window);
 }
 
@@ -653,6 +760,46 @@ void WaylandEventSource::OnPinchEvent(EventType event_type,
   SetTargetAndDispatchEvent(&event, target);
 }
 
+void WaylandEventSource::OnHoldEvent(EventType event_type,
+                                     uint32_t finger_count,
+                                     base::TimeTicks timestamp,
+                                     int device_id,
+                                     wl::EventDispatchPolicy dispatch_policy) {
+  // Lifting the finger from the touchpad will be ignored.
+  if (event_type != ET_TOUCH_PRESSED) {
+    return;
+  }
+
+#if !BUILDFLAG(IS_CHROMEOS_LACROS)
+  // Prevent generating any scroll events if pointer has just been moved.
+  if (!is_fling_active_) {
+    return;
+  }
+  is_fling_active_ = false;
+#endif
+
+  // Prevent fling start if axis stop arrives after hold gesture.
+  if (pointer_scroll_data_) {
+    pointer_scroll_data_->dx = 0;
+    pointer_scroll_data_->dy = 0;
+  }
+
+  pointer_scroll_data_set_.clear();
+
+  ScrollEvent event(ET_SCROLL_FLING_CANCEL, pointer_location_,
+                    pointer_location_, EventTimeForNow(), pointer_flags_, 0, 0,
+                    0, 0, finger_count);
+
+  auto* target = window_manager_->GetCurrentPointerFocusedWindow();
+
+  if (dispatch_policy == wl::EventDispatchPolicy::kImmediate) {
+    SetTargetAndDispatchEvent(&event, target);
+  } else {
+    pointer_frames_.push_back(
+        std::make_unique<FrameData>(event, base::NullCallback()));
+  }
+}
+
 void WaylandEventSource::SetRelativePointerMotionEnabled(bool enabled) {
   if (enabled)
     relative_pointer_location_ = pointer_location_;
@@ -676,33 +823,44 @@ bool WaylandEventSource::IsPointerButtonPressed(EventFlags button) const {
 
 void WaylandEventSource::OnPointerStylusToolChanged(
     EventPointerType pointer_type) {
-  last_pointer_stylus_tool_ = {
+  // When the reported pointer stylus type is `mouse`, handle it as a regular
+  // pointer event.
+  //
+  // TODO(https://crbug.com/1298504): Better handle the `touch` type, which
+  // seems mis-specified in
+  // //t_p/wayland-protocols/unstable/stylus/stylus-unstable-v2.xml.
+  if (pointer_type == ui::EventPointerType::kMouse) {
+    last_pointer_stylus_data_.reset();
+    return;
+  }
+
+  last_pointer_stylus_data_ = {
       .type = pointer_type,
       .tilt = gfx::Vector2dF(),
       .force = std::numeric_limits<float>::quiet_NaN()};
 }
 
 void WaylandEventSource::OnPointerStylusForceChanged(float force) {
-  if (!last_pointer_stylus_tool_.has_value()) {
+  if (!last_pointer_stylus_data_.has_value()) {
     // This is a stray force event that the default tool cannot accept.
     LOG(WARNING) << "Cannot handle force for the default tool!  (the value is "
                  << force << ")";
     return;
   }
 
-  last_pointer_stylus_tool_->force = force;
+  last_pointer_stylus_data_->force = force;
 }
 
 void WaylandEventSource::OnPointerStylusTiltChanged(
     const gfx::Vector2dF& tilt) {
-  if (!last_pointer_stylus_tool_.has_value()) {
+  if (!last_pointer_stylus_data_.has_value()) {
     // This is a stray tilt event that the default tool cannot accept.
     LOG(WARNING) << "Cannot handle tilt for the default tool!  (the value is ["
                  << tilt.x() << "," << tilt.y() << "])";
     return;
   }
 
-  last_pointer_stylus_tool_->tilt = tilt;
+  last_pointer_stylus_data_->tilt = tilt;
 }
 
 const WaylandWindow* WaylandEventSource::GetPointerTarget() const {
@@ -718,11 +876,18 @@ void WaylandEventSource::OnDispatcherListChanged() {
 }
 
 void WaylandEventSource::OnWindowRemoved(WaylandWindow* window) {
-  if (connection_->IsDragInProgress()) {
-    auto* target_window = window_manager_->GetCurrentTouchFocusedWindow();
-    for (auto& touch_point : touch_points_)
-      touch_point.second->window = target_window;
-    return;
+  // A window can be `swallowed` by another window during tab-dragging, which
+  // results in OnWindowRemoved() being called.
+  //
+  // If a window dragging session is active and is touch-based, verify if there
+  // is a valid target window to transfer the touch points to.
+  if (auto* target_window = window_manager_->GetCurrentTouchFocusedWindow()) {
+    auto drag_source = connection_->window_drag_controller()->drag_source();
+    if (drag_source && *drag_source == mojom::DragEventSource::kTouch) {
+      for (auto& touch_point : touch_points_)
+        touch_point.second->window = target_window;
+      return;
+    }
   }
 
   // Clear touch-related data.
@@ -748,43 +913,88 @@ bool WaylandEventSource::ShouldUnsetTouchFocus(WaylandWindow* win,
 }
 
 gfx::Vector2dF WaylandEventSource::ComputeFlingVelocity() {
-  // Return average velocity in the last 200ms.
-  // TODO(fukino): Make the formula similar to libgestures's
-  // RegressScrollVelocity(). crbug.com/1129263.
-  base::TimeDelta dt;
-  float dx = 0.0f;
-  float dy = 0.0f;
-  for (auto& frame : pointer_scroll_data_set_) {
+  struct RegressionSums {
+    float tt_;  // Cumulative sum of t^2.
+    float t_;   // Cumulative sum of t.
+    float tx_;  // Cumulative sum of t * x.
+    float ty_;  // Cumulative sum of t * y.
+    float x_;   // Cumulative sum of x.
+    float y_;   // Cumulative sum of y.
+  };
+
+  const size_t count = pointer_scroll_data_set_.size();
+
+  if (count == 0) {
+    return gfx::Vector2dF();
+  }
+
+  // Prevents small jumps if someone scrolls fast, immediately stops scrolling
+  // and then waits a little before lifting fingers from touchpad.
+  if (pointer_scroll_data_->dt > base::Milliseconds(kFlingStartTimeoutMs)) {
+    return gfx::Vector2dF();
+  }
+
+  if (count == 1) {
+    const auto& pointer_frame = pointer_scroll_data_set_.front();
+    const float dt =
+        pointer_frame.dt.InSecondsF() + pointer_scroll_data_->dt.InSecondsF();
+    return gfx::Vector2dF(pointer_frame.dx * dt, pointer_frame.dy * dt);
+  }
+
+  RegressionSums sums = {0, 0, 0, 0, 0, 0};
+
+  float time = pointer_scroll_data_->dt.InSecondsF();
+  float x_coord = 0;
+  float y_coord = 0;
+
+  // Formula matches libgestures's RegressScrollVelocity()
+  // from src/platform/gestures/src/immediate_interpreter.cc
+  for (const auto& frame : pointer_scroll_data_set_) {
     if (frame.axis_source &&
         *frame.axis_source != WL_POINTER_AXIS_SOURCE_FINGER) {
       break;
     }
-    if (frame.dx == 0 && frame.dy == 0)
-      break;
-    if (dt + frame.dt > base::Milliseconds(200))
-      break;
+    time += frame.dt.InSecondsF();
+    x_coord += frame.dx;
+    y_coord += frame.dy;
 
-    dx += frame.dx;
-    dy += frame.dy;
-    dt += frame.dt;
+    sums.tt_ += time * time;
+    sums.t_ += time;
+    sums.tx_ += time * x_coord;
+    sums.ty_ += time * y_coord;
+    sums.x_ += x_coord;
+    sums.y_ += y_coord;
   }
   pointer_scroll_data_set_.clear();
 
-  float dt_inv = 1.0f / dt.InSecondsF();
-  return dt.is_zero() ? gfx::Vector2dF()
-                      : gfx::Vector2dF(dx * dt_inv, dy * dt_inv);
+  // Note the regression determinant only depends on the values of t, and should
+  // never be zero so long as (1) count > 1, and (2) dt[0] != d[1]. The
+  // condition of (1) was already caught at the beginning of the method.
+  const float det = count * sums.tt_ - sums.t_ * sums.t_;
+  if (!det) {
+    // This will return the average scroll value if dt values are
+    // non-zero.
+    if (sums.t_) {
+      return gfx::Vector2dF(x_coord / sums.t_, y_coord / sums.t_);
+    }
+    return gfx::Vector2dF();
+  }
+
+  const float det_inv = 1.0 / det;
+  return gfx::Vector2dF((count * sums.tx_ - sums.t_ * sums.x_) * det_inv,
+                        (count * sums.ty_ - sums.t_ * sums.y_) * det_inv);
 }
 
 absl::optional<PointerDetails> WaylandEventSource::AmendStylusData() const {
-  if (!last_pointer_stylus_tool_)
+  if (!last_pointer_stylus_data_)
     return absl::nullopt;
 
-  DCHECK_NE(last_pointer_stylus_tool_->type, EventPointerType::kUnknown);
-  return PointerDetails(last_pointer_stylus_tool_->type, /*pointer_id=*/0,
+  DCHECK_NE(last_pointer_stylus_data_->type, EventPointerType::kUnknown);
+  return PointerDetails(last_pointer_stylus_data_->type, /*pointer_id=*/0,
                         /*radius_x=*/1.0f,
-                        /*radius_y=*/1.0f, last_pointer_stylus_tool_->force,
-                        /*twist=*/0.0f, last_pointer_stylus_tool_->tilt.x(),
-                        last_pointer_stylus_tool_->tilt.y());
+                        /*radius_y=*/1.0f, last_pointer_stylus_data_->force,
+                        /*twist=*/0.0f, last_pointer_stylus_data_->tilt.x(),
+                        last_pointer_stylus_data_->tilt.y());
 }
 
 absl::optional<PointerDetails> WaylandEventSource::AmendStylusData(
@@ -811,30 +1021,40 @@ WaylandEventSource::EnsurePointerScrollData() {
   return *pointer_scroll_data_;
 }
 
+// This method behaves differently in Exo than in other window managers.
+// If you place a finger on the touchpad, Exo dispatches axis events with an
+// offset of 0 to indicate that a fling should be aborted. This event does not
+// exist in Linux window managers. Instead, some window managers implement
+// zwp_pointer_gesture_hold_v1 for this. However, for those who don't implement
+// that, it needs to be ensured that flings are aborted when new axis events
+// arrive.
 void WaylandEventSource::ProcessPointerScrollData() {
   DCHECK(pointer_scroll_data_);
-
   int flags = pointer_flags_ | keyboard_modifiers_;
-
-  static constexpr bool supports_trackpad_kinetic_scrolling =
-#if BUILDFLAG(IS_CHROMEOS_LACROS)
-      true;
-#else
-      false;
-#endif
-
   // Dispatch Fling event if pointer.axis_stop is notified and the recent
   // pointer.axis events meets the criteria to start fling scroll.
-  if (pointer_scroll_data_->dx == 0 && pointer_scroll_data_->dy == 0 &&
-      pointer_scroll_data_->is_axis_stop &&
-      supports_trackpad_kinetic_scrolling) {
+  if (pointer_scroll_data_->dx == 0 && pointer_scroll_data_->dy == 0
+#if !BUILDFLAG(IS_CHROMEOS_LACROS)
+      && pointer_scroll_data_->is_axis_stop
+#endif
+  ) {
     gfx::Vector2dF initial_velocity = ComputeFlingVelocity();
     float vx = initial_velocity.x();
     float vy = initial_velocity.y();
-    ScrollEvent event(
-        vx == 0 && vy == 0 ? ET_SCROLL_FLING_CANCEL : ET_SCROLL_FLING_START,
-        pointer_location_, pointer_location_, EventTimeForNow(), flags, vx, vy,
-        vx, vy, kGestureScrollFingerCount);
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+    ScrollEvent event(pointer_scroll_data_->is_axis_stop
+                          ? ET_SCROLL_FLING_START
+                          : ET_SCROLL_FLING_CANCEL,
+                      pointer_location_, pointer_location_, EventTimeForNow(),
+                      flags, vx, vy, vx, vy, kGestureScrollFingerCount);
+#else
+    // In Linux there is no axis event with 0 delta when start scrolling.
+    // A fling is therefore always started at this point.
+    ScrollEvent event(ET_SCROLL_FLING_START, pointer_location_,
+                      pointer_location_, EventTimeForNow(), flags, vx, vy, vx,
+                      vy, kGestureScrollFingerCount);
+    is_fling_active_ = true;
+#endif
     pointer_frames_.push_back(
         std::make_unique<FrameData>(event, base::NullCallback()));
   } else if (pointer_scroll_data_->axis_source) {
@@ -850,6 +1070,18 @@ void WaylandEventSource::ProcessPointerScrollData() {
                    WL_POINTER_AXIS_SOURCE_FINGER ||
                *pointer_scroll_data_->axis_source ==
                    WL_POINTER_AXIS_SOURCE_CONTINUOUS) {
+#if !BUILDFLAG(IS_CHROMEOS_LACROS)
+      // Fling has to be stopped if a new scroll event is received.
+      // From Wayland 1.23 this will be done through hold event.
+      if (is_fling_active_) {
+        is_fling_active_ = false;
+        ScrollEvent stop_fling_event(
+            ET_SCROLL_FLING_CANCEL, pointer_location_, pointer_location_,
+            EventTimeForNow(), flags, 0, 0, 0, 0, kGestureScrollFingerCount);
+        pointer_frames_.push_back(std::make_unique<FrameData>(
+            stop_fling_event, base::NullCallback()));
+      }
+#endif
       ScrollEvent event(ET_SCROLL, pointer_location_, pointer_location_,
                         EventTimeForNow(), flags, pointer_scroll_data_->dx,
                         pointer_scroll_data_->dy, pointer_scroll_data_->dx,
