@@ -6,14 +6,15 @@
 
 #include <aura-shell-client-protocol.h>
 #include <xdg-decoration-unstable-v1-client-protocol.h>
-#include <xdg-shell-unstable-v6-client-protocol.h>
 
 #include "base/logging.h"
+#include "base/notreached.h"
 #include "base/strings/utf_string_conversions.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 #include "ui/base/hit_test.h"
 #include "ui/base/ui_base_features.h"
 #include "ui/ozone/common/features.h"
+#include "ui/ozone/platform/wayland/common/wayland_object.h"
 #include "ui/ozone/platform/wayland/common/wayland_util.h"
 #include "ui/ozone/platform/wayland/host/shell_surface_wrapper.h"
 #include "ui/ozone/platform/wayland/host/shell_toplevel_wrapper.h"
@@ -31,6 +32,9 @@ namespace ui {
 
 namespace {
 
+static_assert(sizeof(uint32_t) == sizeof(float),
+              "Sizes much match for reinterpret cast to be meaningful");
+
 XDGToplevelWrapperImpl::DecorationMode ToDecorationMode(uint32_t mode) {
   switch (mode) {
     case ZXDG_TOPLEVEL_DECORATION_V1_MODE_CLIENT_SIDE:
@@ -98,19 +102,6 @@ bool XDGToplevelWrapperImpl::Initialize() {
     return false;
   }
 
-  static constexpr xdg_toplevel_listener xdg_toplevel_listener = {
-    &ConfigureTopLevel,
-    &CloseTopLevel,
-#if defined(XDG_TOPLEVEL_CONFIGURE_BOUNDS_SINCE_VERSION)
-    // Since v4
-    &ConfigureBounds,
-#endif
-#if defined(XDG_TOPLEVEL_WM_CAPABILITIES_SINCE_VERSION)
-    // Since v5
-    &WmCapabilities,
-#endif
-  };
-
   if (!xdg_surface_wrapper_)
     return false;
 
@@ -120,8 +111,7 @@ bool XDGToplevelWrapperImpl::Initialize() {
     LOG(ERROR) << "Failed to create xdg_toplevel";
     return false;
   }
-  connection_->wayland_window_manager()->NotifyWindowRoleAssigned(
-      wayland_window_);
+  connection_->window_manager()->NotifyWindowRoleAssigned(wayland_window_);
 
   if (connection_->zaura_shell()) {
     uint32_t version =
@@ -139,13 +129,39 @@ bool XDGToplevelWrapperImpl::Initialize() {
     }
   }
 
-  xdg_toplevel_add_listener(xdg_toplevel_.get(), &xdg_toplevel_listener, this);
+  static constexpr xdg_toplevel_listener kXdgToplevelListener = {
+      .configure = &OnToplevelConfigure,
+      .close = &OnToplevelClose,
+      // Since v4
+      .configure_bounds = &OnConfigureBounds,
+      // Since v5
+      .wm_capabilities = &OnWmCapabilities,
+  };
+  xdg_toplevel_add_listener(xdg_toplevel_.get(), &kXdgToplevelListener, this);
 
   InitializeXdgDecoration();
 
   return true;
 }
 
+bool XDGToplevelWrapperImpl::IsSupportedOnAuraToplevel(uint32_t version) const {
+  return aura_toplevel_ &&
+         zaura_toplevel_get_version(aura_toplevel_.get()) >= version;
+}
+
+void XDGToplevelWrapperImpl::SetCanMaximize(bool can_maximize) {
+  if (!aura_toplevel_ || zaura_toplevel_get_version(aura_toplevel_.get()) <
+                             ZAURA_TOPLEVEL_SET_CAN_MAXIMIZE_SINCE_VERSION) {
+    return;
+  }
+
+  if (can_maximize) {
+    zaura_toplevel_set_can_maximize(aura_toplevel_.get());
+  } else {
+    zaura_toplevel_unset_can_maximize(aura_toplevel_.get());
+  }
+}
+
 void XDGToplevelWrapperImpl::SetMaximized() {
   DCHECK(xdg_toplevel_);
   xdg_toplevel_set_maximized(xdg_toplevel_.get());
@@ -156,11 +172,46 @@ void XDGToplevelWrapperImpl::UnSetMaximized() {
   xdg_toplevel_unset_maximized(xdg_toplevel_.get());
 }
 
+void XDGToplevelWrapperImpl::SetCanFullscreen(bool can_fullscreen) {
+  if (!aura_toplevel_ || zaura_toplevel_get_version(aura_toplevel_.get()) <
+                             ZAURA_TOPLEVEL_SET_CAN_FULLSCREEN_SINCE_VERSION) {
+    return;
+  }
+
+  if (can_fullscreen) {
+    zaura_toplevel_set_can_fullscreen(aura_toplevel_.get());
+  } else {
+    zaura_toplevel_unset_can_fullscreen(aura_toplevel_.get());
+  }
+}
+
 void XDGToplevelWrapperImpl::SetFullscreen() {
   DCHECK(xdg_toplevel_);
   xdg_toplevel_set_fullscreen(xdg_toplevel_.get(), nullptr);
 }
 
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+void XDGToplevelWrapperImpl::SetUseImmersiveMode(bool immersive) {
+  if (SupportsTopLevelImmersiveStatus()) {
+    auto mode = immersive ? ZAURA_TOPLEVEL_FULLSCREEN_MODE_IMMERSIVE
+                          : ZAURA_TOPLEVEL_FULLSCREEN_MODE_PLAIN;
+    zaura_toplevel_set_fullscreen_mode(aura_toplevel_.get(), mode);
+  }
+}
+
+bool XDGToplevelWrapperImpl::SupportsTopLevelImmersiveStatus() const {
+  return aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                               ZAURA_TOPLEVEL_SET_FULLSCREEN_MODE_SINCE_VERSION;
+}
+
+void XDGToplevelWrapperImpl::SetTopInset(int height) {
+  if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                            ZAURA_TOPLEVEL_SET_TOP_INSET_SINCE_VERSION) {
+    zaura_toplevel_set_top_inset(aura_toplevel_.get(), height);
+  }
+}
+#endif
+
 void XDGToplevelWrapperImpl::UnSetFullscreen() {
   DCHECK(xdg_toplevel_);
   xdg_toplevel_unset_fullscreen(xdg_toplevel_.get());
@@ -183,8 +234,7 @@ void XDGToplevelWrapperImpl::SurfaceResize(WaylandConnection* connection,
   DCHECK(xdg_toplevel_);
   if (auto serial = GetSerialForMoveResize(connection)) {
     xdg_toplevel_resize(xdg_toplevel_.get(), connection->seat()->wl_object(),
-                        serial->value,
-                        wl::IdentifyDirection(*connection, hittest));
+                        serial->value, wl::IdentifyDirection(hittest));
   }
 }
 
@@ -241,14 +291,13 @@ bool XDGToplevelWrapperImpl::IsConfigured() {
 }
 
 // static
-void XDGToplevelWrapperImpl::ConfigureTopLevel(
-    void* data,
-    struct xdg_toplevel* xdg_toplevel,
-    int32_t width,
-    int32_t height,
-    struct wl_array* states) {
-  auto* surface = static_cast<XDGToplevelWrapperImpl*>(data);
-  DCHECK(surface);
+void XDGToplevelWrapperImpl::OnToplevelConfigure(void* data,
+                                                 xdg_toplevel* toplevel,
+                                                 int32_t width,
+                                                 int32_t height,
+                                                 wl_array* states) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
 
   WaylandWindow::WindowStates window_states{
       .is_maximized =
@@ -259,7 +308,8 @@ void XDGToplevelWrapperImpl::ConfigureTopLevel(
           CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_ACTIVATED),
   };
 
-  if (xdg_toplevel_get_version(xdg_toplevel) >=
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS_LACROS)
+  if (xdg_toplevel_get_version(toplevel) >=
       XDG_TOPLEVEL_STATE_TILED_LEFT_SINCE_VERSION) {
     // All four tiled states have the same since version, so it is enough to
     // check only one.
@@ -270,79 +320,114 @@ void XDGToplevelWrapperImpl::ConfigureTopLevel(
         .bottom =
             CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_TILED_BOTTOM)};
   }
+#endif  // IS_LINUX || IS_CHROMEOS_LACROS
 
-  surface->wayland_window_->HandleToplevelConfigure(width, height,
-                                                    window_states);
+  self->wayland_window_->HandleToplevelConfigure(width, height, window_states);
 }
 
 // static
-void XDGToplevelWrapperImpl::ConfigureAuraTopLevel(
+void XDGToplevelWrapperImpl::OnToplevelClose(void* data,
+                                             xdg_toplevel* toplevel) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
+  self->wayland_window_->OnCloseRequest();
+}
+
+// static
+void XDGToplevelWrapperImpl::OnConfigureBounds(void* data,
+                                               xdg_toplevel* toplevel,
+                                               int32_t width,
+                                               int32_t height) {
+  NOTIMPLEMENTED_LOG_ONCE();
+}
+
+// static
+void XDGToplevelWrapperImpl::OnWmCapabilities(void* data,
+                                              xdg_toplevel* toplevel,
+                                              wl_array* capabilities) {
+  NOTIMPLEMENTED_LOG_ONCE();
+}
+
+// static
+void XDGToplevelWrapperImpl::OnDecorationConfigure(
     void* data,
-    struct zaura_toplevel* zaura_toplevel,
+    zxdg_toplevel_decoration_v1* decoration,
+    uint32_t mode) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
+  self->decoration_mode_ = ToDecorationMode(mode);
+}
+
+// static
+void XDGToplevelWrapperImpl::OnAuraToplevelConfigure(
+    void* data,
+    zaura_toplevel* aura_toplevel,
     int32_t x,
     int32_t y,
     int32_t width,
     int32_t height,
-    struct wl_array* states) {
-  auto* surface = static_cast<XDGToplevelWrapperImpl*>(data);
-  DCHECK(surface);
+    wl_array* states) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
 
-  surface->wayland_window_->HandleAuraToplevelConfigure(
-      x, y, width, height,
-      {.is_maximized =
-           CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_MAXIMIZED),
-       .is_fullscreen =
-           CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_FULLSCREEN),
-       .is_activated =
-           CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_ACTIVATED),
-       .is_snapped_primary =
-           CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_SNAPPED_PRIMARY),
-       .is_snapped_secondary = CheckIfWlArrayHasValue(
-           states, ZAURA_TOPLEVEL_STATE_SNAPPED_SECONDARY),
-       .is_floated =
-           CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_FLOATED)});
+  self->wayland_window_->HandleAuraToplevelConfigure(x, y, width, height, {
+    .is_maximized =
+        CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_MAXIMIZED),
+    .is_fullscreen =
+        CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_FULLSCREEN),
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+    .is_immersive_fullscreen =
+        CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_IMMERSIVE),
+#endif
+    .is_activated =
+        CheckIfWlArrayHasValue(states, XDG_TOPLEVEL_STATE_ACTIVATED),
+    .is_minimized =
+        CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_MINIMIZED),
+    .is_snapped_primary =
+        CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_SNAPPED_PRIMARY),
+    .is_snapped_secondary =
+        CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_SNAPPED_SECONDARY),
+    .is_floated = CheckIfWlArrayHasValue(states, ZAURA_TOPLEVEL_STATE_FLOATED)
+  });
 }
 
 // static
-void XDGToplevelWrapperImpl::OnOriginChange(
+void XDGToplevelWrapperImpl::OnOriginChange(void* data,
+                                            zaura_toplevel* aura_toplevel,
+                                            int32_t x,
+                                            int32_t y) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
+  auto* toplevel_window =
+      static_cast<WaylandToplevelWindow*>(self->wayland_window_);
+  toplevel_window->SetOrigin(gfx::Point(x, y));
+}
+
+// static
+void XDGToplevelWrapperImpl::OnConfigureRasterScale(
     void* data,
-    struct zaura_toplevel* zaura_toplevel,
-    int32_t x,
-    int32_t y) {
-  auto* surface = static_cast<XDGToplevelWrapperImpl*>(data);
-  DCHECK(surface);
-  auto* wayland_toplevel_window =
-      static_cast<WaylandToplevelWindow*>(surface->wayland_window_);
-  wayland_toplevel_window->SetOrigin(gfx::Point(x, y));
+    zaura_toplevel* aura_toplevel,
+    uint32_t scale_as_uint) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
+  auto* wayland_window = static_cast<WaylandWindow*>(self->wayland_window_);
+  float scale = *reinterpret_cast<float*>(&scale_as_uint);
+  wayland_window->SetPendingRasterScale(scale);
 }
 
 // static
-void XDGToplevelWrapperImpl::CloseTopLevel(void* data,
-                                           struct xdg_toplevel* xdg_toplevel) {
-  auto* surface = static_cast<XDGToplevelWrapperImpl*>(data);
-  DCHECK(surface);
-  surface->wayland_window_->OnCloseRequest();
+void XDGToplevelWrapperImpl::OnRotateFocus(void* data,
+                                           zaura_toplevel* aura_toplevel,
+                                           uint32_t serial,
+                                           uint32_t direction,
+                                           uint32_t restart) {
+  auto* self = static_cast<XDGToplevelWrapperImpl*>(data);
+  DCHECK(self);
+  auto* toplevel_window =
+      static_cast<WaylandToplevelWindow*>(self->wayland_window_);
+  toplevel_window->OnRotateFocus(serial, direction, restart);
 }
 
-#if defined(XDG_TOPLEVEL_CONFIGURE_BOUNDS_SINCE_VERSION)
-// static
-void XDGToplevelWrapperImpl::ConfigureBounds(void* data,
-                                             struct xdg_toplevel* xdg_toplevel,
-                                             int32_t width,
-                                             int32_t height) {
-  NOTIMPLEMENTED_LOG_ONCE();
-}
-#endif
-
-#if defined(XDG_TOPLEVEL_WM_CAPABILITIES_SINCE_VERSION)
-// static
-void XDGToplevelWrapperImpl::WmCapabilities(void* data,
-                                            struct xdg_toplevel* xdg_toplevel,
-                                            struct wl_array* capabilities) {
-  NOTIMPLEMENTED_LOG_ONCE();
-}
-#endif
-
 void XDGToplevelWrapperImpl::SetTopLevelDecorationMode(
     DecorationMode requested_mode) {
   if (!zxdg_toplevel_decoration_ || requested_mode == decoration_mode_)
@@ -352,31 +437,35 @@ void XDGToplevelWrapperImpl::SetTopLevelDecorationMode(
                                        ToInt32(requested_mode));
 }
 
-// static
-void XDGToplevelWrapperImpl::ConfigureDecoration(
-    void* data,
-    struct zxdg_toplevel_decoration_v1* decoration,
-    uint32_t mode) {
-  auto* surface = static_cast<XDGToplevelWrapperImpl*>(data);
-  DCHECK(surface);
-  surface->decoration_mode_ = ToDecorationMode(mode);
-}
-
 void XDGToplevelWrapperImpl::InitializeXdgDecoration() {
   if (connection_->xdg_decoration_manager_v1()) {
     DCHECK(!zxdg_toplevel_decoration_);
-    static constexpr zxdg_toplevel_decoration_v1_listener decoration_listener =
-        {
-            &ConfigureDecoration,
-        };
     zxdg_toplevel_decoration_.reset(
         zxdg_decoration_manager_v1_get_toplevel_decoration(
             connection_->xdg_decoration_manager_v1(), xdg_toplevel_.get()));
-    zxdg_toplevel_decoration_v1_add_listener(zxdg_toplevel_decoration_.get(),
-                                             &decoration_listener, this);
+
+    static constexpr zxdg_toplevel_decoration_v1_listener
+        kToplevelDecorationListener = {
+            .configure = &OnDecorationConfigure,
+        };
+    zxdg_toplevel_decoration_v1_add_listener(
+        zxdg_toplevel_decoration_.get(), &kToplevelDecorationListener, this);
   }
 }
 
+wl::Object<wl_region> XDGToplevelWrapperImpl::CreateAndAddRegion(
+    const std::vector<gfx::Rect>& shape) {
+  wl::Object<wl_region> region(
+      wl_compositor_create_region(connection_->compositor()));
+
+  for (const auto& rect : shape) {
+    wl_region_add(region.get(), rect.x(), rect.y(), rect.width(),
+                  rect.height());
+  }
+
+  return region;
+}
+
 XDGSurfaceWrapperImpl* XDGToplevelWrapperImpl::xdg_surface_wrapper() const {
   DCHECK(xdg_surface_wrapper_.get());
   return xdg_surface_wrapper_.get();
@@ -435,8 +524,10 @@ void XDGToplevelWrapperImpl::RequestWindowBounds(const gfx::Rect& bounds) {
 
   // `entered_output` can be null in unit tests, where it doesn't wait for
   // output events.
-  if (!entered_output)
+  if (!entered_output) {
+    DLOG(WARNING) << "Entered output is null, cannot request window bounds.";
     return;
+  }
 
   if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
                             ZAURA_TOPLEVEL_SET_WINDOW_BOUNDS_SINCE_VERSION) {
@@ -463,9 +554,8 @@ bool XDGToplevelWrapperImpl::SupportsScreenCoordinates() const {
              ZAURA_TOPLEVEL_SET_SUPPORTS_SCREEN_COORDINATES_SINCE_VERSION;
 }
 
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
 void XDGToplevelWrapperImpl::EnableScreenCoordinates() {
-  if (!features::IsWaylandScreenCoordinatesEnabled())
-    return;
   if (!SupportsScreenCoordinates()) {
     LOG(WARNING) << "Server implementation of wayland is incompatible, "
                     "WaylandScreenCoordinatesEnabled has no effect.";
@@ -473,14 +563,15 @@ void XDGToplevelWrapperImpl::EnableScreenCoordinates() {
   }
   zaura_toplevel_set_supports_screen_coordinates(aura_toplevel_.get());
 
-  static constexpr zaura_toplevel_listener aura_toplevel_listener = {
-      &ConfigureAuraTopLevel,
-      &OnOriginChange,
-  };
-
-  zaura_toplevel_add_listener(aura_toplevel_.get(), &aura_toplevel_listener,
+  static constexpr zaura_toplevel_listener kAuraToplevelListener = {
+      .configure = &OnAuraToplevelConfigure,
+      .origin_change = &OnOriginChange,
+      .configure_raster_scale = &OnConfigureRasterScale,
+      .rotate_focus = &OnRotateFocus};
+  zaura_toplevel_add_listener(aura_toplevel_.get(), &kAuraToplevelListener,
                               this);
 }
+#endif
 
 void XDGToplevelWrapperImpl::SetZOrder(ZOrderLevel z_order) {
   if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
@@ -511,6 +602,14 @@ void XDGToplevelWrapperImpl::Deactivate() {
   }
 }
 
+void XDGToplevelWrapperImpl::SetScaleFactor(float scale_factor) {
+  if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                            ZAURA_TOPLEVEL_SET_SCALE_FACTOR_SINCE_VERSION) {
+    uint32_t value = *reinterpret_cast<uint32_t*>(&scale_factor);
+    zaura_toplevel_set_scale_factor(aura_toplevel_.get(), value);
+  }
+}
+
 void XDGToplevelWrapperImpl::SetRestoreInfo(int32_t restore_session_id,
                                             int32_t restore_window_id) {
   if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
@@ -546,4 +645,86 @@ void XDGToplevelWrapperImpl::UnSetFloat() {
   }
 }
 
+void XDGToplevelWrapperImpl::CommitSnap(
+    WaylandWindowSnapDirection snap_direction,
+    float snap_ratio) {
+  if (!aura_toplevel_) {
+    return;
+  }
+
+  if (zaura_toplevel_get_version(aura_toplevel_.get()) >=
+          ZAURA_TOPLEVEL_UNSET_SNAP_SINCE_VERSION &&
+      snap_direction == WaylandWindowSnapDirection::kNone) {
+    zaura_toplevel_unset_snap(aura_toplevel_.get());
+    return;
+  }
+
+  if (zaura_toplevel_get_version(aura_toplevel_.get()) >=
+      ZAURA_TOPLEVEL_SET_SNAP_PRIMARY_SINCE_VERSION) {
+    uint32_t value = *reinterpret_cast<uint32_t*>(&snap_ratio);
+    switch (snap_direction) {
+      case WaylandWindowSnapDirection::kPrimary:
+        zaura_toplevel_set_snap_primary(aura_toplevel_.get(), value);
+        return;
+      case WaylandWindowSnapDirection::kSecondary:
+        zaura_toplevel_set_snap_secondary(aura_toplevel_.get(), value);
+        return;
+      case WaylandWindowSnapDirection::kNone:
+        NOTREACHED() << "Toplevel does not support UnsetSnap yet";
+        return;
+    }
+  }
+}
+
+void XDGToplevelWrapperImpl::SetPersistable(bool persistable) const {
+  auto persistable_enum = persistable
+                              ? ZAURA_TOPLEVEL_PERSISTABLE_PERSISTABLE
+                              : ZAURA_TOPLEVEL_PERSISTABLE_NOT_PERSISTABLE;
+
+  if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                            ZAURA_TOPLEVEL_SET_PERSISTABLE_SINCE_VERSION) {
+    zaura_toplevel_set_persistable(aura_toplevel_.get(), persistable_enum);
+  }
+}
+
+void XDGToplevelWrapperImpl::SetShape(std::unique_ptr<ShapeRects> shape_rects) {
+  if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                            ZAURA_TOPLEVEL_SET_SHAPE_SINCE_VERSION) {
+    zaura_toplevel_set_shape(
+        aura_toplevel_.get(),
+        shape_rects ? CreateAndAddRegion(*shape_rects).get() : nullptr);
+  }
+}
+
+void XDGToplevelWrapperImpl::ShowSnapPreview(
+    WaylandWindowSnapDirection snap_direction,
+    bool allow_haptic_feedback) {
+  if (aura_toplevel_ && zaura_toplevel_get_version(aura_toplevel_.get()) >=
+                            ZAURA_TOPLEVEL_INTENT_TO_SNAP_SINCE_VERSION) {
+    uint32_t zaura_shell_snap_direction = ZAURA_TOPLEVEL_SNAP_DIRECTION_NONE;
+    switch (snap_direction) {
+      case WaylandWindowSnapDirection::kPrimary:
+        zaura_shell_snap_direction = ZAURA_TOPLEVEL_SNAP_DIRECTION_PRIMARY;
+        break;
+      case WaylandWindowSnapDirection::kSecondary:
+        zaura_shell_snap_direction = ZAURA_TOPLEVEL_SNAP_DIRECTION_SECONDARY;
+        break;
+      case WaylandWindowSnapDirection::kNone:
+        break;
+    }
+    zaura_toplevel_intent_to_snap(aura_toplevel_.get(),
+                                  zaura_shell_snap_direction);
+    return;
+  }
+}
+
+void XDGToplevelWrapperImpl::AckRotateFocus(uint32_t serial, uint32_t handled) {
+  zaura_toplevel_ack_rotate_focus(aura_toplevel_.get(), serial, handled);
+  connection_->Flush();
+}
+
+XDGToplevelWrapperImpl* XDGToplevelWrapperImpl::AsXDGToplevelWrapper() {
+  return this;
+}
+
 }  // namespace ui
