@@ -24,8 +24,6 @@ class WaylandUtils : public PlatformUtils {
 
   gfx::ImageSkia GetNativeWindowIcon(intptr_t target_window_id) override;
   std::string GetWmWindowClass(const std::string& desktop_base_name) override;
-  std::unique_ptr<PlatformUtils::ScopedDisableClientSideDecorationsForTest>
-  DisableClientSideDecorationsForTest() override;
   void OnUnhandledKeyEvent(const KeyEvent& key_event) override;
 
  private:
