@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include "ui/ozone/platform/x11/x11_window.h"
+#include "base/memory/raw_ptr.h"
 
 #include "base/command_line.h"
 #include "base/containers/contains.h"
@@ -102,7 +103,7 @@ class TestPlatformWindowDelegate : public PlatformWindowDelegate {
   void set_window(X11Window* window) { window_ = window; }
 
  private:
-  X11Window* window_ = nullptr;
+  raw_ptr<X11Window> window_ = nullptr;
   gfx::AcceleratedWidget widget_ = gfx::kNullAcceleratedWidget;
   PlatformWindowState state_ = PlatformWindowState::kUnknown;
   PlatformWindowDelegate::BoundsChange changed_{false};
@@ -423,7 +424,7 @@ TEST_F(X11WindowTest, MAYBE_WindowManagerTogglesFullscreen) {
   gfx::Rect initial_bounds = window->GetBoundsInPixels();
   {
     WMStateWaiter waiter(x11_window, "_NET_WM_STATE_FULLSCREEN", true);
-    window->ToggleFullscreen();
+    window->SetFullscreen(true, display::kInvalidDisplayId);
     waiter.Wait();
   }
   EXPECT_EQ(window->GetPlatformWindowState(), PlatformWindowState::kFullScreen);
@@ -464,7 +465,7 @@ TEST_F(X11WindowTest, MAYBE_WindowManagerTogglesFullscreen) {
 
   // Calling Widget::SetFullscreen(false) should clear the widget's fullscreen
   // state and clean things up.
-  window->ToggleFullscreen();
+  window->SetFullscreen(false, display::kInvalidDisplayId);
   EXPECT_NE(window->GetPlatformWindowState(), PlatformWindowState::kFullScreen);
   delegate.WaitForBoundsChange({false});
   EXPECT_EQ(initial_bounds, window->GetBoundsInPixels());
