@@ -7,11 +7,13 @@
 #include "base/memory/scoped_refptr.h"
 #include "base/no_destructor.h"
 #include "base/notreached.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/trace_event/trace_event.h"
 #include "build/chromeos_buildflags.h"
 #include "net/base/network_interfaces.h"
+#include "third_party/skia/include/core/SkPath.h"
 #include "third_party/skia/include/core/SkRegion.h"
 #include "ui/base/buildflags.h"
 #include "ui/base/cursor/cursor.h"
@@ -605,9 +607,9 @@ bool X11Window::HasCapture() const {
   return X11WindowManager::GetInstance()->located_events_grabber() == this;
 }
 
-void X11Window::ToggleFullscreen() {
-  // Check if we need to fullscreen the window or not.
-  bool fullscreen = state_ != PlatformWindowState::kFullScreen;
+void X11Window::SetFullscreen(bool fullscreen, int64_t target_display_id) {
+  // TODO(crbug.com/1034783) Support `target_display_id` on this platform.
+  DCHECK_EQ(target_display_id, display::kInvalidDisplayId);
   if (fullscreen)
     CancelResize();
 
@@ -729,11 +731,16 @@ void X11Window::Minimize() {
 }
 
 void X11Window::Restore() {
-  should_maximize_after_map_ = false;
-  restore_in_flight_ = true;
-  SetWMSpecState(false, x11::GetAtom("_NET_WM_STATE_MAXIMIZED_VERT"),
-                 x11::GetAtom("_NET_WM_STATE_MAXIMIZED_HORZ"));
-  SetWMSpecState(false, x11::GetAtom("_NET_WM_STATE_HIDDEN"), x11::Atom::None);
+  if (IsMinimized()) {
+    restore_in_flight_ = true;
+    SetWMSpecState(false, x11::GetAtom("_NET_WM_STATE_HIDDEN"),
+                   x11::Atom::None);
+  } else if (IsMaximized()) {
+    restore_in_flight_ = true;
+    should_maximize_after_map_ = false;
+    SetWMSpecState(false, x11::GetAtom("_NET_WM_STATE_MAXIMIZED_VERT"),
+                   x11::GetAtom("_NET_WM_STATE_MAXIMIZED_HORZ"));
+  }
 }
 
 PlatformWindowState X11Window::GetPlatformWindowState() const {
@@ -1097,7 +1104,8 @@ void X11Window::SetDecorationInsets(const gfx::Insets* insets_px) {
   }
 }
 
-void X11Window::SetOpaqueRegion(const std::vector<gfx::Rect>* region_px) {
+void X11Window::SetOpaqueRegion(
+    absl::optional<std::vector<gfx::Rect>> region_px) {
   auto atom = x11::GetAtom("_NET_WM_OPAQUE_REGION");
   if (!region_px) {
     x11::DeleteProperty(xwindow_, atom);
@@ -1113,7 +1121,7 @@ void X11Window::SetOpaqueRegion(const std::vector<gfx::Rect>* region_px) {
   x11::SetArrayProperty(xwindow_, atom, x11::Atom::CARDINAL, value);
 }
 
-void X11Window::SetInputRegion(const gfx::Rect* region_px) {
+void X11Window::SetInputRegion(absl::optional<gfx::Rect> region_px) {
   if (!region_px) {
     // Reset the input region.
     connection_->shape().Mask({
@@ -1549,6 +1557,11 @@ void X11Window::CancelDrag() {
   QuitDragLoop();
 }
 
+void X11Window::UpdateDragImage(const gfx::ImageSkia& image,
+                                const gfx::Vector2d& offset) {
+  NOTIMPLEMENTED();
+}
+
 absl::optional<gfx::AcceleratedWidget> X11Window::GetDragWidget() {
   DCHECK(drag_location_delegate_);
   return drag_location_delegate_->GetDragWidget();
@@ -1812,9 +1825,8 @@ void X11Window::CloseXWindow() {
   // Unregister from the global security surface list if necessary.
   if (is_security_surface_) {
     auto& security_surfaces = GetSecuritySurfaces();
-    security_surfaces.erase(
-        std::find(security_surfaces.begin(), security_surfaces.end(), xwindow_),
-        security_surfaces.end());
+    security_surfaces.erase(base::ranges::find(security_surfaces, xwindow_),
+                            security_surfaces.end());
   }
 
   connection_->DestroyWindow({xwindow_});
@@ -1884,6 +1896,10 @@ bool X11Window::IsMinimized() const {
 }
 
 bool X11Window::IsMaximized() const {
+  // In X11, if a maximized window is minimized, it will have both the "hidden"
+  // and "maximized" states.
+  if (IsMinimized())
+    return false;
   return (HasWMSpecProperty(window_properties_,
                             x11::GetAtom("_NET_WM_STATE_MAXIMIZED_VERT")) &&
           HasWMSpecProperty(window_properties_,
@@ -2270,13 +2286,7 @@ void X11Window::OnWindowMapped() {
 
 void X11Window::OnConfigureEvent(const x11::ConfigureNotifyEvent& configure,
                                  bool send_event) {
-  DCHECK_EQ(xwindow_, configure.event);
-
-  // ConfigureNotifyEvent could be received for child windows. Ignore events for
-  // child windows.
-  if (xwindow_ != configure.window) {
-    return;
-  }
+  DCHECK_EQ(xwindow_, configure.window);
 
   if (pending_counter_value_) {
     DCHECK(!configure_counter_value_);
@@ -2405,7 +2415,7 @@ void X11Window::DispatchResize(bool origin_changed) {
     // _NET_WM_SYNC_REQUEST is disabled by the compositor.
     delayed_resize_task_.Reset(base::BindOnce(
         &X11Window::DelayedResize, base::Unretained(this), origin_changed));
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
         FROM_HERE, delayed_resize_task_.callback());
     return;
   }
