@@ -15,6 +15,7 @@
 #include "ui/accessibility/ax_enums.mojom.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/base/metadata/metadata_impl_macros.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/color/color_id.h"
 #include "ui/color/color_provider.h"
 #include "ui/compositor/layer.h"
@@ -27,6 +28,7 @@
 #include "ui/views/animation/ink_drop.h"
 #include "ui/views/background.h"
 #include "ui/views/controls/button/label_button_border.h"
+#include "ui/views/controls/highlight_path_generator.h"
 #include "ui/views/painter.h"
 #include "ui/views/style/platform_style.h"
 #include "ui/views/view_class_properties.h"
@@ -34,6 +36,11 @@
 
 namespace views {
 
+namespace {
+constexpr Button::ButtonState kEnabledStates[] = {
+    Button::STATE_NORMAL, Button::STATE_HOVERED, Button::STATE_PRESSED};
+}  // namespace
+
 LabelButton::LabelButton(PressedCallback callback,
                          const std::u16string& text,
                          int button_context)
@@ -59,20 +66,24 @@ LabelButton::LabelButton(PressedCallback callback,
 
 LabelButton::~LabelButton() {
   // TODO(pbos): Revisit explicit removal of InkDrop for classes that override
-  // Add/RemoveLayerBeneathView(). This is done so that the InkDrop doesn't
+  // Add/RemoveLayerFromRegions(). This is done so that the InkDrop doesn't
   // access the non-override versions in ~View.
   views::InkDrop::Remove(this);
 }
 
 gfx::ImageSkia LabelButton::GetImage(ButtonState for_state) const {
   for_state = ImageStateForState(for_state);
-  return button_state_image_models_[for_state].Rasterize(GetColorProvider());
+  return GetImageModel(for_state).Rasterize(GetColorProvider());
 }
 
 void LabelButton::SetImage(ButtonState for_state, const gfx::ImageSkia& image) {
   SetImageModel(for_state, ui::ImageModel::FromImageSkia(image));
 }
 
+const ui::ImageModel& LabelButton::GetImageModel(ButtonState for_state) const {
+  return button_state_image_models_[for_state];
+}
+
 void LabelButton::SetImageModel(ButtonState for_state,
                                 const ui::ImageModel& image_model) {
   if (button_state_image_models_[for_state] == image_model)
@@ -99,6 +110,10 @@ void LabelButton::SetText(const std::u16string& text) {
   SetTextInternal(text);
 }
 
+void LabelButton::SetLabelStyle(views::style::TextStyle text_style) {
+  label_->SetTextStyle(text_style);
+}
+
 void LabelButton::ShrinkDownThenClearText() {
   if (GetText().empty())
     return;
@@ -118,18 +133,50 @@ void LabelButton::SetTextColor(ButtonState for_state, SkColor color) {
   explicitly_set_colors_[for_state] = true;
 }
 
+void LabelButton::SetTextColorId(ButtonState for_state, ui::ColorId color_id) {
+  button_state_colors_[for_state] = color_id;
+  if (for_state == STATE_DISABLED) {
+    label_->SetDisabledColorId(color_id);
+  } else if (for_state == GetState()) {
+    label_->SetEnabledColorId(color_id);
+  }
+  explicitly_set_colors_[for_state] = true;
+}
+
+float LabelButton::GetFocusRingCornerRadius() const {
+  return focus_ring_corner_radius_;
+}
+
+void LabelButton::SetFocusRingCornerRadius(float radius) {
+  if (focus_ring_corner_radius_ == radius)
+    return;
+  focus_ring_corner_radius_ = radius;
+  InkDrop::Get(this)->SetSmallCornerRadius(focus_ring_corner_radius_);
+  InkDrop::Get(this)->SetLargeCornerRadius(focus_ring_corner_radius_);
+  views::InstallRoundRectHighlightPathGenerator(this, gfx::Insets(),
+                                                focus_ring_corner_radius_);
+  OnPropertyChanged(&focus_ring_corner_radius_, kPropertyEffectsPaint);
+}
+
 void LabelButton::SetEnabledTextColors(absl::optional<SkColor> color) {
-  ButtonState states[] = {STATE_NORMAL, STATE_HOVERED, STATE_PRESSED};
   if (color.has_value()) {
-    for (auto state : states)
+    for (auto state : kEnabledStates) {
       SetTextColor(state, color.value());
+    }
     return;
   }
-  for (auto state : states)
+  for (auto state : kEnabledStates) {
     explicitly_set_colors_[state] = false;
+  }
   ResetColorsFromNativeTheme();
 }
 
+void LabelButton::SetEnabledTextColorIds(ui::ColorId color_id) {
+  for (auto state : kEnabledStates) {
+    SetTextColorId(state, color_id);
+  }
+}
+
 SkColor LabelButton::GetCurrentTextColor() const {
   return label_->GetEnabledColor();
 }
@@ -381,9 +428,10 @@ void LabelButton::Layout() {
 }
 
 void LabelButton::GetAccessibleNodeData(ui::AXNodeData* node_data) {
-  if (GetIsDefault())
-    node_data->AddState(ax::mojom::State::kDefault);
   Button::GetAccessibleNodeData(node_data);
+  if (GetIsDefault()) {
+    node_data->AddState(ax::mojom::State::kDefault);
+  }
 }
 
 ui::NativeTheme::Part LabelButton::GetThemePart() const {
@@ -407,7 +455,7 @@ ui::NativeTheme::State LabelButton::GetThemeState(
     case STATE_DISABLED:
       return ui::NativeTheme::kDisabled;
     case STATE_COUNT:
-      NOTREACHED();
+      NOTREACHED_NORETURN();
   }
   return ui::NativeTheme::kNormal;
 }
@@ -433,27 +481,29 @@ void LabelButton::UpdateImage() {
     image_->SetImage(GetImage(GetVisualState()));
 }
 
-void LabelButton::AddLayerBeneathView(ui::Layer* new_layer) {
+void LabelButton::AddLayerToRegion(ui::Layer* new_layer,
+                                   views::LayerRegion region) {
   image()->SetPaintToLayer();
   image()->layer()->SetFillsBoundsOpaquely(false);
   ink_drop_container()->SetVisible(true);
-  ink_drop_container()->AddLayerBeneathView(new_layer);
+  ink_drop_container()->AddLayerToRegion(new_layer, region);
 }
 
-void LabelButton::RemoveLayerBeneathView(ui::Layer* old_layer) {
-  ink_drop_container()->RemoveLayerBeneathView(old_layer);
+void LabelButton::RemoveLayerFromRegions(ui::Layer* old_layer) {
+  ink_drop_container()->RemoveLayerFromRegions(old_layer);
   ink_drop_container()->SetVisible(false);
   image()->DestroyLayer();
 }
 
 void LabelButton::GetExtraParams(ui::NativeTheme::ExtraParams* params) const {
-  params->button.checked = false;
-  params->button.indeterminate = false;
-  params->button.is_default = GetIsDefault();
-  params->button.is_focused = HasFocus() && IsAccessibilityFocusable();
-  params->button.has_border = false;
-  params->button.classic_state = 0;
-  params->button.background_color = label_->GetBackgroundColor();
+  auto& button = absl::get<ui::NativeTheme::ButtonExtraParams>(*params);
+  button.checked = false;
+  button.indeterminate = false;
+  button.is_default = GetIsDefault();
+  button.is_focused = HasFocus() && IsAccessibilityFocusable();
+  button.has_border = false;
+  button.classic_state = 0;
+  button.background_color = label_->GetBackgroundColor();
 }
 
 PropertyEffects LabelButton::UpdateStyleToIndicateDefaultStatus() {
@@ -584,36 +634,36 @@ void LabelButton::VisualStateChanged() {
 }
 
 void LabelButton::ResetColorsFromNativeTheme() {
-  if (!GetWidget()) {
-    // If there is no widget, we can't actually get the real colors here.
-    // An OnThemeChanged() will fire once a widget is available.
-    return;
-  }
-
-  const ui::ColorProvider* color_provider = GetColorProvider();
   // Since this is a LabelButton, use the label colors.
-  SkColor colors[STATE_COUNT] = {
-      color_provider->GetColor(ui::kColorLabelForeground),
-      color_provider->GetColor(ui::kColorLabelForeground),
-      color_provider->GetColor(ui::kColorLabelForeground),
-      color_provider->GetColor(ui::kColorLabelForegroundDisabled),
-  };
+  ui::ColorId color_ids[STATE_COUNT] = {
+      ui::kColorLabelForeground, ui::kColorLabelForeground,
+      ui::kColorLabelForeground, ui::kColorLabelForegroundDisabled};
 
   label_->SetBackground(nullptr);
   label_->SetAutoColorReadabilityEnabled(false);
 
   for (size_t state = STATE_NORMAL; state < STATE_COUNT; ++state) {
     if (!explicitly_set_colors_[state]) {
-      SetTextColor(static_cast<ButtonState>(state), colors[state]);
+      SetTextColorId(static_cast<ButtonState>(state), color_ids[state]);
       explicitly_set_colors_[state] = false;
     }
   }
 }
 
 void LabelButton::ResetLabelEnabledColor() {
-  const SkColor color = button_state_colors_[GetState()];
-  if (GetState() != STATE_DISABLED && label_->GetEnabledColor() != color)
-    label_->SetEnabledColor(color);
+  if (GetState() == STATE_DISABLED) {
+    return;
+  }
+  const absl::variant<SkColor, ui::ColorId>& color =
+      button_state_colors_[GetState()];
+  if (absl::holds_alternative<SkColor>(color) &&
+      label_->GetEnabledColor() != absl::get<SkColor>(color)) {
+    label_->SetEnabledColor(absl::get<SkColor>(color));
+  } else if (absl::holds_alternative<ui::ColorId>(color)) {
+    // Omitting the check that the new color id differs from the existing color
+    // id, because the setter already does that check.
+    label_->SetEnabledColorId(absl::get<ui::ColorId>(color));
+  }
 }
 
 Button::ButtonState LabelButton::ImageStateForState(
@@ -634,6 +684,7 @@ ADD_PROPERTY_METADATA(gfx::Size, MaxSize)
 ADD_PROPERTY_METADATA(bool, IsDefault)
 ADD_PROPERTY_METADATA(int, ImageLabelSpacing)
 ADD_PROPERTY_METADATA(bool, ImageCentered)
+ADD_PROPERTY_METADATA(float, FocusRingCornerRadius)
 END_METADATA
 
 }  // namespace views
