@@ -243,7 +243,7 @@ class Target : public Item {
     return output_type_ == ACTION || output_type_ == ACTION_FOREACH ||
            output_type_ == COPY_FILES || output_type_ == CREATE_BUNDLE ||
            output_type_ == BUNDLE_DATA || output_type_ == GENERATED_FILE ||
-           (IsBinary() && has_swift_values() && swift_values().builds_module());
+           builds_swift_module();
   }
 
   // Returns the iterator range which can be used in range-based for loops
@@ -298,6 +298,10 @@ class Target : public Item {
     return allow_circular_includes_from_;
   }
 
+  // Pool option
+  const LabelPtrPair<Pool>& pool() const { return pool_; }
+  void set_pool(LabelPtrPair<Pool> pool) { pool_ = std::move(pool); }
+
   const InheritedLibraries& inherited_libraries() const {
     return inherited_libraries_;
   }
@@ -315,14 +319,22 @@ class Target : public Item {
   const SwiftValues& swift_values() const;
   bool has_swift_values() const { return swift_values_.get(); }
 
+  // Return true if this targets builds a SwiftModule
+  bool builds_swift_module() const {
+    return IsBinary() && has_swift_values() &&
+           source_types_used().SwiftSourceUsed();
+  }
+
   RustValues& rust_values();
   const RustValues& rust_values() const;
   bool has_rust_values() const { return rust_values_.get(); }
 
   // Transitive closure of libraries that are depended on by this target
-  InheritedLibraries& rust_transitive_libs() { return rust_transitive_libs_; }
-  const InheritedLibraries& rust_transitive_libs() const {
-    return rust_transitive_libs_;
+  const InheritedLibraries& rust_transitive_inherited_libs() const {
+    return rust_transitive_inherited_libs_;
+  }
+  const InheritedLibraries& rust_transitive_inheritable_libs() const {
+    return rust_transitive_inheritable_libs_;
   }
 
   const UniqueVector<SourceDir>& all_lib_dirs() const { return all_lib_dirs_; }
@@ -491,6 +503,8 @@ class Target : public Item {
 
   std::set<Label> allow_circular_includes_from_;
 
+  LabelPtrPair<Pool> pool_;
+
   // Static libraries, shared libraries, and source sets from transitive deps
   // that need to be linked.
   InheritedLibraries inherited_libraries_;
@@ -524,8 +538,18 @@ class Target : public Item {
   // Used for Rust targets.
   std::unique_ptr<RustValues> rust_values_;
 
-  // Used by all targets, only useful to generate Rust targets though.
-  InheritedLibraries rust_transitive_libs_;
+  // Used by all targets, only useful to generate Rust targets though. These
+  // present 2 different views of the public flags:
+  //
+  // Lists all transitive libraries, and for each one the public bit says if
+  // there is a public chain such that this target can make direct use of the
+  // lib. For each library marked public: "I have access to these targets."
+  InheritedLibraries rust_transitive_inherited_libs_;
+  // Lists all transitive libraries, and for each one the public bit says if a
+  // target depending on this target would inherit the libraries as public too.
+  // For each library marked public: "If you depend on me, you get access to
+  // these targets."
+  InheritedLibraries rust_transitive_inheritable_libs_;
 
   // User for Swift targets.
   std::unique_ptr<SwiftValues> swift_values_;
