@@ -32,6 +32,7 @@
 #include <QtCore/qiodevice.h>
 #include <QtCore/qmimedatabase.h>
 #include <QtCore/qmimedata.h>
+#include <QtCore/qpointer.h>
 #include <QtCore/qurl.h>
 
 namespace QtWebEngineCore {
@@ -150,9 +151,11 @@ private:
             m_firstBytePosition = m_byteRange.first_byte_position();
 
 //        m_taskRunner->PostTask(FROM_HERE,
-        content::GetUIThreadTaskRunner({})->PostTask(FROM_HERE,
-                       base::BindOnce(&URLRequestCustomJobProxy::initialize, m_proxy,
-                                      m_request.url, m_request.method, m_request.request_initiator, std::move(headers)));
+        content::GetUIThreadTaskRunner({})->PostTask(
+                FROM_HERE,
+                base::BindOnce(&URLRequestCustomJobProxy::initialize, m_proxy, m_request.url,
+                               m_request.method, m_request.request_initiator, std::move(headers),
+                               m_request.request_body));
     }
 
     void CompleteWithFailure(network::CorsErrorStatus cors_error)
@@ -262,11 +265,17 @@ private:
                 headers += "Access-Control-Allow-Credentials: true\n";
             }
         }
+        for (auto it = m_additionalResponseHeaders.cbegin();
+             it != m_additionalResponseHeaders.cend(); ++it) {
+            headers += it.key().toLower().toStdString() + ": " + it.value().toLower().toStdString()
+                    + "\n";
+        }
         m_head->headers = base::MakeRefCounted<net::HttpResponseHeaders>(net::HttpUtil::AssembleRawHeaders(headers));
         m_head->encoded_data_length = m_head->headers->raw_headers().length();
 
         if (!m_redirect.is_empty()) {
-            m_head->content_length = m_head->encoded_body_length = -1;
+            m_head->content_length = {};
+            m_head->encoded_body_length = {};
             net::RedirectInfo::FirstPartyURLPolicy first_party_url_policy =
                     m_request.update_first_party_url_on_redirect ? net::RedirectInfo::FirstPartyURLPolicy::UPDATE_URL_ON_REDIRECT
                                                                  : net::RedirectInfo::FirstPartyURLPolicy::NEVER_CHANGE_URL;
@@ -334,7 +343,8 @@ private:
         }
         m_head->headers = base::MakeRefCounted<net::HttpResponseHeaders>(net::HttpUtil::AssembleRawHeaders(headers));
         m_head->encoded_data_length = m_head->headers->raw_headers().length();
-        m_head->content_length = m_head->encoded_body_length = -1;
+        m_head->content_length = {};
+        m_head->encoded_body_length = {};
         m_client->OnReceiveResponse(std::move(m_head), mojo::ScopedDataPipeConsumerHandle(), absl::nullopt);
         CompleteWithFailure(net::Error(error));
     }
