@@ -4,9 +4,10 @@
 #include "web_engine_context.h"
 
 #include <math.h>
+#include <QtGui/private/qrhi_p.h>
 
 #include "base/base_switches.h"
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/command_line.h"
 #include "base/files/file_path.h"
 #include "base/power_monitor/power_monitor.h"
@@ -31,14 +32,17 @@
 #endif
 #include "components/discardable_memory/service/discardable_shared_memory_manager.h"
 #include "components/download/public/common/download_task_runner.h"
-#include "components/power_scheduler/power_mode_arbiter.h"
 #include "components/viz/common/features.h"
 #include "components/web_cache/browser/web_cache_manager.h"
 #include "content/app/mojo_ipc_support.h"
 #include "content/browser/devtools/devtools_http_handler.h"
+#include "content/browser/gpu/gpu_main_thread_factory.h"
+#include "content/browser/renderer_host/render_process_host_impl.h"
 #include "content/browser/scheduler/browser_task_executor.h"
 #include "content/browser/startup_data_impl.h"
 #include "content/browser/startup_helper.h"
+#include "content/browser/utility_process_host.h"
+#include "content/gpu/in_process_gpu_thread.h"
 #include "content/public/app/content_main.h"
 #include "content/public/app/content_main_runner.h"
 #include "content/public/browser/browser_main_runner.h"
@@ -53,7 +57,8 @@
 #include "content/public/common/content_paths.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/main_function_params.h"
-#include "content/public/common/network_service_util.h"
+#include "content/renderer/in_process_renderer_thread.h"
+#include "content/utility/in_process_utility_thread.h"
 #include "gpu/command_buffer/service/gpu_switches.h"
 #include "gpu/config/gpu_finch_features.h"
 #include "media/audio/audio_manager.h"
@@ -70,6 +75,7 @@
 #include "ui/base/ui_base_features.h"
 #include "ui/events/event_switches.h"
 #include "ui/native_theme/native_theme_features.h"
+#include "ui/gl/gl_utils.h"
 #include "ui/gl/gl_switches.h"
 #if defined(Q_OS_WIN)
 #include "sandbox/win/src/sandbox_types.h"
@@ -77,7 +83,7 @@
 #endif // Q_OS_WIN
 
 #if defined(Q_OS_MACOS)
-#include "base/mac/foundation_util.h"
+#include "base/apple/foundation_util.h"
 #endif
 
 #if QT_CONFIG(accessibility)
@@ -104,6 +110,7 @@
 #include <qopenglcontext_platform.h>
 #endif
 #include <QQuickWindow>
+#include <QRegularExpression>
 #include <QStringList>
 #include <QSurfaceFormat>
 #include <QNetworkProxy>
@@ -193,11 +200,13 @@ static const char *getGLType(bool enableGLSoftwareRendering, bool disableGpu)
         return glType;
 
 #if defined(Q_OS_MACOS)
-    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Metal)
-        return gl::kGLImplementationANGLEName;
-#elif defined(Q_OS_WIN)
-    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11)
+    return gl::kGLImplementationANGLEName;
+#else
+#if defined(Q_OS_WIN)
+    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11
+        || QQuickWindow::graphicsApi() == QSGRendererInterface::Vulkan) {
         return gl::kGLImplementationANGLEName;
+    }
 #endif
 
     if (!qt_gl_global_share_context() || !qt_gl_global_share_context()->isValid()) {
@@ -211,15 +220,10 @@ static const char *getGLType(bool enableGLSoftwareRendering, bool disableGpu)
     switch (sharedFormat.renderableType()) {
     case QSurfaceFormat::OpenGL:
         if (sharedFormat.profile() == QSurfaceFormat::CoreProfile) {
-#if defined(Q_OS_MACOS)
-            // Chromium supports core profile only on mac
-            glType = gl::kGLImplementationCoreProfileName;
-#else
             glType = gl::kGLImplementationDesktopName;
             qWarning("An OpenGL Core Profile was requested, but it is not supported "
                      "on the current platform. Falling back to a non-Core profile. "
                      "Note that this might cause rendering issues.");
-#endif
         } else {
             glType = gl::kGLImplementationDesktopName;
         }
@@ -235,6 +239,7 @@ static const char *getGLType(bool enableGLSoftwareRendering, bool disableGpu)
                  "https://bugreports.qt.io");
     }
     return glType;
+#endif // defined(Q_OS_MACOS)
 }
 #else
 static const char *getGLType(bool /*enableGLSoftwareRendering*/, bool disableGpu)
@@ -242,16 +247,42 @@ static const char *getGLType(bool /*enableGLSoftwareRendering*/, bool disableGpu
     if (disableGpu)
         return gl::kGLImplementationDisabledName;
 #if defined(Q_OS_MACOS)
-    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Metal)
-        return gl::kGLImplementationANGLEName;
+    return gl::kGLImplementationANGLEName;
 #elif defined(Q_OS_WIN)
-    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11)
+    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11
+        || QQuickWindow::graphicsApi() == QSGRendererInterface::Vulkan) {
         return gl::kGLImplementationANGLEName;
+    }
 #endif
     return gl::kGLImplementationDisabledName;
 }
 #endif // QT_CONFIG(opengl)
 
+#if defined(Q_OS_WIN)
+static QString getAdapterLuid() {
+    static const bool preferSoftwareDevice = qEnvironmentVariableIntValue("QSG_RHI_PREFER_SOFTWARE_RENDERER");
+    QRhiD3D11InitParams rhiParams;
+    QRhi::Flags flags;
+    if (preferSoftwareDevice) {
+        flags |= QRhi::PreferSoftwareRenderer;
+    }
+    QScopedPointer<QRhi> rhi(QRhi::create(QRhi::D3D11,&rhiParams,flags,nullptr));
+    // mimic what QSGRhiSupport and QBackingStoreRhi does
+    if (!rhi && !preferSoftwareDevice) {
+        flags |= QRhi::PreferSoftwareRenderer;
+        rhi.reset(QRhi::create(QRhi::D3D11, &rhiParams, flags));
+    }
+    if (rhi) {
+        const QRhiD3D11NativeHandles *handles =
+                static_cast<const QRhiD3D11NativeHandles *>(rhi->nativeHandles());
+        Q_ASSERT(handles);
+        return QString("%1,%2").arg(handles->adapterLuidHigh).arg(handles->adapterLuidLow);
+    } else {
+        return QString();
+    }
+}
+#endif
+
 #if QT_CONFIG(webengine_pepper_plugins)
 void dummyGetPluginCallback(const std::vector<content::WebPluginInfo>&)
 {
@@ -281,13 +312,11 @@ static void logContext(const char *glType, base::CommandLine *cmd)
                 "Using Supported QSG Backend: %s\n"
                 "Using Software Dynamic GL: %s\n"
                 "Using Shared GL: %s\n"
-                "Using Multithreaded OpenGL: %s\n\n"
                 "Init Parameters:\n %s",
                 glType, type, profile, sharedFormat.majorVersion(), sharedFormat.minorVersion(),
                 qUtf8Printable(QSGRhiSupport::instance()->rhiBackendName()),
                 usingSupportedSGBackend() ? "yes" : "no", usingSoftwareDynamicGL() ? "yes" : "no",
                 qt_gl_global_share_context() ? "yes" : "no",
-                !WebEngineContext::isGpuServiceOnUIThread() ? "yes" : "no",
                 qPrintable(params.join(" ")));
 #else
         qCDebug(webEngineContextLog,
@@ -443,7 +472,6 @@ void WebEngineContext::destroy()
     // on IO thread (triggered by ~BrowserMainRunner). But by that time the UI
     // task runner is not working anymore so we need to do this earlier.
     cleanupVizProcess();
-    destroyGpuProcess();
     // Flush the UI message loop before quitting.
     flushMessages();
 
@@ -573,18 +601,6 @@ ProxyAuthentication WebEngineContext::qProxyNetworkAuthentication(QString host,
 
 const static char kChromiumFlagsEnv[] = "QTWEBENGINE_CHROMIUM_FLAGS";
 const static char kDisableSandboxEnv[] = "QTWEBENGINE_DISABLE_SANDBOX";
-const static char kDisableInProcGpuThread[] = "QTWEBENGINE_DISABLE_GPU_THREAD";
-
-// static
-bool WebEngineContext::isGpuServiceOnUIThread()
-{
-    static bool threadedGpu =
-#if QT_CONFIG(opengl) && !defined(Q_OS_MACOS)
-            QOpenGLContext::supportsThreadedOpenGL() &&
-#endif
-            !qEnvironmentVariableIsSet(kDisableInProcGpuThread);
-    return !threadedGpu;
-}
 
 static void initializeFeatureList(base::CommandLine *commandLine, std::vector<std::string> enableFeatures, std::vector<std::string> disableFeatures)
 {
@@ -636,7 +652,7 @@ WebEngineContext::WebEngineContext()
 #if defined(Q_OS_MACOS)
     // The bundled handling is currently both completely broken in Chromium,
     // and unnecessary for us.
-    base::mac::SetOverrideAmIBundled(false);
+    base::apple::SetOverrideAmIBundled(false);
 #endif
 
     base::ThreadPoolInstance::Create("Browser");
@@ -683,8 +699,6 @@ WebEngineContext::WebEngineContext()
         qInfo() << "Sandboxing disabled by user.";
     }
 
-    parsedCommandLine->AppendSwitch(switches::kEnableThreadedCompositing);
-
     // Do not advertise a feature we have removed at compile time
     parsedCommandLine->AppendSwitch(switches::kDisableSpeechAPI);
 
@@ -710,7 +724,6 @@ WebEngineContext::WebEngineContext()
     disableFeatures.push_back(features::kWebOTP.name);
     disableFeatures.push_back(features::kWebPayments.name);
     disableFeatures.push_back(features::kWebUsb.name);
-    disableFeatures.push_back(media::kPictureInPicture.name);
 
     if (useEmbeddedSwitches) {
         // embedded switches are based on the switches for Android, see content/browser/android/content_startup_flags.cc
@@ -719,11 +732,40 @@ WebEngineContext::WebEngineContext()
         parsedCommandLine->AppendSwitch(cc::switches::kDisableCompositedAntialiasing);
     }
 
-#if QT_CONFIG(webengine_vulkan)
+#if QT_CONFIG(webengine_vulkan) && defined(USE_OZONE)
     if (QQuickWindow::graphicsApi() == QSGRendererInterface::Vulkan) {
         enableFeatures.push_back(features::kVulkan.name);
         parsedCommandLine->AppendSwitchASCII(switches::kUseVulkan,
                                              switches::kVulkanImplementationNameNative);
+        const char deviceExtensionsVar[] = "QT_VULKAN_DEVICE_EXTENSIONS";
+        QByteArrayList requiredDeviceExtensions = { "VK_EXT_external_memory_dma_buf",
+                                                    "VK_EXT_image_drm_format_modifier" };
+        if (qEnvironmentVariableIsSet(deviceExtensionsVar)) {
+            QByteArrayList envExtList = qgetenv(deviceExtensionsVar).split(';');
+            int found = 0;
+            for (const QByteArray &ext : requiredDeviceExtensions) {
+                if (envExtList.contains(ext))
+                    found++;
+            }
+            if (found != requiredDeviceExtensions.size()) {
+                qWarning().nospace()
+                        << "Vulkan rendering may fail because " << deviceExtensionsVar
+                        << " environment variable is already set but it doesn't contain"
+                        << " some of the required Vulkan device extensions:\n"
+                        << qPrintable(requiredDeviceExtensions.join('\n'));
+            }
+        } else {
+            qputenv(deviceExtensionsVar, requiredDeviceExtensions.join(';'));
+        }
+    }
+#endif // QT_CONFIG(webengine_vulkan) && defined(USE_OZONE)
+
+#if defined(Q_OS_WIN)
+    if (QQuickWindow::graphicsApi() == QSGRendererInterface::Direct3D11
+        || QQuickWindow::graphicsApi() == QSGRendererInterface::Vulkan) {
+        const QString luid = getAdapterLuid();
+        if (!luid.isEmpty())
+            parsedCommandLine->AppendSwitchASCII(switches::kUseAdapterLuid, luid.toStdString());
     }
 #endif
 
@@ -758,8 +800,11 @@ WebEngineContext::WebEngineContext()
             // "Could not share GL contexts" warnings, because it's not possible to share between Core and
             // legacy profiles. See GLContextWGL::Initialize().
             if (sharedFormat.renderableType() == QSurfaceFormat::OpenGL
-                && sharedFormat.profile() != QSurfaceFormat::CoreProfile)
-                parsedCommandLine->AppendSwitch(switches::kDisableES3GLContext);
+                && sharedFormat.profile() != QSurfaceFormat::CoreProfile) {
+                gl::GlWorkarounds workarounds = gl::GetGlWorkarounds();
+                workarounds.disable_es3gl_context = true;
+                gl::SetGlWorkarounds(workarounds);
+            }
 #endif
         }
 #endif //QT_CONFIG(opengl)
@@ -798,7 +843,6 @@ WebEngineContext::WebEngineContext()
     base::PowerMonitor::Initialize(std::make_unique<base::PowerMonitorDeviceSource>());
     content::ProcessVisibilityTracker::GetInstance();
     m_discardableSharedMemoryManager = std::make_unique<discardable_memory::DiscardableSharedMemoryManager>();
-    power_scheduler::PowerModeArbiter::GetInstance()->OnThreadPoolAvailable();
 
     m_mojoIpcSupport = std::make_unique<content::MojoIpcSupport>(content::BrowserTaskExecutor::CreateIOThread());
     download::SetIOTaskRunner(m_mojoIpcSupport->io_thread()->task_runner());
@@ -877,11 +921,14 @@ base::CommandLine *WebEngineContext::initCommandLine(bool &useEmbeddedSwitches,
     }
 
     base::CommandLine *parsedCommandLine = base::CommandLine::ForCurrentProcess();
+    int index = appArgs.indexOf(QRegularExpression(QLatin1String("--webEngineArgs"),
+                                                   QRegularExpression::CaseInsensitiveOption));
     if (qEnvironmentVariableIsSet(kChromiumFlagsEnv)) {
         appArgs = appArgs.mid(0, 1); // Take application name and drop the rest
         appArgs.append(parseEnvCommandLine(qEnvironmentVariable(kChromiumFlagsEnv)));
+        if (index > -1)
+            qWarning("Note 'webEngineArgs' are overridden by QTWEBENGINE_CHROMIUM_FLAGS");
     } else {
-        int index = appArgs.indexOf(QLatin1String("--webEngineArgs"));
         if (index > -1) {
             appArgs.erase(appArgs.begin() + 1, appArgs.begin() + index + 1);
         } else {
@@ -898,6 +945,22 @@ base::CommandLine *WebEngineContext::initCommandLine(bool &useEmbeddedSwitches,
     appArgs.removeAll(QStringLiteral("--disable-embedded-switches"));
     appArgs.removeAll(QStringLiteral("--enable-embedded-switches"));
 
+    bool isRemoteDebugPort =
+            (-1
+             != appArgs.indexOf(QRegularExpression(QStringLiteral("--remote-debugging-port=.*"),
+                                                   QRegularExpression::CaseInsensitiveOption)))
+            || !qEnvironmentVariable("QTWEBENGINE_REMOTE_DEBUGGING").isEmpty();
+    bool isRemoteAllowOrigins =
+            (-1
+             != appArgs.indexOf(QRegularExpression(QStringLiteral("--remote-allow-origins=.*"),
+                                                   QRegularExpression::CaseInsensitiveOption)));
+
+    if (isRemoteDebugPort && !isRemoteAllowOrigins) {
+        appArgs.append(QStringLiteral("--remote-allow-origins=*"));
+        qWarning("Added {--remote-allow-origins=*} to command-line arguments "
+                 "to avoid web socket connection errors during remote debugging.");
+    }
+
     base::CommandLine::StringVector argv;
     argv.resize(appArgs.size());
 #if defined(Q_OS_WIN)
@@ -917,6 +980,13 @@ bool WebEngineContext::closingDown()
     return m_closingDown;
 }
 
+void WebEngineContext::registerMainThreadFactories()
+{
+    content::UtilityProcessHost::RegisterUtilityMainThreadFactory(content::CreateInProcessUtilityThread);
+    content::RenderProcessHostImpl::RegisterRendererMainThreadFactory(content::CreateInProcessRendererThread);
+    content::RegisterGpuMainThreadFactory(content::CreateInProcessGpuThread);
+}
+
 } // namespace
 
 QT_BEGIN_NAMESPACE
@@ -937,7 +1007,7 @@ const char *qWebEngineChromiumVersion() noexcept
 
 const char *qWebEngineChromiumSecurityPatchVersion() noexcept
 {
-    return "114.0.5735.133"; // FIXME: Remember to update
+    return "122.0.6261.128"; // FIXME: Remember to update
 }
 
 QT_END_NAMESPACE
